#include <iprs.h>
#include "crg.h"

/* This procedure is for calculating the mean of the points in each cluster.
 * The idea is for using these means for the fuzzy clustering stuff.
 */

extern int nunary_features, nbinary_features;

void bren_calculate_means(CLUSTER *root)
{
	float *mean;
	int num_features, num_els=0, i;

	if (root->type!='0')
	{
		if (root->type=='U')
			num_features = nunary_features;
		else num_features = nbinary_features;

		root->mean = (float*)iprs_calloc(num_features, sizeof(float));
		if (root->type=='U')
		{
			foreach(UNARY,pt,root->elements)
			{
				for (i=0; i<num_features; i++)
				{
					root->mean[i] += pt->value[i];
				}
				num_els++;
			}
			endforeach;
		}
		else
		{
			foreach (BINARY,pt,root->elements)
			{
				for (i=0; i<num_features; i++)
				{
					root->mean[i] += pt->value[i];
				}
				num_els++;
			}
			endforeach;
		}
		for (i=0; i<num_features; i++)
		{
			root->mean[i] /= (float)num_els;
		}
	}
	foreach (CLUSTER, cl, root->children)
		bren_calculate_means(cl);
	endforeach;
}


/*
 * This function calculates the distance between the given point and all
 * the clusters in the cluster list.
 */

float *bren_calc_distances(float *pt, LIST clusters, int nfeatures)
{
	float *distances;
	int cluster_index=0;
	CLUSTER *tmpc;

	distances = (float*)iprs_calloc(list_length(clusters),nfeatures);
	foreach(CLUSTER, c, clusters)
	{
		tmpc = c;
		print_cluster(c);
		distances[cluster_index++]=multid_feuclid_dist(pt,tmpc->mean,nfeatures);
	}
	endforeach;
	return(distances);
}

int find_nearest_point(float *pt, int nfeatures, float **pts, int npts)
{
	int index, i;
	float dist, max_dist=FLT_MAX;

	for (i=0; i<npts; i++)
	{
		dist = multid_feuclid_dist(pt, pts[i], nfeatures);
		if (dist<max_dist)
		{
			max_dist=dist;
			index=i;
		}
	}
	return(index);
}

float multid_feuclid_dist(float *pt1, float *pt2, int numdims)
{
        float ans=0.0;
        int i;

        for (i=0; i<numdims; i++)
                ans += (pt1[i]-pt2[i])*(pt1[i]-pt2[i]);

        ans = (float)sqrt((double)ans);
        return(ans);
}

/*
 * Euclidian distance.
 */

float euclid_dist(int x1, int y1, int x2, int y2)
{
	float ans;

	ans = (float)sqrt((double)(x1-x2)*(double)(x1-x2)+
					   (double)(y1-y2)*(double)(y1-y2));
	return(ans);
}

float feuclid_dist(float x1, float y1, float x2, float y2)
{
	float ans;

	ans = (float)sqrt((double)(x1-x2)*(double)(x1-x2)+
					   (double)(y1-y2)*(double)(y1-y2));
	return(ans);
}


/* calculates the centroid of the edge */
void bren_centroid_edge(IPRS_EDGE *contour, float *x_cent, float *y_cent)
{
	int i;

	iprs_debug_enter("bren_centroid_edge");

	*x_cent = *y_cent = 0;
	for (i=0; i<contour->numpixels; i++)
	{
		(*x_cent) += contour->x[i];
		(*y_cent) += contour->y[i];
	}

	(*x_cent) = (*x_cent) / contour->numpixels;
	(*y_cent) = (*y_cent) / contour->numpixels;

	iprs_debug_exit();
}
