#include <iprs.h>
/* #include <bren_fe.h> */
#include <bren_images.h>
#include "sure.h"
#include "sureextern.h"
#include <stdio.h>
#include <errno.h>

typedef struct _snake_path
{
	SNAKE *snake;
	LIST model_paths;
} SNAKE_PATH;

typedef struct _view_num
{
	int view, num;
	float fuzzy;
} VIEW_NUM;

void bren_get_modelview(int view_num, int part_num, char *view_file,
						char *reg_file, char *part_file,
						IPRS_INT_IMAGE **view_image,
						IPRS_REGION_IMAGE **region_image,
						int *reg_no)
{
	FILE *view_pipe, *reg_pipe, *part_pipe;
	char view_name[400], reg_name[400], part_name[400], dir_name[400];
	int num;
	IPRS_IMAGE *in_image, *int_image;
	char *search, *previous_search;
	
	if (!(view_pipe = fopen(view_file, "r")))
	{
		fprintf(stderr, "Couldn't open view file %s\n", view_file);
		exit(-1);
	}
	num=0;
	while ((fscanf(view_pipe, "%s", view_name)!=EOF)&&(num<view_num))
		num++;
	fclose(view_pipe);
	in_image = iprs_load_image(view_name);
	int_image = iprs_imagetoint(in_image);
	*view_image = (IPRS_INT_IMAGE *)int_image->image[0];
	iprs_free_image(in_image);

	if (!(reg_pipe = fopen(reg_file, "r")))
	{
		fprintf(stderr, "Couldn't open region file %s\n", reg_file);
		exit(-1);
	}
	num=0;
	while ((fscanf(reg_pipe, "%s", reg_name)!=EOF)&&(num<view_num))
		num++;
	fclose(reg_pipe);
	*region_image = iprs_load_regionimage(reg_name);

	if (!(part_pipe = fopen(part_file, "r")))
	{
		fprintf(stderr, "Couldn't open part file %s\n", part_file);
		exit(-1);
	}
	num = *reg_no = 0;
	strcpy(dir_name, view_name);
	/* strip off file extension */
	previous_search = strtok(dir_name, ".");
	previous_search = strtok(dir_name, "/");
	/* strip off pathname prefix */
	while ((search = strtok(NULL, "/"))!=NULL)
		previous_search = search;
	
	while ((fscanf(part_pipe, "%s", part_name))&&(num<part_num))
	{
		if (strstr(part_name, previous_search))
			(*reg_no)++;
		num++;
	}
	fclose(part_pipe);
}

/*
 * This function performs the hypothesis verification by correlating the
 * scene with the model views after determing which model view is the 
 * most likely to correspond.
 */

void bren_count_list_parts(LIST part_nums, int part_name, int view_name,
						   int class_name)
{
	PART_NUM *pn;
	int found=FALSE;

	foreach (PART_NUM, pn, part_nums)
	{
		if (pn->view==view_name)
		{
			pn->num++;
			found=TRUE;
			break;
		}
	} endforeach;
	if (!found)
	{
		pn = (PART_NUM*)iprs_calloc(1, sizeof(PART_NUM));
		pn->part = part_name;
		pn->view = view_name;
		pn->class = class_name;
		pn->num=1;
		list_append(pn, part_nums);
	}
}

int part_num_compare(PART_NUM **pn1, PART_NUM **pn2)
{
	if ((*pn1)->num<(*pn2)->num) return(-1);
	else return(1);
}

void region_moments(IPRS_REGION *region, IPRS_INT_IMAGE *intensity,
					float *mean, float *variance)
{
	float *data, *moments;
	int i, width;

	width = iprs_getwidth_intimage(intensity);
	data = (float*)iprs_calloc(region->numpixels, sizeof(float));
	for (i=0; i<region->numpixels; i++)
		data[i] = (float)intensity->image[region->x[i]+region->y[i]*width];

	moments = bren_extract_moments(region->numpixels, data, 2);
	*mean = moments[0];
	*variance = moments[1];
	iprs_free(moments);
	iprs_free(data);
}

/*
 * The correlation coefficient is a normalised correlation function.
 * Ref: Digital Image Processing, Gonzalez and Woods, Addison-Wesley, 1992.
 */
float bren_correlation_coefficient(IPRS_INT_IMAGE *scene_intensity,
								   IPRS_INT_IMAGE *model_intensity,
								   IPRS_REGION *model_region,
								   int delta_x, int delta_y,
								   float model_mean_intensity)
{
	IPRS_REGION *scene_region;
	int model_width, model_height, scene_width, scene_height;
	int i;
	float sum_diff=0.0, sum_squares_scene=0.0, sum_squares_model=0.0;
	float scene_mean_intensity, var;

	model_width = iprs_getwidth_intimage(model_intensity);
	model_height = iprs_getheight_intimage(model_intensity);
	scene_width = iprs_getwidth_intimage(scene_intensity);
	scene_height = iprs_getheight_intimage(scene_intensity);

	scene_region = bren_ccopy_region(model_region, 0);
	for (i=0; i<scene_region->numpixels; i++)
	{
		scene_region->x[i] += delta_x;
		scene_region->y[i] += delta_y;
	}
	region_moments(scene_region, scene_intensity, &scene_mean_intensity, &var);

	for (i=0; i<model_region->numpixels; i++)
	{
		float diff_model=0.0, diff_scene=0.0;

		if ((bren_in_interval(model_region->x[i], 0, model_width-1))&&
			(bren_in_interval(model_region->y[i], 0, model_height-1)))
		{
			diff_model = 
				model_intensity->image[
					model_region->x[i]+model_region->y[i]*model_width] -
				model_mean_intensity;
		}
		if ((bren_in_interval(scene_region->x[i], 0, scene_width-1))&&
			(bren_in_interval(scene_region->y[i], 0, scene_height-1)))
		{
			diff_scene = 
				scene_intensity->image[
				    scene_region->x[i]+scene_region->y[i]*scene_width] -
				scene_mean_intensity;
		}

/* 		if (diff_model*diff_scene>0) */
			sum_diff += diff_model*diff_scene;
		sum_squares_scene += bren_sqr(diff_scene);
		sum_squares_model += bren_sqr(diff_model);
	}
	iprs_free_region(scene_region);
	return(sum_diff/sqrt(sum_squares_model*sum_squares_scene));
}

float bren_sum_squared_error(IPRS_INT_IMAGE *scene_intensity,
							 IPRS_REGION_IMAGE *scene_regions, int index,
							 IPRS_INT_IMAGE *model_intensity,
							 IPRS_REGION *model_region,
							 int delta_x, int delta_y,
							 float model_mean_intensity,
							 int *region_overlap,
							 IPRS_INT_IMAGE *scene_region_image)
{
	IPRS_REGION *match_region;
	int model_width, model_height, scene_width, scene_height;
	int i;
	float sum_diff=0.0, scene_diff=0.0, model_diff=0.0;
	float scene_mean_intensity, var;
	int diff;
	IPRS_INT_IMAGE *tmp, *shifted_model_intensity;
	IPRS_REGION_IMAGE *regions;
	IPRS_REGION *scene_region=scene_regions->region[index];

	model_width = iprs_getwidth_intimage(model_intensity);
	model_height = iprs_getheight_intimage(model_intensity);
	scene_width = iprs_getwidth_intimage(scene_intensity);
	scene_height = iprs_getheight_intimage(scene_intensity);

	tmp = iprs_create_2dintimage(scene_width, scene_height);
	shifted_model_intensity = 
		iprs_create_2dintimage(model_width, model_height);
	match_region = bren_ccopy_region(model_region, 0);
	for (i=0; i<match_region->numpixels; i++)
	{
		int dx=model_region->x[i]+delta_x, dy=model_region->y[i]+delta_y;
		match_region->x[i] += delta_x;
		match_region->y[i] += delta_y;
		if ((bren_in_interval(dx, 0, model_width-1))&&
			(bren_in_interval(dy, 0, model_height-1)))
		{
			shifted_model_intensity->image[dx+dy*model_width] =
				model_intensity->image[model_region->x[i]+
									   model_region->y[i]*model_width];
			if (scene_region_image->image[dx+dy*scene_width])
				region_overlap
					[scene_region_image->image[dx+dy*scene_width]-1]++;
		}
	}
	for (i=0; i<scene_regions->numregions; i++)
	{
		if ((float)region_overlap[i]/scene_regions->region[i]->numpixels>0.7)
			region_overlap[i]=1;
		else region_overlap[i]=0;
	}
		
/* { */
/* 	IPRS_EDGE *ed; */
/* 	ed = bren_region_bdry(scene_region); */
/* 	bren_highlight_region(match_region, tmp, 128); */
/* 	bren_highlight_edge(ed, tmp, 255); */
/* 	make_intgif(tmp, "tmp2.gif"); */
/* 	iprs_set_intimage(tmp, 0); */
/* 	iprs_free_edge(ed); */
/* } */
	bren_highlight_region(match_region, tmp, 1);
	bren_highlight_region(scene_region, tmp, 1);
	iprs_free_region(match_region);
	regions = iprs_extract_intregions(tmp);
	match_region = regions->region[0];

	for (i=0; i<match_region->numpixels; i++)
	{
		float diff_model=0.0, diff_scene=0.0;

		if ((bren_in_interval(match_region->x[i], 0, model_width-1))&&
			(bren_in_interval(match_region->y[i], 0, model_height-1)))
		{
			int tmp;
			diff = shifted_model_intensity->image[
					match_region->x[i]+match_region->y[i]*model_width] -
				scene_intensity->image[
				    match_region->x[i]+match_region->y[i]*scene_width];
			sum_diff += (float)abs(diff); /* bren_sqr(diff); */
			model_diff += (float)abs(shifted_model_intensity->image[
					match_region->x[i]+match_region->y[i]*model_width]);
			scene_diff += (float)abs(scene_intensity->image[
				    match_region->x[i]+match_region->y[i]*scene_width]);
		}
	}
	iprs_free_regionimage(regions);
	iprs_free_intimage(tmp);
	iprs_free_intimage(shifted_model_intensity);
	return(1-sum_diff/(model_diff+scene_diff));
}
												
#define NUM_REG_CORR 3
												
float bren_correlate_objects(IPRS_INT_IMAGE *scene_intensity,
							 IPRS_REGION_IMAGE *scene_regions, 
							 IPRS_INT_IMAGE *scene_region_image,
							 int index, IPRS_INT_IMAGE *model_intensity, 
							 IPRS_REGION_IMAGE *model_regs, int reg_no,
							 float *max_region_overlap)
{
	BREN_VECTOR *scene_cent, *model_cent;
	BREN_VECTOR *model_axis1, *model_axis2;
	BREN_VECTOR *scene_axis1, *scene_axis2;
	int i, j;
	float model_mean, model_var;
	float corr_coeff, max_corr_coeff=-FLT_MAX;
	IPRS_REGION *model_region;
	double var1, var2;
	LINE_SEGMENT axis1, axis2;
	IPRS_INT_IMAGE *rotated_model_intensity;
	int model_width, model_height;
	int *region_overlap;
	float best_ratio[NUM_REG_CORR];
	int best_reg[NUM_REG_CORR];

	for (i=0; i<NUM_REG_CORR; i++) best_ratio[i]=FLT_MAX;
	region_overlap=(int*)iprs_calloc(scene_regions->numregions,sizeof(int));
	model_width = iprs_getwidth_intimage(model_intensity);
	model_height = iprs_getheight_intimage(model_intensity);
	/* calculate the scene part centroid */
	scene_cent = bren_region_mean(scene_regions->region[index]);
	/* calculate the principal axis of the scene region */
/* 	scene_axis1 = bren_create_vector(2); */
/* 	scene_axis2 = bren_create_vector(2); */
/* 	model_axis1 = bren_create_vector(2); */
/* 	model_axis2 = bren_create_vector(2); */
/* 	bren_region_principal_axes(scene_regions->region[index], scene_cent, */
/* 							   scene_axis1, scene_axis2, &var1, &var2); */
	/* set the axis */
/* 	axis1.p1.x = scene_cent->data[0]; */
/* 	axis1.p1.y = scene_cent->data[1]; */
/* 	axis1.p2.x = (float)(scene_cent->data[0]+scene_axis1->data[0]); */
/* 	axis1.p2.y = (float)(scene_cent->data[1]+scene_axis1->data[1]); */

	model_region = bren_concatregionimage_toregion(model_regs);
	region_moments(model_region, model_intensity, &model_mean, &model_var);
	/* loop through model parts */
	for (i=0; i<model_regs->numregions; i++)
	{
		IPRS_REGION *rotated_model_region;
		float angle, ratio;
		int jj, kk;
		/* calculate the model part centroid */

		/* only test regions which are near each other in size */
		ratio = (float)model_regs->region[i]->numpixels/
			(float)scene_regions->region[index]->numpixels;
		for (jj=0; jj<NUM_REG_CORR; jj++)
		{
			if (fabs(ratio-1.0)<best_ratio[jj])
			{
				for (kk=NUM_REG_CORR-1; kk>jj; kk--)
				{
					best_ratio[kk] = best_ratio[kk-1];
					best_reg[kk] = best_reg[kk-1];
				}
				best_ratio[jj] = fabs(ratio-1.0);
				best_reg[jj] = i;
				break;
			}
		}
	}
/* 		if ((ratio<0.5)||(ratio>2.0)) continue; */

	for (j=0; (j<NUM_REG_CORR)&&(j<model_regs->numregions); j++)
	{
		i = best_reg[j];
		model_cent = bren_region_mean(model_regs->region[i]);
		/* calculate the principal axis of the model region */
/* 		bren_region_principal_axes(model_regs->region[i], model_cent, */
/* 								   model_axis1, model_axis2, &var1, &var2); */
		/* set the axis */
/* 		axis2.p1.x = model_cent->data[0]; */
/* 		axis2.p1.y = model_cent->data[1]; */
/* 		axis2.p2.x = (float)(model_cent->data[0]+model_axis1->data[0]); */
/* 		axis2.p2.y = (float)(model_cent->data[1]+model_axis1->data[1]); */
/* 		angle = bren_extract_relativeorientation(&axis1, &axis2); */
		/* should we add or subtract the angle to the model */
/* 		if (model_axis1->data[1]/model_axis1->data[0]> */
/* 			scene_axis1->data[1]/scene_axis1->data[0]) */
/* 			angle = -angle; */

/* 		rotated_model_region = bren_rotate_region(model_region, 0, */
/* 												  model_cent->data[0], */
/* 												  model_cent->data[1]); */
/* 		iprs_set_intimage(rotated_model_intensity, 0); */
/* 		for (j=0; j<model_region->numpixels; j++) */
/* 		{ */
/* 			if ((bren_in_interval(rotated_model_region->x[j],0,model_width-1)) */
/* 			 &&(bren_in_interval(rotated_model_region->y[j],0,model_height-1))) */
/* 			{ */
/* 				rotated_model_intensity->image[rotated_model_region->x[j]+ */
/* 									rotated_model_region->y[j]*model_width] = */
/* 				model_intensity->image[model_region->x[j] + */
/* 									   model_region->y[j]*model_width]; */
/* 			} */
/* 		} */
/* 		make_intgif(rotated_model_intensity, "tmp.gif"); */
		corr_coeff = bren_sum_squared_error(scene_intensity,
							  scene_regions, index,
							  model_intensity,
							  model_region,
							  scene_cent->data[0]-model_cent->data[0],
							  scene_cent->data[1]-model_cent->data[1],
							  model_mean, region_overlap, scene_region_image);
		if (corr_coeff>max_corr_coeff)
		{
			int k;
			max_corr_coeff = corr_coeff;
			for (k=0; k<scene_regions->numregions; k++)
			{
				if (region_overlap[k])
					max_region_overlap[k] = max_corr_coeff;
				else max_region_overlap[k] = 0.0;
			}
		}
/* 		iprs_free_region(rotated_model_region); */
		bren_free_vector(model_cent);
	}
/* 	bren_free_vector(scene_axis1); */
/* 	bren_free_vector(scene_axis2); */
/* 	bren_free_vector(model_axis1); */
/* 	bren_free_vector(model_axis2); */
	bren_free_vector(scene_cent);
	iprs_free_region(model_region);
	iprs_free(region_overlap);
	return(max_corr_coeff);
}


void bren_hypothesis_verification(SCENE *scene, LIST *part_lists,
								  char *view_file_name, char *region_file_name,
								  char *part_file_name, 
								  int use_n_classifications)
{
	int i, j, k, l, index=0, start_corr;
	LIST *model_parts_matching_scene_parts;
	PART_NUM ***array_of_model_parts_matching;
	int *lengths, *used_parts;
	float max_correlation=-FLT_MAX;
	IPRS_INT_IMAGE *best_model_image=NULL;
	IPRS_REGION_IMAGE *best_model_regs=NULL;
	float *region_overlap, *max_region_overlap, **correlation_matrix;
	int width, height;
	IPRS_INT_IMAGE *scene_region_image;

	width = iprs_getwidth_intimage(scene->intensity);
	height = iprs_getheight_intimage(scene->intensity);
	best_model_image = iprs_create_2dintimage(width, height);

	model_parts_matching_scene_parts = 
		(LIST*)iprs_calloc(list_length(scene->parts),sizeof(LIST));
	for (i=0; i<list_length(scene->parts); i++)
		model_parts_matching_scene_parts[i] = NIL;
	foreach (PART, part, scene->parts)
	{
		foreach (LIST_ITEM, snake_path_list, part_lists[index])
		{
			foreach (SNAKE_PATH, snake_path, snake_path_list)
			{
				int *scene_parts_in_snake;
				/* which scene parts are in the snake */
				scene_parts_in_snake = (int*)iprs_calloc(
											snake_path->snake->path->length, 
											sizeof(int));
				for (i=0; i<snake_path->snake->path->length; i++)
					scene_parts_in_snake[i] = 
						snake_path->snake->path->part[i]->name;
				foreach(PATH, model_path, snake_path->model_paths)
				{
					/* here we want to add the corresponding model parts to
					 * to the scene parts list and then pick the most 
					 * likely one.
					 */
					for (i=0; i<1 /* model_path->length */; i++)
						bren_count_list_parts(
						  model_parts_matching_scene_parts[
							  scene_parts_in_snake[i]],
						  model_path->part[i]->name,
						  model_path->part[i]->view,
						  model_path->part[i]->class);
				} endforeach;
				iprs_free(scene_parts_in_snake);
			} endforeach;
		} endforeach;
/* 		printf("scene part %d\n", part->name); */
/* 		foreach (PART_NUM, part_num, model_parts_matching_scene_parts[index]) */
/* 		{ */
/* 			printf("part_num->view=%d, part_num->name=%d, part_num->num=%d\n", */
/* 				   part_num->view, part_num->part, part_num->num); */
/* 			printf(""); */
/* 		} endforeach; */
		index++;
	} endforeach;

	/* convert the lists to arrays and then sort */
	array_of_model_parts_matching = 
		(PART_NUM***)iprs_calloc(list_length(scene->parts),sizeof(PART_NUM));
	lengths = (int*)iprs_calloc(list_length(scene->parts), sizeof(int));
	for (i=0; i<list_length(scene->parts); i++)
	{
		int index=0;
		lengths[i] = list_length(model_parts_matching_scene_parts[i]);
		array_of_model_parts_matching[i] = 
			(PART_NUM**)iprs_calloc(lengths[i], sizeof(PART_NUM));
		foreach (PART_NUM, part_num, model_parts_matching_scene_parts[i])
		{
			array_of_model_parts_matching[i][index++]=part_num;
		} endforeach;
		list_free(model_parts_matching_scene_parts[i]);
		qsort(array_of_model_parts_matching[i], lengths[i], sizeof(PART_NUM*),
			  (int (*)())part_num_compare);
	}
	iprs_free(model_parts_matching_scene_parts);

	/* now perform the correlation matching */
	index=0;
	used_parts = (int*)iprs_calloc(use_n_classifications, sizeof(int));
	region_overlap=(float*)iprs_calloc(scene->regions->numregions,
									   sizeof(float));
	max_region_overlap=(float*)iprs_calloc(scene->regions->numregions,
										   sizeof(float));
	correlation_matrix = (float**)iprs_calloc(scene->regions->numregions,
											  sizeof(float*));
	for (k=0; k<scene->regions->numregions; k++)
		correlation_matrix[k] = (float*)iprs_calloc(nclasses, sizeof(float));
	scene_region_image = iprs_create_2dintimage(width, height);
	bren_highlight_regionimage(scene->regions, scene_region_image);
	foreach (PART, part, scene->parts)
	{
		int num_tested=0;
		max_correlation = -FLT_MAX;
		j=0;
		while ((num_tested<use_n_classifications)&&
			   (array_of_model_parts_matching[index])&&
			   (array_of_model_parts_matching[index][j]))
		{
			IPRS_INT_IMAGE *model_image;
			IPRS_REGION_IMAGE *model_regs;
			int reg_no, k;
			float corr;
			char use=TRUE;

			/* only use views from different objects */
			for (k=0; k<num_tested; k++)
			{
				if (used_parts[k] == 
					array_of_model_parts_matching[index][j]->class)
					use=FALSE;
			}
			if (!use) 
			{
				j++;
				continue;
			}
			used_parts[num_tested] = 
				array_of_model_parts_matching[index][j]->class;
			
			/* find the first training part belonging to the class */
			for (start_corr=array_of_model_parts_matching[index][j]->part;
				 (training_parts[start_corr]->class==
				 array_of_model_parts_matching[index][j]->class)&&
				 (bren_in_interval(start_corr, 1 ,nparts-1));
				 start_corr--);
			if (start_corr>0) start_corr++;
			start_corr = training_parts[start_corr]->view;
			/* test all views of the model class */
			for (i=start_corr; i<start_corr+5; i++)
			{
/* 			bren_get_modelview(array_of_model_parts_matching[index][j]->view, */
/* 							   array_of_model_parts_matching[index][j]->part, */
/* 							   view_file_name, region_file_name, */
/* 							   part_file_name, &model_image, &model_regs,  */
/* 							   &reg_no); */
			bren_get_modelview(i, 0,
							   view_file_name, region_file_name,
							   part_file_name, &model_image, &model_regs, 
							   &reg_no);
			corr = bren_correlate_objects(scene->intensity, scene->regions, 
										  scene_region_image,
										  index, model_image, model_regs, 
										  reg_no, region_overlap);
			if (corr>max_correlation)
			{
				int k;
/* 				make_intgif(model_image, "best_model.gif"); */
/* 				bren_regionimage_togif_cat(model_regs,best_model_image, */
/* 										   "best_regs"); */
				printf("correlation = %f\n", corr);
				max_correlation = corr;
				part->class = array_of_model_parts_matching[index][j]->class;
				if (best_model_image) iprs_free_intimage(best_model_image);
				if (best_model_regs) iprs_free_regionimage(best_model_regs);
				for (k=0; k<scene->regions->numregions; k++)
				{
					printf("region_overlap[%d]=%f\n", k, region_overlap[k]);
					max_region_overlap[k] = region_overlap[k];
				}
				best_model_image = model_image;
				best_model_regs = model_regs;
			}
			else
			{
				iprs_free_intimage(model_image);
				iprs_free_regionimage(model_regs);
			}
		}
			num_tested++; j++;
		}
		for (k=0; k<scene->regions->numregions; k++)
		{
/* 			correlation_matrix[k][part->class] += max_region_overlap[k]; */
			/*
			 * We probably want to find the best correlation for this part
			 * for this class, rather than just adding them all up.
			 */
			if (max_region_overlap[k] > correlation_matrix[k][part->class])
				correlation_matrix[k][part->class] = max_region_overlap[k];
		}
		index++;
	} endforeach;

	k=0;
	foreach (PART, part, scene->parts)
	{
		int class=0;
		for (l=1; l<nclasses; l++)
			if (correlation_matrix[k][l]>correlation_matrix[k][class])
				class = l;
		part->class = class;
		k++;
	} endforeach;
	for (k=0; k<scene->regions->numregions; k++)
		iprs_free(correlation_matrix[k]);
	iprs_free(correlation_matrix);
	iprs_free(region_overlap);
	iprs_free(max_region_overlap);
	iprs_free(used_parts);
}
/*
 * This function extracts the most likely view that each part has indexed
 * in the model base, which is then used for hypothesis verification.
 */

void bren_final_classification(SCENE *scene, int use_n_classifications, 
							   int nclasses, char *view_file_name,
							   char *region_file_name,
							   char *part_file_name)
{
	LIST *part_lists;
	int index=0, i, j, k;

	part_lists = (LIST *)iprs_calloc(list_length(scene->parts), sizeof(LIST));
	foreach (PART, part, scene->parts)
	{
		int *use_class;
		part_lists[index] = list_initialize();
		/* find the best use_n_classifications classifications */
		use_class = (int*)iprs_calloc(use_n_classifications, sizeof(int));
		for (i=0; i<use_n_classifications; i++) use_class[i]=-1;
		use_class[0]=0;
		/* check all classifications for this part */
		for (i=1; i<nclasses; i++)
		{
			/* see if current classification > saved ones */
			for (j=0; j<use_n_classifications; j++)
			{
				if ((use_class[j]==-1)||
					(part->cvector[use_class[j]]<part->cvector[i]))
				{
					/* shuffle smaller ones along */
					for (k=j+1; k<use_n_classifications; k++)
						use_class[k] = use_class[k-1];
					/* insert current classification */
					use_class[j] = i;
					j = use_n_classifications; 
				}
			}
		}
		for (i=0; i<use_n_classifications; i++)
		{
			LIST views=NIL;
			LIST snake_path_list=NIL;
			int max=0, max_view;
			/* loop through all snakes involved in this part */
			foreach (SNAKE, snake, part->snakes)
			{
				SNAKE_PATH *snake_path;
				snake_path = (SNAKE_PATH*)iprs_calloc(1, sizeof(SNAKE_PATH));
				snake_path->snake = snake;
				snake_path->model_paths = list_initialize();
				/* test all classifications of the snake */
				foreach (CLASSIFICATION,classification,snake->classifications)
				{
					/* test each model path in the cluster */
					foreach (PATH, path, classification->cluster->paths)
					{
						/*
						 * if the classification includes current class
						 * add the snake and path to the snake_path list.
						 */
						if (path->part[0]->class==use_class[i])
						{
							char add=TRUE;
							list_append(path, snake_path->model_paths);
							foreach (VIEW_NUM, view, views)
							{
								if (view->view==path->part[0]->view)
								{
									view->num++;
									add = FALSE;
									if(classification->fuzzy_value>view->fuzzy)
										view->fuzzy=snake->compatibility;
								}
							} endforeach;
							if (add)
							{
								VIEW_NUM *tmp;
								tmp=(VIEW_NUM*)iprs_calloc(1,sizeof(VIEW_NUM));
								tmp->view=path->part[0]->view;
								tmp->num=1;
								tmp->fuzzy = classification->fuzzy_value;
								list_append(tmp, views);
							}
						}
					} endforeach;
				} endforeach;
				/*
				 * if a valid path exists for the current snake add it to
				 * the snake_path list.
				 */
				if (list_length(snake_path->model_paths)>0)
					list_append(snake_path, snake_path_list);
				else
				{
					list_free(snake_path->model_paths);
					iprs_free(snake_path);
				}
			} endforeach;
			/* find the most prominent view in this class */
			max=max_view=0;
			foreach (VIEW_NUM, view, views)
			{
				if (view->num > max)
				{
					max_view = view->view;
					max = view->num;
				}
				iprs_free(view);
			} endforeach;
			list_free(views);
			/* only use model paths from this view for verification */
			foreach (SNAKE_PATH, snake_path, snake_path_list)
			{
				LIST current_pos=snake_path->model_paths;
				/*
				 * I need to do this shit to remove from a list while 
				 * traversing. 
				 */
				while ((current_pos!=NULL)&&(current_pos->tail!=NULL))
				{
					PATH *model_path=(PATH*)current_pos->head;
					/* a different view - delete */
					if (model_path->part[0]->view != max_view)
						list_remove(model_path, snake_path->model_paths);
					else current_pos = current_pos->tail;
				}
			} endforeach;
			list_append(snake_path_list, part_lists[index]);
		}
		iprs_free(use_class);
/* 		foreach (LIST_ITEM, snake_path_list, part_lists[index]) */
/* 		{ */
/* 			foreach (SNAKE_PATH, snake_path, snake_path_list) */
/* 			{ */
/* 				printf("scene snake is: "); */
/* 				print_path2(snake_path->snake->path); */
/* 				printf("model_paths are:\n"); */
/* 				foreach (PATH, model_path, snake_path->model_paths) */
/* 				{ */
/* 					print_path2(model_path); */
/* 				} endforeach; */
/* 			} endforeach; */
/* 		} endforeach; */
		index++;
	} endforeach;

	bren_hypothesis_verification(scene, part_lists, view_file_name,
								 region_file_name, part_file_name,
								 use_n_classifications);
	index=0;
	foreach (PART, part, scene->parts)
	{
		printf("%d: classif = %d\n", index++, part->class);
	} endforeach;
}

					
