/* ----------------------------------------------------------------------
 * Function prototypes
 * ---------------------------------------------------------------------- */

/* crg */

void add_clist_to_queue(LIST clist, LIST queue);
void check_labels(float **label, int nlabel, int nbinary,
				  int npart, int class, int view);
LIST cluster_feature_list(FEATURE_LIST *f, int nclusters, float fuzzyq, 
						  float delta_centres_threshold);
LIST recluster_cluster(CLUSTER *c, int nclusters, float fuzzyq, 
					   float delta_centres_threshold);
float compute_entropy(int *class);
void compute_range(float feature_difference);
void compute_zscore();
void compute_mean_sdev();
void create_cluster_tree(CLUSTER *root, int nclusters, float fuzzyq, 
						 float delta_centres_threshold);
FEATURE_LIST *create_feature_list(CLUSTER *c);
void free_feature_list(FEATURE_LIST *f);
CLUSTER *create_root_cluster();
int feature_in_rule(float *value, RULE *rule);
void read_database(char *filename);
void save_training(CLUSTER *root, char *infilename, char *outfilename);

/* crganalyze */

void print_cluster_tree(CLUSTER *c, LIST leaf_list, int *leaf_number);
void print_rules(LIST leaf_list);
void analyze_rules(CLUSTER *c,float cvol,int *nrule,int *ncase,float *volume);
void compute_unary_volume();
void compute_binary_volume();
void mark_important_features(CLUSTER *c);
void print_unresolved();
void print_rule_summary(int *nrule, int *ncase, float *volume);
void print_summary();
void analyze_cluster_tree(CLUSTER *root);

/* crgcluster */

IPRS_HRECT_RULE *single_cluster(FEATURE_LIST *f);
IPRS_HRECT_RULE *leader_cluster(FEATURE_LIST *f, float threshold);
IPRS_HRECT_RULE *agglomerate_cluster(FEATURE_LIST *f, int max_ncluster);
float fuzzy_validity(FEATURE_LIST *f, float **cluster_centres, int nclusters, 
					 float fuzzyq);

/* crgorder */

float sort_cluster(CLUSTER *c);
void rename_clusters(CLUSTER *c);
void sort_cluster_tree(CLUSTER *root);

/* crgpath */

PATHNODE *create_path_tree();
PATHNODE *create_path_subtree(PART *part, PATHNODE *parent,
							  UNARY *unary, BINARY *binary);
int part_in_path(PART *part, PATHNODE *pathnode);
PATH *get_path(PATHNODE *node);
void free_path(PATH *path);

/* crgsplit */

LIST fuzzy_split(CLUSTER *leaf, LIST queue, float fuzzyq, 
		   float delta_centres_threshold);
LIST fuzzy_split2(CLUSTER *leaf, LIST queue, float fuzzyq, 
		   float delta_centres_threshold);
LIST split(CLUSTER *leaf, LIST queue, float fuzzyq, 
		   float delta_centres_threshold);
void best_split(CLUSTER *c, CLUSTER *leaf,
                int   *best_found,
                float *best_entropy,
                int   *best_feature,
                float *best_value);
float split_entropy(int *c1, int *c2, int n, int npoints);
LIST split_cluster(CLUSTER *c, int best_feature, float bbound);
void split_rule(CLUSTER *c, int best_feature, float bound1, RULE **nrule);
void remove_cluster_tree_from_queue(CLUSTER *c, LIST queue);
LIST split_paths(CLUSTER *c, LIST queue);
float fuzzy_split_entropy(LIST cluster_list);

/* varia */

void print_parts();
void output_xgobi(char *basename);
void print_cluster(CLUSTER *cptr);
void free_cluster(CLUSTER *c);
void free_cluster_tree(CLUSTER *c);
IPRS_FEATURE_SPACE *crg_create_featurespace(int numf,int nump);
void crg_free_featurespace(IPRS_FEATURE_SPACE *sptr);
void process_debug_options(int noptions, char **option);
int memory_used();
float timer();
void error_exit(char *message);
void output_clustertree_preorder(CLUSTER *root);
void output_fuzzyclustertree_preorder(CLUSTER *root);

/* bren_functions */
IPRS_FEATURE_SPACE *bren_removefeatures_featurespace(IPRS_FEATURE_SPACE *src,
													 int *usefeatures,
													 int num_newfeatures);
void bren_calculate_means(CLUSTER *root);
float *bren_calc_distances(float *pt, LIST clusters, int nfeatures);
#define bren_free(ptr, free_function) \
            {if (ptr) {free_function(ptr); ptr=NULL;}}
float **fuzzy_kmeans_cluster(FEATURE_LIST *f, int nclusters, float fuzzyq,
							 float delta_centres_threshold);
float multid_feuclid_dist(float *pt1, float *pt2, int numdims);
