/* ====================================================================
 * Package:		classify
 * ==================================================================== */

/* Include files */

#include <iprs.h>
#include "sure.h"
#include "sureextern.h"

long tmp1, tmp2, exp_snake_mem=0, snake_union_mem=0; 
long snake_prepend_mem=0, union_1=0, union_2=0;
long add_best_mem1=0, add_best_mem2=0;
long int compute_snake_mem=0, initial_mem=0;
long int snake_mem=0, classification_mem=0;
long int path_mem=0;
long int s_mem=0, new_snakes_mem=0, plist_mem=0;

/* --------------------------------------------------------------------
 * Procedure: scene_classify
 * Comments:  classify a scene using
 *			  1) initial snakes
 *			  2) intra-snake permutation constraint
 *            3) inter-snake single classification constraint
 *			  3) inter-snake compatibility
 *			  4) intra-snake relaxation labeling
 * -------------------------------------------------------------------- */

void classify_parts(LIST parts)
{
	float max_part_classification, max_classification=0.0;
	int i;

	foreach(PART,part,parts) {
		float tmp_max=0.0;

		max_part_classification=0.0;
		foreach(SNAKE,snake,part->snakes) {
			float tmp_max=0.0, tmp_max_class=0.0;
			snake_classification_from_rules(snake);
			for (i=0; i<nclasses; i++)
			{
				tmp_max += snake->cvector[i];
				if (snake->cvector[i]>tmp_max_class)
					tmp_max_class = snake->cvector[i];
			}
			if (tmp_max>max_classification)
			{
				max_classification = tmp_max;
			}
			if (tmp_max_class>max_part_classification)
			{
				max_part_classification = tmp_max_class;
				if (verbosity>1)
				{
					printf("max snake: %f ", max_part_classification);
					print_path2(snake->path);
					print_classification_vector(snake->cvector);
					printf("\n");
				}
			}
		}
		endforeach;
		/* get rid of snakes not within a certain percentage of the */
		/* maximum for that part. */
	    {
			LIST current_pos=part->snakes;
			while ((current_pos!=NULL)&&(current_pos->tail!=NULL))
			{
				SNAKE *snake=current_pos->head;
				float sum=0.0;
				if (verbosity>0)
					printf("numclassifications=%d\n", 
						   list_length(snake->classifications));
				for (i=0; i<nclasses; i++)
					if (snake->cvector[i]>sum)
						sum = snake->cvector[i];

				if (sum/(float)max_part_classification < cutoff_snake_percentage)
				{
					/* NB. REMOVING AN ITEM FROM A LIST WHILE TRAVERSING IT */
					/* COULD BE VERY DANGEROUS TO THE HEALTH OF THIS PROGRAM */
					list_remove(snake, part->snakes);
					if (verbosity>1)
					{
						printf("removing snake: ");
						print_path2(snake->path);
						print_classification_vector(snake->cvector);
					}
					snake_free(snake);
				}
				else
				{
					current_pos = current_pos->tail;
				}
			}
		}
	}
	endforeach;
	/* normalise snake classification vectors */
	foreach(PART,part,scene->parts) {
		for (i=0; i<nclasses; i++)
			part->cvector[i]=0.0;
		foreach(SNAKE,snake,part->snakes) {
			for (i=0;i<nclasses;i++)
			{
				/* try to normalise to 1 */
				snake->cvector[i] /= max_classification;
			}
		}
		endforeach;
	}
	endforeach;
	foreach(PART,part,scene->parts) {
		float total=0;
		for (i=0; i<nclasses; i++)
			part->cvector[i]=0.0;
		foreach(SNAKE,snake,part->snakes) {
			for (i=0;i<nclasses;i++)
			{
				part->cvector[i] += snake->cvector[i];
				total += snake->cvector[i];
			}
		}
		endforeach;
		if (list_length(part->snakes)>0)
			for (i=0;i<nclasses;i++)
				part->cvector[i] /= (float)total;/*  list_length(part->snakes); */
	}
	endforeach;
}

void print_num_snakes(PART *part)
{
	int i=0;
	int num_class=0;
	printf("num_snakes=%d\n", list_length(part->snakes));
	foreach (SNAKE,snake,part->snakes)
		num_class += list_length(snake->classifications);
	endforeach;
	printf("num_class=%d\n",num_class);
}

void classify_scene()
{
	int i;
	float max_classification=0.0, max_part_classification=0.0;
	float *compat_cvector;
	/* Compute all snakes */

	foreach(PART,part,scene->parts) {
		part->snakes=NIL;
/* 		part->touched_snakes=NIL; */
	}
	endforeach;

	foreach(PART,part,scene->parts) {
		compute_snakes(part);
	}
	endforeach;

	/* ----------------------------------- */
	/* Classification using initial snakes */
	/* ----------------------------------- */

	classify_parts(scene->parts);

	printf("\nPart classification using initial snakes\n");
	print_part_classification();
	print_classification_entropy();
	if (classification_known) classification_summary();

	/* --------------------------------- */
	/* (intra-snake) relaxation labeling */
	/* --------------------------------- */

	if (1) /* use_relaxation_labeling )*/ {
		relaxation_labeling();
		printf("\nPart classification after relaxation labeling\n");
		print_part_classification();
		print_classification_entropy();
/* 		printf("\nFinal decision is:\n"); */
/* 		print_final_decision(); */
	}

	/* perform the hypothesis verification etc. */
/* 	bren_final_classification(scene, 2, nclasses, view_file, */
/* 							  reg_file, part_file); */

	/* ------------------------- */
	/* summary of classification */
	/* ------------------------- */

	if (classification_known) classification_summary();
}

void original_classify_scene()
{
	int i;
	float max_classification=0.0;
	float *compat_cvector;
	/* Compute all snakes */

	foreach(PART,part,scene->parts) {
		part->snakes=NIL;
		part->touched_snakes=NIL;
	}
	endforeach

	foreach(PART,part,scene->parts) {
		compute_snakes(part);
	}
	endforeach

	/* ----------------------------------- */
	/* Classification using initial snakes */
	/* ----------------------------------- */

	foreach(PART,part,scene->parts) {
		float tmp_max=0.0, max_part_classification=0.0;

		foreach(SNAKE,snake,part->snakes) {
			float tmp_max=0.0;
			snake_classification_from_rules(snake);
			for (i=0; i<nclasses; i++)
				tmp_max += snake->cvector[i];
			if (tmp_max>max_classification)
				max_classification = tmp_max;
			if (tmp_max>max_part_classification)
				max_part_classification = tmp_max;
		}
		endforeach;
		/* get rid of snakes not within a certain percentage of the */
		/* maximum for that part. */
	    {
			LIST current_pos=part->snakes;
			while ((current_pos!=NULL)&&(current_pos->tail!=NULL))
			{
				SNAKE *snake=current_pos->head;
				float sum=0.0;
				if (verbosity>0)
					printf("numclassifications=%d\n", 
						   list_length(snake->classifications));
				for (i=0; i<nclasses; i++)
					sum += snake->cvector[i];

				if (sum/(float)max_part_classification < 0.1)
				{
					/* NB. REMOVING AN ITEM FROM A LIST WHILE TRAVERSING IT */
					/* COULD BE VERY DANGEROUS TO THE HEALTH OF THIS PROGRAM */
					list_remove(snake, part->snakes);
					snake_free(snake);
				}
				else
				{
					current_pos = current_pos->tail;
				}
			}
		}
/* 		part_classification_from_snakes(part); */
	}
	endforeach;
	/* normalise snake classification vectors */
	foreach(PART,part,scene->parts) {
		for (i=0; i<nclasses; i++)
			part->cvector[i]=0.0;
		foreach(SNAKE,snake,part->snakes) {
			for (i=0;i<nclasses;i++)
			{
				/* try to normalise to 1 */
				snake->cvector[i] /= max_classification;
				part->cvector[i] += snake->cvector[i];
			}
		}
		endforeach;
		for (i=0;i<nclasses;i++)
			part->cvector[i] /= (float)list_length(part->snakes);
	}
	endforeach;

	foreach(PART,part,scene->parts) {
		compute_touched_snakes(part);
	}
	endforeach;

	compat_cvector = (float*)iprs_calloc(nclasses, sizeof(float));
	foreach(PART,part,scene->parts) {
		for (i=0; i<nclasses; i++) compat_cvector[i]=0.0;
		printf("touched snakes for part %d\n", part->name);
		foreach(SNAKE, snake, part->touched_snakes)
		{
			for (i=0; i<nclasses; i++) 
				compat_cvector[i] += snake->cvector[i];
			print_path(snake->path);
			print_classification_vector(snake->cvector);
			printf("\n");
		}
		endforeach;
		for (i=0; i<nclasses; i++)
			compat_cvector[i] /= (float)list_length(part->touched_snakes);
		printf("compatibility cvector: ");
		print_classification_vector(compat_cvector);
		printf("\n");
	}
	endforeach

	printf("\nPart classification using initial snakes\n");
	print_part_classification();
	print_classification_entropy();
	if (classification_known) classification_summary();

	/* ---------------------------- */
	/* snake permutation constraint */
	/* ---------------------------- */

	compute_model_parts();
	compute_permutation_snakes();
	if (verbosity>0 && use_permutation_constraint) {
		printf("\nModel parts for all snakes\n");
		print_model_parts();
		print_snakes();
	}

	if (use_permutation_constraint) {
		snake_permutation_constraint();
		foreach(PART,part,scene->parts) {
			part_classification_from_snakes(part);
		}
		endforeach
		printf("\nPart classification after snake permutation\n");
		print_part_classification();
		print_classification_entropy();
		if (classification_known) classification_summary();
	}

	/* ------------------------------------------- */
	/* single classification constraint            */
	/* (if successful redo permutation constraint) */
	/* ------------------------------------------- */

	if (use_single_classification_constraint) {
		if (single_classification_constraint()) {
			snake_permutation_constraint();
			foreach(PART,part,scene->parts) {
				part_classification_from_snakes(part);
			}
			endforeach
			printf("\nPart classification after single classification\n");
			print_part_classification();
			print_classification_entropy();
			if (classification_known) classification_summary();
		}
	}

	/* ------------------------- */
	/* inter-snake compatibility */
	/* ------------------------- */

	if (use_intersnake_compatibility) {
		analyze_intersnake_compatibility();
		foreach(PART,part,scene->parts) {
			part_classification_from_weighted_snakes(part);
		}
		endforeach
		printf("\nPart classification after snake compatibility\n");
		print_part_classification();
		print_classification_entropy();
		if (classification_known) classification_summary();
	}

	/* --------------------------------- */
	/* (intra-snake) relaxation labeling */
	/* --------------------------------- */

	if (use_relaxation_labeling) {
		relaxation_labeling();
		printf("\nPart classification after relaxation labeling\n");
		print_part_classification();
		print_classification_entropy();
		printf("\nFinal decision is:\n");
		print_final_decision();
	}

	/* ------------------------- */
	/* summary of classification */
	/* ------------------------- */

	if (classification_known) classification_summary();
}

/* --------------------------------------------------------------------
 * Procedure: compute_snakes
 * Comment:   determine all snakes starting at a part
 * -------------------------------------------------------------------- */

void compute_snakes(PART *part)
{
	CLASSIFICATION *classification;
	LIST queue=NIL,slist;
	PATH *path;
	SNAKE *snake;
	long int numloops=0;
	int i;

	path=create_path(part);
	snake=create_snake(path);

	queue = bren_sure_add_best_clusters(cluster_tree, snake, NIL, 'U',
								fuzzy_nclusters, fuzzyq);

	if (list_length(queue) == 0) {
		printf("Cannot classify part %d\n",part->name);
		return;
	}

	while (!list_empty(queue)) {
		numloops++;
		snake=list_remove_first(queue);
		slist=expand_snake(snake);
		if (list_empty(slist)) {
			snake_list_union(snake,part->snakes);
		}
		else {
			foreach(SNAKE,s,slist) {
				list_prepend(s,queue);
			}
			endforeach;
			snake_free(snake);
		}
		list_free(slist);
	}
}

/* --------------------------------------------------------------------
 * Procedure: create_snake
 * Comment:   create snake
 * -------------------------------------------------------------------- */

PATH *path_copy(PATH *orig)
{
	PATH *copy;
	int i;

	copy = (PATH*)iprs_calloc(1, sizeof(PATH));
    copy->length = orig->length;
	copy->part = (PART **)iprs_calloc(copy->length, sizeof(PART*));
	copy->unary=(UNARY **)iprs_calloc(copy->length,sizeof(UNARY *));
	copy->binary=(BINARY **)iprs_calloc(copy->length,sizeof(BINARY *));
	for (i=0; i<copy->length; i++)
	{
		copy->part[i] = orig->part[i];
		copy->unary[i] = orig->unary[i];
		copy->binary[i] = orig->binary[i];
	}
	return(copy);
}

SNAKE *create_snake(PATH *path)
{
	int i,n;
	SNAKE *snake;

	snake=(SNAKE *)iprs_malloc(sizeof(SNAKE));
	snake->path=path_copy(path);
	snake->classifications=NIL;
	snake->permutations=NIL;
	n=path->length;
	snake->modelparts=(LIST *)iprs_calloc(n,sizeof(LIST));
	for (i=0;i<n;i++) {
		snake->modelparts[i]=NIL;
	}
	snake->cvector=(float *)iprs_calloc(nclasses,sizeof(float));
	for (i=0;i<nclasses;i++)
		snake->cvector[i]=0.0;
	snake->compatibility=1.0;

	return snake;
}

/* --------------------------------------------------------------------
 * Procedure: expand_snake
 * Comment:   expand path of a snake and create associated snakes
 * -------------------------------------------------------------------- */

void path_free(PATH *path)
{
	if (path->part) iprs_free(path->part);
	if (path->unary) iprs_free(path->unary);
	if (path->binary) iprs_free(path->binary);
	iprs_free(path);
}

LIST expand_snake(SNAKE *snake)
{
	UNARY *unary;
	BINARY *binary;
	LIST plist,slist;
	CLASSIFICATION *c;
	PATH *path;
	SNAKE *s;
	LIST new_snakes;
	int i;

	slist = NIL;

	c=list_first(snake->classifications);
	if (c->level >= max_level) return slist;
	if (c->level >= runtime_max_level) return slist;
	if (c->cluster->type == 'U') {
		plist=expand_path(snake->path);
		if (list_empty(plist)) 
		{
			list_free(plist);
			return slist;
		}
		foreach(PATH,path,plist) {
			s=create_snake(path);
			binary=path->binary[path->length-1];
			foreach(CLASSIFICATION,classification,snake->classifications) {
				new_snakes = bren_sure_add_best_clusters(
											classification->cluster,
											s, classification->fuzzy_cfn, 'B',
											fuzzy_nclusters, fuzzyq);
				foreach (SNAKE, s, new_snakes)
				{
					list_append(s, slist);
				}
				endforeach;
				list_free(new_snakes);
			}
			endforeach;
			path_free(path);
			snake_free(s);
		}
		endforeach;
		list_free(plist);
	}
	else {   /* c->cluster->type == 'B' */
		path=snake->path;
		s=create_snake(path);
		unary=path->unary[path->length-1];
		foreach(CLASSIFICATION,classification,snake->classifications) {
			new_snakes = bren_sure_add_best_clusters(classification->cluster,
										s, classification->fuzzy_cfn, 'U',
										fuzzy_nclusters, fuzzyq);
			foreach (SNAKE, s, new_snakes)
			{
				list_append(s, slist);
			}
			endforeach;
			list_free(new_snakes);
		}
		endforeach;
		snake_free(s);
	}
	return slist;
}

/* --------------------------------------------------------------------
 * Procedure: create_classification
 * Comment:   create classification
 * -------------------------------------------------------------------- */

/* need to add fuzzy membership list as parameter so can copy fuzzy 
   membership values from all the classifcations in the ancestor path 
   also add current fuzzy membership */

CLASSIFICATION *create_classification(PATH *path,CLUSTER *cluster,
									  LIST fuzzy_cfn, float current_fuzzy)
{
	int i;
	CLASSIFICATION *classification;
	float *cf, total_fuzz;

	classification=(CLASSIFICATION *)iprs_malloc(sizeof(CLASSIFICATION));
	classification->level=cluster->level;
	classification->cluster=cluster;
	classification->cvector=(float *)iprs_calloc(nclasses,sizeof(float));
	classification->path=path;
	classification->fuzzy_cfn = list_copy(fuzzy_cfn);
	cf = (float*)iprs_malloc(sizeof(float));
	*cf = current_fuzzy;
	list_append(cf, classification->fuzzy_cfn);
	total_fuzz=1.0;
	foreach (float, fuzz, classification->fuzzy_cfn)
	{
		/* there are two ways of combining the fuzzy memberships:
		   multiplying (probability formulation), or minimum. I
		   have chosen to use minimum since it doesn't decrease with
		   increasing depth of the tree.
		*/
		if ((*fuzz)<total_fuzz)
			classification->fuzzy_value = (*fuzz);
	}
	endforeach;

	for (i=0;i<nclasses;i++)
		classification->cvector[i]=
			cluster->cvector[i]*classification->fuzzy_value;

	return classification;
}

CLASSIFICATION *classification_copy(CLASSIFICATION *old)
{
	int i;
	CLASSIFICATION *classification;
	float *cf, total_fuzz;

	classification=(CLASSIFICATION *)iprs_malloc(sizeof(CLASSIFICATION));
	classification->level=old->level;
	classification->cluster=old->cluster;
	classification->cvector=(float *)iprs_calloc(nclasses,sizeof(float));
	for (i=0; i<nclasses; i++)
		classification->cvector[i] = old->cvector[i];
	classification->path=old->path;
	classification->fuzzy_cfn = list_copy(old->fuzzy_cfn);
	classification->fuzzy_value = old->fuzzy_value;
	return(classification);
}

void classification_free(CLASSIFICATION *c)
{
	if (c)
	{
		list_free(c->fuzzy_cfn);
		iprs_free(c->cvector);
/* 		path_free(c->path); */
		iprs_free(c);
	}
}

/* --------------------------------------------------------------------
 * Procedure: create_path
 * Comment:   create path
 * -------------------------------------------------------------------- */

PATH *create_path(PART *part)
{
	PATH *path;

	path=(PATH *)iprs_malloc(sizeof(PATH));
	path->length=1;
	path->part=(PART **)iprs_calloc(1,sizeof(PART *));
	path->unary=(UNARY **)iprs_calloc(1,sizeof(UNARY *));
	path->binary=(BINARY **)iprs_calloc(1,sizeof(BINARY *));
	path->part[0]=part;
	path->unary[0]=part->unary;

	return path;
}

/* --------------------------------------------------------------------
 * Procedure: expand_path
 * Comment:   expand path by one part, return list of new paths
 * -------------------------------------------------------------------- */

LIST expand_path(PATH *oldpath)
{
	int length,i;
	LIST plist;
	PATH *newpath;
	PART *lastpart,*newpart;
	int tmp1, tmp2;

	plist=NIL;
	lastpart=oldpath->part[oldpath->length-1];

	foreach(BINARY,binary,lastpart->binary) {
		newpart=binary->part2;
		if ( !part_in_path(newpart,oldpath) ) {
			newpath=(PATH *)iprs_malloc(sizeof(PATH));
			length=oldpath->length;
			newpath->length=length+1;
			newpath->part=(PART **)iprs_calloc(length+1,sizeof(PATH *));
			newpath->unary=(UNARY **)iprs_calloc(length+1,sizeof(UNARY *));
			newpath->binary=(BINARY **)iprs_calloc(length+1,sizeof(BINARY *));
			for (i=0;i<length;i++) {
				newpath->part[i]=oldpath->part[i];
				newpath->unary[i]=oldpath->unary[i];
				newpath->binary[i]=oldpath->binary[i];
			}
			newpath->part[length]=newpart;
			newpath->unary[length]=newpart->unary;
			newpath->binary[length]=binary;
			list_append(newpath,plist);
		}
	}
	endforeach

	return plist;
}

/* --------------------------------------------------------------------
 * Procedure: part_classification_from_snakes
 * Comment:   compute part classification from all snakes starting at
 *            part
 * -------------------------------------------------------------------- */

float part_classification_from_snakes(PART *part)
{
	int i;
	int total;
	float max=0.0;

	for (i=0;i<nclasses;i++) {
		part->cvector[i]=0.0;
	}
	total=0;

	foreach(SNAKE,snake,part->snakes) {
		float tmp_max=0.0;
		for (i=0;i<nclasses;i++) {
			part->cvector[i] += snake->cvector[i];
			tmp_max += snake->cvector[i];
		}
		if (tmp_max>max)
		{
			max = tmp_max;
/* 			for (i=0;i<nclasses;i++) { */
/* 				part->cvector[i] = snake->cvector[i]; */
/* 			} */
		}
		total++;
	}
	endforeach;

	if (total > 0) {
		for (i=0;i<nclasses;i++) {
			part->cvector[i] /= (float)total;
			max += part->cvector[i];
		}
	}
	return(max);
}

/* --------------------------------------------------------------------
 * Procedure: part_classification_from_weighted_snakes
 * Comment:   compute part classification from all snakes starting at
 *            part
 * -------------------------------------------------------------------- */

void part_classification_from_weighted_snakes(PART *part)
{
	int i;
	float weight;
	int total;

	for (i=0;i<nclasses;i++) {
		part->cvector[i]=0.0;
	}
	total=0;

	foreach(SNAKE,snake,part->snakes) {
		weight=snake->compatibility;
		for (i=0;i<nclasses;i++) {
			part->cvector[i] += weight * snake->cvector[i];
			total ++;
		}
	}
	endforeach

	if (total > 0) {
		for (i=0;i<nclasses;i++) {
			part->cvector[i] /= total;
		}
	}
}

/* --------------------------------------------------------------------
 * Procedure: snake_classification_from_rules
 * Comment:   compute snake_classification from all classification rules
 * -------------------------------------------------------------------- */

void snake_classification_from_rules(SNAKE *snake)
{
	int i;
	int total;
	float max=0.0;

	for (i=0;i<nclasses;i++) {
		snake->cvector[i]=0.0;
	}
	total=0;

	foreach(CLASSIFICATION,classification,snake->classifications) {
		float tmp_max=0.0;
		for (i=0;i<nclasses;i++) {
/* 			snake->cvector[i] += classification->cvector[i]; */
			if (classification->cvector[i]>tmp_max)
				tmp_max = classification->cvector[i];
		}
		if (tmp_max>max)
		{
			max = tmp_max;
		}
/* 		total++; */
	}
	endforeach;

   {
	   LIST current_pos=snake->classifications;
	   while ((current_pos!=NULL)&&(current_pos->tail!=NULL))
	   {
		   CLASSIFICATION *classification=(CLASSIFICATION*)current_pos->head;
		   float tmp_max=0.0;
		   for (i=0;i<nclasses;i++) {
			   if (classification->cvector[i]>tmp_max)
				   tmp_max = classification->cvector[i];
		   }
		   /*
			* Remove classifications not within 50% of the maximum.
			*/
		   if (tmp_max/max > cutoff_snake_percentage) 
		   {
			    for (i=0;i<nclasses;i++)
					snake->cvector[i] += classification->cvector[i];
				total++;
				current_pos = current_pos->tail;
		   }
		   else
		   {
			   list_remove(classification,snake->classifications);
			   classification_free(classification);
		   }
	    }
    }

	for (i=0;i<nclasses;i++) {
		snake->cvector[i] /= (float)total;
	}
}

/* --------------------------------------------------------------------
 * Procedure: classification_summary
 * Comments:  
 * -------------------------------------------------------------------- */

void classification_summary()
{
	int nparts,correct,ncorrect,notclassified,i;
	float ctrue;

	nparts=list_length(scene->parts);
	ncorrect=0;
	notclassified=0;
	foreach(PART,part,scene->parts) {
		ctrue=part->cvector[part->true_classification];
		correct=1;
		for (i=0;i<nclasses;i++) {
			if (part->cvector[i] > ctrue) {
				correct=0;
				break;
			}
		}
		if (correct) {
			if (part->cvector[part->true_classification] > 0.0)
				ncorrect++;
			else
				notclassified++;
		}
	}
	endforeach

	printf("\nSumary of part classification\n");
	printf("Number of parts           %d\n",nparts);
	printf("Number correct            %d\n",ncorrect);
	if (notclassified > 0) {
		printf("Not classified            %d\n",notclassified);
	}
}

/* --------------------------------------------------------------------
 * Procedure: compute_touched_snakes
 * Comments:  compute list of all snakes that pass through a part
 * -------------------------------------------------------------------- */

void compute_touched_snakes(PART *part)
{
	int i;

	foreach(SNAKE,snake,part->snakes) {
		for (i=0;i<snake->path->length;i++) {
			snake_list_union(snake,snake->path->part[i]->touched_snakes);
		}
	}
	endforeach
}

/* --------------------------------------------------------------------
 * Procedure: snake_list_union
 * Comments:  add snake to list if it is not already in list
 * -------------------------------------------------------------------- */

void snake_list_union(SNAKE *snake,LIST slist)
{
	long int tmp1, tmp2;

	foreach(SNAKE,s,slist) {
		if (snake == s) return;
		else if (equal_path(snake->path,s->path)) 
		{
			foreach(CLASSIFICATION, cfn, snake->classifications)
			{
				list_append(classification_copy(cfn), s->classifications);
			}
			endforeach;
			snake_free(snake);
			return;
		}
	}
	endforeach
	list_append(snake,slist);
}

/* --------------------------------------------------------------------
 * Procedure: snake_free
 * Comments:  free all things associated with the snake
 * -------------------------------------------------------------------- */

void snake_free(SNAKE *snake)
{
	if (snake->modelparts)
	{
		int i;

		for (i=0; i<snake->path->length; i++)
			list_free(snake->modelparts[i]);
		bren_free(snake->modelparts, iprs_free);
	}
	bren_free(snake->path, path_free);
	foreach (CLASSIFICATION, cl, snake->classifications)
		classification_free(cl);
	endforeach;
	list_free(snake->classifications);
	list_free(snake->permutations);
	bren_free(snake->cvector, iprs_free);
	bren_free(snake, iprs_free);
}
