/* ====================================================================
 * Package:		print
 * ==================================================================== */

/* Include files */

#include <iprs.h>
#include "sure.h"
#include "sureextern.h"

/* --------------------------------------------------------------------
 * Procedure: print_classification_entropy
 * Comment:   
 * -------------------------------------------------------------------- */

void print_classification_entropy()
{
	float e,total;
	int n;

	total=0.0;
	foreach(PART,part,scene->parts) {
		e=compute_entropy(part->cvector);
		total+=e;
	}
	endforeach

	n=list_length(scene->parts);
	total /= (float)n;

	printf("Average entropy of classification vectors = %f\n",total);
}

/* --------------------------------------------------------------------
 * Procedure: print_classification_vector
 * Comment:   
 * -------------------------------------------------------------------- */

void print_classification_vector(float *class)
{
	int i;

	printf("[");
	for (i=0;i<nclasses;i++) {
		if (class[i] > 0.95)
			printf(" 1.000000");
		else if (class[i] >= 0.000001)
			printf(" %8.6f",class[i]);
		else
			printf(" --------");
	}
	printf("]");
}

/* --------------------------------------------------------------------
 * Procedure: print_model_parts
 * Comments:  
 * -------------------------------------------------------------------- */

void print_model_parts()
{
	int i,j,n;

	foreach(PART,part,scene->parts) {
		i=0;
		foreach(SNAKE,snake,part->snakes) {
			printf("Part %2d Snake %2d ",part->name,++i);
			print_path(snake->path);
			printf("  ");
			n=snake->path->length;
			for (j=0;j<n;j++) {
				printf("%d=(",snake->path->part[j]->name);
				foreach(PART,model,snake->modelparts[j])
					printf(" %d",model->name);
				endforeach
				printf(" ) ");
			}
			printf("\n");
		}
		endforeach
	}
	endforeach
}

/* --------------------------------------------------------------------
 * Procedure: print_part_classification
 * Comment:   
 * -------------------------------------------------------------------- */

void print_part_classification()
{
	foreach(PART,part,scene->parts) {
		printf("Part %2d: ",part->name);
		print_classification_vector(part->cvector);
		printf(" %d snakes",list_length(part->snakes));
		if (classification_known)
			printf(", true class = %d",part->true_classification);
		printf("\n");

		if (verbosity>1) {
			foreach(SNAKE,snake,part->snakes) {
				printf("         ");
				print_classification_vector(snake->cvector);
				printf(" ");
				print_path(snake->path);
				printf(" %d classifications",
					   list_length(snake->classifications));
				printf("\n");
				if (list_length(snake->classifications) > 0) {
					foreach(CLASSIFICATION,c,snake->classifications) {
						printf("         ");
						print_classification_vector(c->cvector);
						printf("\n");
					}
					endforeach
				}
			}
			endforeach
		}
	}
	endforeach
}

void print_final_decision()
{
	foreach(PART,part,scene->parts) {
		float *cvector, max=0.0;
		int i, index=0;

		for (i=0; i<nclasses; i++)
		{
			if (part->cvector[i]>max)
			{
				max = part->cvector[i];
				index = i;
			}
		}
		for (i=0; i<nclasses; i++)
			part->cvector[i] = 0.0;
		part->cvector[index] = 1.0;
		printf("Part %2d: ",part->name);
		print_classification_vector(part->cvector);
		printf(" %d snakes",list_length(part->snakes));
		if (classification_known)
			printf(", true class = %d",part->true_classification);
		printf("\n");
	}
	endforeach;
}

/* --------------------------------------------------------------------
 * Procedure: print_part_list
 * Comment:   
 * -------------------------------------------------------------------- */

void print_part_list(LIST parts)
{
	printf("(");
	foreach(PART,part,parts)
		printf(" %d",part->name);
	endforeach
	printf(" )");
}

/* --------------------------------------------------------------------
 * Procedure: print_path
 * Comments:  
 * -------------------------------------------------------------------- */

void print_path(PATH *path)
{
	int i;

	printf("(");
	for (i=0;i<path->length;i++)
		printf(" %d",path->part[i]->name);
	printf(" )");
}

void print_path2(PATH *path)
{
	int i;

	printf("(");
	for (i=0;i<path->length;i++)
		printf(" %d",path->part[i]->name);
	printf(" )\n");
}

/* --------------------------------------------------------------------
 * Procedure: print_snakes
 * Comments:  
 * -------------------------------------------------------------------- */

void print_snakes()
{
	printf("\nSnake permutations\n");
	foreach(PART,part,scene->parts) {
		foreach(SNAKE,snake,part->snakes) {
			if (!list_empty(snake->permutations)) {
				printf("Part %d ",part->name);
				print_path(snake->path);
				printf(" -> ");
				foreach(SNAKE,s,snake->permutations) {
					print_path(s->path);
					printf(" ");
				}
				printf("\n");
				endforeach
			}
		}
		endforeach
	}
	endforeach
}

/* --------------------------------------------------------------------
 * Procedure: print_cluster
 * Comments:  print everything known about a cluster (for debugging).
 * -------------------------------------------------------------------- */

void print_cluster(CLUSTER *c)
{
    int i, ll=0;

    printf("\nCluster %d Level:%2d", c->id,c->level);
	printf(" %s",c->name);
	if (c->cfs_possible==0) printf(" (-)");
	else if (c->cfs_possible==1) printf(" (+)");
	printf("\n");
    if (c->level>0) {
/*         printf("Rule:  "); */
/*         for (i=0;i<c->rule->nfeatures;i++) { */
/*             printf("[%6f] ",(c->mean[i]*unary_sdev[i])+unary_mean[i]); */
/*         } */
/*         printf("\n"); */

		printf("Paths: ");
		ll=0;
        foreach(PATH,path,c->paths) {
			printf("( ");
			for (i=0;i<path->length;i++)
				printf("%d ",path->part[i]->name);
			printf(") ");
			ll += path->length;
			if (ll>80)
			{
				printf("\n");
				ll=0;
			}
        }
        endforeach
		printf("\n");
		printf("Classes: ");
		for (i=0;i<nclasses;i++) {
			if (c->class[i] > 0) printf(" %d", i);
		}
		printf("\n");
	}
}

void print_cluster_tree(CLUSTER *root)
{
	print_cluster(root);
	foreach (CLUSTER, c, root->children)
		print_cluster_tree(c);
	endforeach;
}
