/* ----------------------------------------------------------------------
 * Function prototypes
 * ---------------------------------------------------------------------- */

/* sure */

int main(int argc, char **argv);
void process_options(int argc, char **argv);
void usage();
void read_training();
void read_scene();
void read_classification();
void check_labels(float **label, int nlabel, int nbinary,
				  int npart, int class, int view);
void generate_classified_regions(LIST parts, IPRS_REGION_IMAGE *regs,
								 int class_number);


/* class */

void classify_scene();
void compute_snakes(PART *part);
SNAKE *create_snake(PATH *path);
LIST expand_snake(SNAKE *snake);
CLASSIFICATION *create_classification(PATH *path,CLUSTER *cluster, 
									  LIST fuzzy_cfn, float current_fuzzy);
void classification_free(CLASSIFICATION *c);
PATH *create_path(PART *part);
LIST expand_path(PATH *path);
float part_classification_from_snakes(PART *part);
void part_classification_from_weighted_snakes(PART *part);
void snake_classification_from_rules(SNAKE *snake);
void classification_summary();
void snake_list_union(SNAKE *snake,LIST slist);
void compute_touched_snakes(PART *part);
void snake_free(SNAKE *snake);
PATH *path_copy(PATH *orig);

/* sureinter */

void snake_permutation_constraint();
void compute_model_parts();
void compute_permutation_snakes();
int intersect_model_parts(SNAKE *snake1, SNAKE *snake2);
int constrain_model_parts(SNAKE *snake1, SNAKE *snake2);
void delete_inconsistent_snakes();
void snake_classification_from_modelparts(SNAKE *snake);
int single_classification_constraint();
void analyze_intersnake_compatibility();
float snake_compatibility(SNAKE *snake1, SNAKE *snake2);
LIST snake_intersection(SNAKE *snake1,SNAKE *snake2);
float part_snake_compatibility(PART *part,SNAKE *snake1,SNAKE *snake2);
void relaxation_labeling();
float intrasnake_compatibility(SNAKE *snake);
void brendans_snake_compatibility(SNAKE *snake, float *cvector);

/* sureprint */

void print_classification_entropy();
void print_classification_vector(float *class);
void print_model_parts();
void print_part_classification();
void print_part_list(LIST parts);
void print_path(PATH *path);
void print_snakes();
void print_cluster(CLUSTER *c);
void print_final_decsion();

/* surevaria */

float compute_entropy(float *class);
int equal_path(PATH *path1, PATH *path2);
void error_exit(char *message);
int feature_in_rule(float *value, RULE *rule);
int memory_used();
int part_in_path(PART *part,PATH *path);
int path_permutation(PATH *path1, PATH *path2);
float timer();

/* bren_functions */
IPRS_FEATURE_SPACE *bren_removefeatures_featurespace(IPRS_FEATURE_SPACE *src,
													 int *usefeatures,
													 int num_newfeatures);
LIST bren_sure_add_best_clusters(CLUSTER *parent, SNAKE *snake,
								 LIST fuzzy_cfn, char type,
								 int nclusters, float fuzzyq);
#define bren_free(ptr, free_function) \
            {if (ptr) {free_function(ptr); ptr=NULL;}}
void bren_final_classification(SCENE *scene, int use_n_classifications, 
							   int nclasses, char *view_file_name,
							   char *region_file_name, 
							   char *part_file_name);

/* extern declarations */

extern float fuzzyq;
extern int fuzzy_nclusters;

unsigned long bren_getusage();

#include "sure.proto"
