/* ====================================================================
 * Packages:		
 * ==================================================================== */

/* Include files */

#include <iprs.h>
#include "sure.h"
#include "sureextern.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

/* --------------------------------------------------------------------
 * Procedure: compute_entropy
 * Comments:
 * -------------------------------------------------------------------- */

float compute_entropy(float *class)
{
    int i;
    float e;

	e=0.0;
    for (i=0;i<nclasses;i++) {
        if (class[i]>0.0) {
            e -= class[i]*logf(class[i]);
        }
    }
    return e;
}

/* --------------------------------------------------------------------
 * Procedure: equal_path
 * Comments:
 * -------------------------------------------------------------------- */

int equal_path(PATH *path1, PATH *path2)
{
    int i;

    if (path1->length != path2->length) return 0;
    for (i=0;i<path1->length;i++) {
        if (path1->part[i] != path2->part[i]) return 0;
    }
    return 1;
}

/* --------------------------------------------------------------------
 * Procedure: error_exit
 * Comments:  print error message and exit
 * -------------------------------------------------------------------- */

void error_exit(char *message)
{
	fprintf(stderr,"%s\n",message);
	exit(1);
}

/* --------------------------------------------------------------------
 * Procedure: feature_in_rule
 * Comments: does feature value fall within the rule hyper-rectangle?
 * -------------------------------------------------------------------- */

int feature_in_rule(float *value, RULE *rule)
{
	int i;
	for (i=0;i<rule->nfeatures;i++) {
		if (value[i] < rule->lower[i]) return(0);
		if (value[i] > rule->upper[i]) return(0);
	}
	return(1);
}

/* --------------------------------------------------------------------
 * Procedure: memory_used
 * Purpose:   return memory used in bytes
 * Comment:   SGI specific
 * -------------------------------------------------------------------- */

int memory_used()
{
	char path[100];
	pid_t pid;
	struct stat *s;

	s=(struct stat *)iprs_malloc(sizeof(struct stat));

	pid=getpid();
	sprintf(path,"/debug/%-d",pid);
	stat(path,s);
	return((int)(s->st_size));
}

/* --------------------------------------------------------------------
 * Procedure: part_in_path
 * Comment:
 * -------------------------------------------------------------------- */

int part_in_path(PART *part,PATH *path)
{
    int i,length;

    length=path->length;
    for (i=0;i<length;i++) {
        if (part==path->part[i]) return 1;
    }
    return 0;
}

/* --------------------------------------------------------------------
 * Procedure: path_permutation
 * Comments:
 * -------------------------------------------------------------------- */

int path_permutation(PATH *path1, PATH *path2)
{
    int i,j,found;

    if (path1->length != path2->length) return 0;
    for (i=0;i<path1->length;i++) {
        found=0;
        for (j=0;j<path2->length;j++) {
            if (path1->part[i] == path2->part[j]) {
                found=1;
                break;
            }
        }
        if (!found) return 0;
    }
    return 1;
}

/* --------------------------------------------------------------------
 * Procedure: timer
 * Purpose:   return interval (in secs) since the last call to timer.
 * Comments:  Results for intervals longer than 1 year are incorrect.
 *            (365 days = 31536000 seconds)
 * -------------------------------------------------------------------- */

float timer()
{
        struct itimerval old, new;
        float v;

        new.it_value.tv_sec     = 31536000L;
        new.it_value.tv_usec    =         0;
        new.it_interval.tv_sec  =         0;
        new.it_interval.tv_usec =         0;
        setitimer(ITIMER_VIRTUAL,&new,&old);
        v = (float)(31536000L-old.it_value.tv_sec)-
            (float)old.it_value.tv_usec*1.e-6;
        return v;
}
