/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:             iprs_ccopy_edge.c                               *****
*****   AUTHOR:           brendan (mccane@coral.cs.jcu.edu.au)            *****
*****   DATE:             4/3/93                                          *****
*****   COMMENTS:         For copying an edge.                            *****
*****   MODIFICATIONS:                                                    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/


#include "iprs.h"

IPRS_EDGE_IMAGE *bren_create_edgeimage(int num)
{
	IPRS_EDGE_IMAGE *eptr;

	iprs_debug_enter("bren_create_edgeimage");
	eptr=(IPRS_EDGE_IMAGE *)iprs_calloc(1,sizeof(IPRS_EDGE_IMAGE));
	eptr->numedges=num;
	eptr->parameter=(char *)NULL;
	eptr->edge=(IPRS_EDGE **)iprs_calloc(num,sizeof(IPRS_EDGE*));
	iprs_debug_exit();
	return(eptr);
}

IPRS_EDGE *iprs_create_edge(int numpixels)
{
	IPRS_EDGE *new_edge;

	iprs_debug_enter("iprs_create_edge");

	new_edge = (IPRS_EDGE *)iprs_calloc(1, sizeof(IPRS_EDGE));
	new_edge->numpixels = numpixels;
	if (numpixels>0)
	{
		new_edge->x = (int*)iprs_calloc(numpixels, sizeof(int));
		new_edge->y = (int*)iprs_calloc(numpixels, sizeof(int));
	}
	else
	{
		new_edge->x = NULL;
		new_edge->y = NULL;
	}

	iprs_debug_exit();
	return(new_edge);
}

IPRS_EDGE_IMAGE *iprs_create_edgeimage(int num)
{
	IPRS_EDGE_IMAGE *eptr;
	int i;

	iprs_debug_enter("iprs_create_edgeimage");
	eptr=(IPRS_EDGE_IMAGE *)iprs_calloc(1,sizeof(IPRS_EDGE_IMAGE));
	eptr->numedges=num;
	eptr->parameter=(char *)NULL;
	if(num) {
		eptr->edge=(IPRS_EDGE **)iprs_calloc(num,sizeof(IPRS_EDGE *));
		for(i=0;i<num;i++) {
			eptr->edge[i]=(IPRS_EDGE *)iprs_calloc(1,sizeof(IPRS_EDGE));
		}
	} else {
		eptr->edge=(IPRS_EDGE **)NULL;
	}
	iprs_debug_exit();
	return(eptr);
}


BREN_FLOAT_EDGE *bren_create_floatedge(int numpixels)
{
	BREN_FLOAT_EDGE *new_edge;

	iprs_debug_enter("iprs_create_edge");

	new_edge = (BREN_FLOAT_EDGE *)iprs_calloc(1, sizeof(BREN_FLOAT_EDGE));
	new_edge->numpixels = numpixels;
	new_edge->x = (float*)iprs_calloc(numpixels, sizeof(float));
	new_edge->y = (float*)iprs_calloc(numpixels, sizeof(float));

	iprs_debug_exit();
	return(new_edge);
}

IPRS_EDGE *iprs_ccopy_edge(IPRS_EDGE *edge)
{
	IPRS_EDGE *new_edge;
	int i;

	iprs_debug_enter("iprs_ccopy_edge");

	new_edge = iprs_create_edge(edge->numpixels);
	new_edge->sig = edge->sig;

	for (i=0; i<edge->numpixels; i++)
	{
		new_edge->x[i] = edge->x[i];
		new_edge->y[i] = edge->y[i];
	}

	iprs_debug_exit();
	return(new_edge);
}

BREN_FLOAT_EDGE *bren_ccopy_intfloatedge(IPRS_EDGE *edge)
{
	BREN_FLOAT_EDGE *new_edge;
	int i;

	iprs_debug_enter("iprs_ccopy_edge");

	new_edge = bren_create_floatedge(edge->numpixels);

	for (i=0; i<edge->numpixels; i++)
	{
		new_edge->x[i] = (float)edge->x[i];
		new_edge->y[i] = (float)edge->y[i];
	}

	iprs_debug_exit();
	return(new_edge);
}

IPRS_EDGE_IMAGE *bren_ccopy_edgeimage(IPRS_EDGE_IMAGE *ed)
{
	IPRS_EDGE_IMAGE *new_ed;
	int i;

	iprs_debug_enter("iprs_ccopy_edgeimage");

	new_ed = bren_create_edgeimage(ed->numedges);

	for (i=0; i<ed->numedges; i++)
		new_ed->edge[i] = iprs_ccopy_edge(ed->edge[i]);

	iprs_debug_exit();
	return(new_ed);
}

IPRS_EDGE_IMAGE *bren_ccopy_edgeimage_range(IPRS_EDGE_IMAGE *ed,int low,
											int high)
{
	IPRS_EDGE_IMAGE *new_ed;
	int i, index=0;

	iprs_debug_enter("iprs_ccopy_edgeimage");

	new_ed = bren_create_edgeimage(high-low);

	for (i=low; i<high; i++)
		new_ed->edge[index++] = iprs_ccopy_edge(ed->edge[i]);

	iprs_debug_exit();
	return(new_ed);
}


void iprs_free_edge(IPRS_EDGE *edge)
{
	iprs_debug_enter("iprs_free_edge");

	if (!edge)
	{
		fprintf(stderr, "IPRS_NULL_IMAGE in iprs_free_edge\n");
        return;
    }

	bren_free(edge->x, free);
	bren_free(edge->y, free);
	bren_free(edge, free);

	iprs_debug_exit();
}

void iprs_free_edgeimage(IPRS_EDGE_IMAGE *eptr)
{
	int i;

	iprs_debug_enter("iprs_free_edgeimage");
	if(eptr->parameter) iprs_free(eptr->parameter);
	if(eptr->edge) {
		for(i=0;i<(eptr->numedges);i++) {
			if(eptr->edge[i]) {
				if(eptr->edge[i]->x) iprs_free(eptr->edge[i]->x);
				if(eptr->edge[i]->y) iprs_free(eptr->edge[i]->y);
				iprs_free(eptr->edge[i]);
			}
		}
		iprs_free(eptr->edge);
	}
	iprs_free(eptr);
	iprs_debug_exit();
}


void bren_print_edge(FILE *fp, IPRS_EDGE *edge)
{
	int j;

	iprs_debug_enter("bren_print_edge");
	if (edge)
	{
		fprintf(fp, "edge->numpixels=%d, sig=%f\n", edge->numpixels, edge->sig);
		for (j=0; j<edge->numpixels; j++)
		{
			fprintf(fp, "x,y[%d] = %d, %d\n", j, edge->x[j], 255-edge->y[j]);
		}
	}
	iprs_debug_exit();
}

void bren_print_floatedge(FILE *fp, BREN_FLOAT_EDGE *edge)
{
	int j;

	iprs_debug_enter("bren_print_floatedge");
	fprintf(fp, "edge->numpixels=%d\n", edge->numpixels);
	for (j=0; j<edge->numpixels; j++)
	{
		fprintf(fp, "x,y = %f, %f\n", edge->x[j], 255-edge->y[j]);
	}
	iprs_debug_exit();
}

void bren_print_edgeimage(FILE *fp, IPRS_EDGE_IMAGE *edges)
{
	int i;

	iprs_debug_enter("bren_print_edgeimage");
	fprintf(fp, "number of edges = %d\n", edges->numedges);
	for (i=0; i<edges->numedges; i++)
	{
		fprintf(fp, "edge[%d]->numpixels = %d\n", i, edges->edge[i]->numpixels);
		bren_print_edge(fp, edges->edge[i]);
	}
	iprs_debug_exit();
}

void bren_splitedgeimage_tointimage(IPRS_EDGE_IMAGE *edges, IPRS_INT_IMAGE *im)
{
	int i, j, width, height;
	IPRS_EDGE *scan;
	int *map;

	width = iprs_getwidth_intimage(im);
	height = iprs_getheight_intimage(im);
	map = im->image;

	for (i=0; i<edges->numedges; i++)
	{
		scan = edges->edge[i];
		if ((scan)&&(scan->numpixels>0))
		for (j=0; j<scan->numpixels; j++)
		{
			if ((scan->x[j]<width)&&(scan->y[j]<height))
				map[scan->x[j]+scan->y[j]*width] = i+3;	
		}
	}
}

void bren_edgeimage_tointimage(IPRS_EDGE_IMAGE *edges, IPRS_INT_IMAGE *im)
{
	int i, j, width, height;
	IPRS_EDGE *scan;
	int *map;

	width = iprs_getwidth_intimage(im);
	height = iprs_getheight_intimage(im);
	map = im->image;

	for (i=0; i<edges->numedges; i++)
	{
		scan = edges->edge[i];
		if ((scan)&&(scan->numpixels>0))
		for (j=0; j<scan->numpixels; j++)
		{
			if ((scan->x[j]<width)&&(scan->y[j]<height))
				map[scan->x[j]+scan->y[j]*width] = i+3;	
		}
	}
}
			

void bren_edgeimage_toimage(IPRS_EDGE_IMAGE *edges, IPRS_INT_IMAGE *im)
{
	int i, j, width, height;
	IPRS_EDGE *scan;
	int *map;

	width = iprs_getwidth_intimage(im);
	height = iprs_getheight_intimage(im);
	map = im->image;

	for (i=0; i<edges->numedges; i++)
	{
		scan = edges->edge[i];
		if ((scan)&&(scan->numpixels>0))
		{
			for (j=1; j<scan->numpixels-1; j++)
				if ((scan->x[j]<width)&&(scan->y[j]<height))
					map[scan->x[j]+scan->y[j]*width] = 60;	
			if ((scan->x[0]<width)&&(scan->y[0]<height))
				map[scan->x[0]+scan->y[0]*width] = 127;
			if ((scan->x[scan->numpixels-1]<width)&&
				(scan->y[scan->numpixels-1]<height))
				map[scan->x[scan->numpixels-1]+
					scan->y[scan->numpixels-1]*width] = 127;
		}
	}
}

void bren_edgeimage_tobinimage(IPRS_EDGE_IMAGE *edges, IPRS_INT_IMAGE *im)
{
	int i, j, width, height;
	IPRS_EDGE *scan;
	int *map;

	width = iprs_getwidth_intimage(im);
	height = iprs_getheight_intimage(im);
	map = im->image;

	for (i=0; i<edges->numedges; i++)
	{
		scan = edges->edge[i];
		if ((scan)&&(scan->numpixels>0))
		{
			for (j=0; j<scan->numpixels; j++)
				if ((scan->x[j]<width)&&(scan->y[j]<height)&&
					(scan->x[j]>=0)&&(scan->y[j]>=0))
					map[scan->x[j]+scan->y[j]*width] = 1;	
		}
	}
}

void bren_highlight_edge(IPRS_EDGE *edge, IPRS_INT_IMAGE *im, int value)
{
	int i, width, height;

	if (!edge) return;

	width = iprs_getwidth_intimage(im);
	height = iprs_getheight_intimage(im);

	for (i=0; i<edge->numpixels; i++)
		if ((edge->x[i]>=0)&&(edge->x[i]<width)&&
			(edge->y[i]>=0)&&(edge->y[i]<height))
			im->image[edge->x[i]+edge->y[i]*width] = value;
}

void bren_addedge_edgeimage(IPRS_EDGE_IMAGE *edges, IPRS_EDGE *new_edge)
{
	iprs_debug_enter("bren_addedge_edgeimage");

	edges->edge = (IPRS_EDGE **)iprs_realloc(edges->edge, 
								(edges->numedges+1)*sizeof(IPRS_EDGE*));

	edges->numedges++;

	edges->edge[edges->numedges-1] = new_edge;

	iprs_debug_exit();
}

void bren_append_edgeimage(IPRS_EDGE_IMAGE *orig, IPRS_EDGE_IMAGE *new)
{
	int i;

	iprs_debug_enter("bren_append_edgeimage");

	for (i=0;i<new->numedges;i++)
		bren_addedge_edgeimage(orig, iprs_ccopy_edge(new->edge[i]));

	iprs_debug_exit();
}

/*
 * Copy part of the src edge to part of the dest edge.
 *
 * PARAMETERS:
 *				src       - src edge
 *				src_start - starting index for src
 *				src_end   - ending index for src
 *				src_inc   - increment for src (usually 1 or -1)
 *				dest      - destination edge
 *				dest_ind  - starting position of dest. At completion
 *							contains current position.
 *				dest_inc  - increment for dest edge (usually 1 or -1).
 */	
void bren_partial_edgecopy(IPRS_EDGE *src, int src_start, int src_end,
						   int src_inc, IPRS_EDGE *dest, int *dest_ind,
						   int dest_inc)
{
	int i;

	for (i=src_start; i*src_inc<src_end*src_inc; i+=src_inc)
	{
		if ((i<0)||(i>=src->numpixels)) break;
		if ((*dest_ind<0)||(*dest_ind>=dest->numpixels)) break;
		dest->x[*dest_ind] = src->x[i];
		dest->y[*dest_ind] = src->y[i];
		(*dest_ind) += dest_inc;
	}
}

/* Save and load functions */

void bren_siprs_edge(IPRS_EDGE *edge, IPRS_FILE *out)
{
	int np=0;
	if (!edge) iprs_write_int(out, &np, 1);
	else
	{
		iprs_write_int(out,&(edge->numpixels),1);
		if (edge->numpixels)
		{
			iprs_write_int(out, edge->x, edge->numpixels);
			iprs_write_int(out, edge->y, edge->numpixels);
		}
	}
}

IPRS_EDGE *bren_liprs_edge(IPRS_FILE *in)
{
	IPRS_EDGE *ret;
	int numpixels;

	iprs_read_int(in, &numpixels, 1);
	if (!numpixels) return(NULL);
	else
	{
		ret = iprs_create_edge(numpixels);
		iprs_read_int(in, ret->x, numpixels);
		iprs_read_int(in, ret->y, numpixels);
	}
	return(ret);
}

IPRS_EDGE *bren_reverse_edge(IPRS_EDGE *in)
{
	IPRS_EDGE *rev;
	int i, back=0;

	iprs_debug_enter("bren_reverse_edge");
	rev = iprs_create_edge(in->numpixels);
	for (i=in->numpixels-1; i>=0; i--)
	{
		rev->x[back] = in->x[i];
		rev->y[back++] = in->y[i];
	}
	iprs_debug_exit();
	return(rev);
}

IPRS_EDGE *bren_addpixel_edge(IPRS_EDGE *ed, int x, int y)
{
	int i;
	IPRS_EDGE *new_edge;
	if (!ed)
	{
		new_edge = iprs_create_edge(1);
		new_edge->x[0] = x;
		new_edge->y[0] = y;
	}
	else
	{
		new_edge = iprs_create_edge(ed->numpixels+1);
		for (i=0; i<ed->numpixels; i++)
		{
			new_edge->x[i] = ed->x[i];
			new_edge->y[i] = ed->y[i];
		}
		new_edge->x[i] = x;
		new_edge->y[i] = y;
	}
	return(new_edge);
}

void bren_addpixel_edge_inplace(IPRS_EDGE *ed, int x, int y)
{
	if (ed)
	{
		int i;
		char addpixel=TRUE;

		for (i=0; i<ed->numpixels; i++)
			if ((ed->x[i]==x)&&(ed->y[i]==y))
				addpixel=FALSE;
		if (addpixel)
		{
			ed->x = (int*)iprs_realloc(ed->x, (ed->numpixels+1)*sizeof(int));
			ed->y = (int*)iprs_realloc(ed->y, (ed->numpixels+1)*sizeof(int));
			ed->x[ed->numpixels] = x;
			ed->y[ed->numpixels++] = y;
		}
	}
}
