/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:             iprs_fm_regionimage.c                           *****
*****   AUTHOR:           brendan (mccane@coral.cs.jcu.edu.au)            *****
*****   DATE:             25/1/94                                         *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS:                                                    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/


/* #include "bren_fm.h" */
#include <iprs.h>
#include "make_gif.h"

void iprs_free_region(IPRS_REGION *reg)
{
	if (reg)
	{
		bren_free(reg->x, free);
		bren_free(reg->y, free);
		bren_free(reg->edge, free);
		bren_free(reg->region, free);
		bren_free(reg, free);
	}
}

IPRS_REGION_IMAGE *bren_create_regionimage(int num)
{
    IPRS_REGION_IMAGE *jptr;

    iprs_debug_enter("iprs_create_regionimage");
    jptr=(IPRS_REGION_IMAGE *)iprs_calloc(1,sizeof(IPRS_REGION_IMAGE));
    jptr->numregions=num;
    jptr->parameter=(char *)NULL;
    jptr->region=(IPRS_REGION **)iprs_calloc(num,sizeof(IPRS_REGION*));
    iprs_debug_exit();
    return(jptr);
}

IPRS_REGION *bren_create_region(int numpixels, int numregs, int numedges)
{
	IPRS_REGION *new_reg;

	iprs_debug_enter("bren_create_region");
	new_reg = (IPRS_REGION *)iprs_calloc(1, sizeof(IPRS_REGION));
	new_reg->numpixels = numpixels;
	new_reg->x = (int*)iprs_calloc(numpixels, sizeof(int));
	new_reg->y = (int*)iprs_calloc(numpixels, sizeof(int));
	new_reg->numregions = numregs;
	if (numregs>0) new_reg->region = (int*)iprs_calloc(numregs, sizeof(int));
	if (numedges>0) new_reg->edge = (int*)iprs_calloc(numedges, sizeof(int));

	iprs_debug_exit();
	return(new_reg);
}

IPRS_REGION *bren_ccopy_region(IPRS_REGION *reg, int numedges)
{
	IPRS_REGION *new_reg;
	int i;

	iprs_debug_enter("bren_ccopy_region");
	new_reg = bren_create_region(reg->numpixels, reg->numregions, numedges);
	for (i=0; i<reg->numpixels; i++)
	{
		new_reg->x[i] = reg->x[i];
		new_reg->y[i] = reg->y[i];
	}
	if (reg->region)
	{
		for (i=0; i<reg->numregions; i++)
			new_reg->region[i] = reg->region[i];
	}
	if (reg->edge)
	{
		for (i=0; i<numedges; i++)
			new_reg->edge[i] = reg->edge[i];
	}
	iprs_debug_exit();
	return(new_reg);
}

IPRS_REGION_IMAGE *bren_ccopy_regionimage(IPRS_REGION_IMAGE *src)
{
	IPRS_REGION_IMAGE *dest;
	int i;

	if (src)
	{
		dest = bren_create_regionimage(src->numregions);
		dest->numedges = src->numedges;
		for (i=0; i<src->numregions; i++)
			dest->region[i] = bren_ccopy_region(src->region[i], src->numedges);
		return(dest);
	}
	else return(NULL);
}

void bren_print_region(FILE *fp, IPRS_REGION *reg)
{
	fprintf(stdout, "numpixels=%d\n", reg->numpixels);
}

void bren_print_regionimage(FILE *fp, IPRS_REGION_IMAGE *regs)
{
	int i;

	fprintf(stdout, "numregions=%d\n", regs->numregions);
	for (i=0; i<regs->numregions; i++)
	{
		fprintf(stdout, "reg %d:\n", i);
		bren_print_region(stdout, regs->region[i]);
	}
}

int region_compare(IPRS_REGION **reg1, IPRS_REGION **reg2)
{
	if ((*reg1)->numpixels<(*reg2)->numpixels) return(1);
	else if ((*reg1)->numpixels==(*reg2)->numpixels) return(0);
	else return(-1);
}

void bren_order_regionimage(IPRS_REGION_IMAGE *regs)
{
	qsort(regs->region, regs->numregions, sizeof(IPRS_REGION*),
			(int (*)())region_compare);
}

void bren_highlight_region(IPRS_REGION *reg, IPRS_INT_IMAGE *int_regs, 
						   int value)
{
	int i, width, height;

	width = iprs_getwidth_intimage(int_regs);
	height = iprs_getheight_intimage(int_regs);
	for (i=0; i<reg->numpixels; i++)
	{
		if ((bren_in_interval(reg->x[i], 0, width-1))&&
			(bren_in_interval(reg->y[i], 0, height-1)))
			int_regs->image[reg->x[i]+reg->y[i]*width]=value;
	}
}

void bren_highlight_regionbyte(IPRS_REGION *reg, IPRS_BYTE_IMAGE *int_regs, 
						   int value)
{
	int i, width;

	width = iprs_getwidth_byteimage(int_regs);
	for (i=0; i<reg->numpixels; i++)
		int_regs->image[reg->x[i]+reg->y[i]*width]=value;
}

void bren_highlight_regionimage(IPRS_REGION_IMAGE *regs, IPRS_INT_IMAGE *intim)
{
	int i;

	for (i=0; i<regs->numregions; i++)
		bren_highlight_region(regs->region[i], intim, i+1);
}

/* void bren_regionimage_togif_sep(IPRS_REGION_IMAGE *regs, */
/* 								IPRS_EDGE_IMAGE *edges, */
/* 								IPRS_INT_IMAGE *im, */
/* 								char *base) */
/* { */
/* 	int i; */
/* 	char name[50]; */

/* 	for (i=0; i<regs->numregions; i++) */
/* 	{ */
/* 		iprs_set_intimage(im, 0); */
/* 		if (edges) bren_edgeimage_toimage(edges, im); */
/* 		bren_highlight_region(regs->region[i], im, 255); */
/* 		sprintf(name, "%s_%03d.gif", base, i); */
/* 		make_intgif(im, name); */
/* 	} */
/* } */

/* void bren_regionimage_togif_cat(IPRS_REGION_IMAGE *regs, */
/* 								IPRS_INT_IMAGE *im, */
/* 								char *base) */
/* { */
/* 	int i; */
/* 	char name[50]; */

/* 	iprs_set_intimage(im, 0); */
/* 	for (i=0; i<regs->numregions; i++) */
/* 	{ */
/* 		bren_highlight_region(regs->region[i], im, i+1); */
/* 	} */
/* 	sprintf(name, "%s.gif", base); */
/* 	make_intgif(im, name); */
/* } */

void bren_addpt_toregion(IPRS_REGION *reg, int x, int y)
{
	reg->x = (int*)iprs_realloc(reg->x, (reg->numpixels+1)*sizeof(int));
	reg->y = (int*)iprs_realloc(reg->y, (reg->numpixels+1)*sizeof(int));

	reg->x[reg->numpixels] = x;
	reg->y[reg->numpixels++] = y;
}

void bren_removenegpts_fromregion(IPRS_REGION *reg)
{
	int numvalid=0;
	int *newx, *newy;

	int i;
	for (i=0; i<reg->numpixels; i++)
		if (reg->x[i]>=0) numvalid++;
	newx = (int*)iprs_calloc(numvalid, sizeof(int));
	newy = (int*)iprs_calloc(numvalid, sizeof(int));
	numvalid=0;
	for (i=0; i<reg->numpixels; i++)
	{
		if (reg->x[i]>=0)
		{
			newx[numvalid] = reg->x[i];
			newy[numvalid++] = reg->y[i];
		}
	}
	iprs_free(reg->x);
	iprs_free(reg->y);
	reg->x = newx;
	reg->y = newy;
	reg->numpixels = numvalid;
}

void bren_removept_fromregion(IPRS_REGION *reg, int index)
{
	int i;

	for (i=index+1; i<reg->numpixels; i++)
	{
		reg->x[i-1] = reg->x[i];
		reg->y[i-1] = reg->y[i];
	}
		
	reg->x = (int*)iprs_realloc(reg->x, (reg->numpixels-1)*sizeof(int));
	reg->y = (int*)iprs_realloc(reg->y, (reg->numpixels-1)*sizeof(int));
	reg->numpixels--;
}

void bren_addregion_toregionimage(IPRS_REGION_IMAGE *regs, IPRS_REGION *reg)
{
	regs->region = (IPRS_REGION**)iprs_realloc(regs->region, 
							      (regs->numregions+1)*sizeof(IPRS_REGION*));
	regs->region[regs->numregions++] = reg;
}

float bren_mindist_pt_toregion(IPRS_REGION *reg, int x, int y)
{
	int i;
	float dist, min_dist=FLT_MAX;

	for (i=0; (i<reg->numpixels)&&(min_dist>0.0); i++)
	{
		dist = euclid_dist(reg->x[i], reg->y[i], x, y);
		if (dist<min_dist) min_dist=dist;
	}
	return(min_dist);
}

IPRS_REGION *bren_concatregionimage_toregion(IPRS_REGION_IMAGE *reg_im)
{
	IPRS_REGION *reg;
	int numpix, i, j, index=0;

	numpix = 0;
	for (i=0; i<reg_im->numregions; i++)
		numpix += reg_im->region[i]->numpixels;
	reg = bren_create_region(numpix, 0, 0);
	for (i=0; i<reg_im->numregions; i++)
	{
		IPRS_REGION *tmp=reg_im->region[i];
		for (j=0; j<tmp->numpixels; j++)
		{
			reg->x[index] = tmp->x[j];
			reg->y[index++] = tmp->y[j];
		}
	}
	return(reg);
}
