/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:             bren_fm_vector.c                                *****
*****   AUTHOR:           brendan (mccane@coral.cs.jcu.edu.au)            *****
*****   DATE:             3/10/94                                         *****
*****   COMMENTS:         Defines the vector class.                       *****
*****   MODIFICATIONS:                                                    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include <iprs.h>

BREN_VECTOR *bren_create_vector(int numels)
{
	BREN_VECTOR *ret;

	ret = (BREN_VECTOR*)iprs_calloc(1, sizeof(BREN_VECTOR));
	ret->numels = numels;
	ret->data = (float*)iprs_calloc(numels, sizeof(float));
	return(ret);
}

void bren_free_vector(BREN_VECTOR *vec)
{
	if (!vec) return;
	if (vec->data) bren_free(vec->data, free);
	bren_free(vec, free);
}

/* Performs the outer product on two vectors. Return matrix must be */
/* allocated. */
void bren_outerproduct_vector(BREN_VECTOR *vec1, BREN_VECTOR *vec2, 
							  BREN_MATRIX *ret)
{
	int i, j;

	if (vec1->numels!=vec2->numels)
	{
		fprintf(stderr, "error in bren_fm_vector.c\n");
		return;
	}
	if ((ret->numcols!=vec1->numels)||(ret->numrows!=vec1->numels))
	{
		fprintf(stderr, "error in bren_fm_vector.c\n");
		return;
	}
	for (i=0; i<vec1->numels; i++)
		for (j=0; j<vec2->numels; j++)
			ret->data[i][j] = vec1->data[i]*vec2->data[j];
}

void bren_print_vector(FILE *fp, BREN_VECTOR *vec)
{
	int i;

	for (i=0; i<vec->numels; i++)
		fprintf(fp, "%f ", vec->data[i]);
}
