#ifndef BREN_IMAGES
#define BREN_IMAGES

#define BREN_IMAGETYPE_ZCROSS		100
#define BREN_EDGETYPE_FLOAT			101
#define BREN_IMAGETYPE_SYMMAXIS		102
#define BREN_IMAGETYPE_LINK			103
#define BREN_IMAGETYPE_RIBBON		104
#define BREN_IMAGETYPE_OBJECT		105
#define BREN_IMAGETYPE_ELLIPSE      106

#define BREN_JUN_T	2			/* a T-junction */

#define XPOSYPOS 2
#define XPOSYNEG 3
#define XNEGYPOS 4
#define XNEGYNEG 5

	enum SUBIMAGES {INTENSITY, INT_EDGES, INT_REGIONS, EDGES, REGIONS,
					JUNCTIONS, LINKS, RIBBONS, WORKING, NUMSUB};
	extern int SUBIMAGETYPES[];

	typedef struct {
		int x, y;
	} POINT;

	typedef struct {
		double x, y;
	} FPOINT;

	typedef struct {
		double m, c;
	} LINE;

	typedef struct {
		FPOINT p1, p2;
	} LINE_SEGMENT;

	typedef struct
	{
		char sign;
		float gradient;
	} BREN_ZCROSS;

	typedef struct
	{
		char *parameter;
		int numdim;
		int *dim;
		BREN_ZCROSS *image;
	} BREN_ZCROSS_IMAGE;

	typedef struct
	{
		int x, y;
	} BREN_POINT;

	typedef struct
	{
		int numpixels;
		float *x, *y;
	} BREN_FLOAT_EDGE;	

	typedef struct
	{
		IPRS_EDGE *edge1, *edge2;
		int ed1_ind, ed2_ind;
		int e2_start, e2_end;
		float pt1_x, pt1_y, pt2_x, pt2_y;	/* endpoints of axis */	
		float aspect;
		float alpha;				/* angle between curves */
		float beta;					/* angle between ends joining curves */
		float skew;
		float nld;					/* normalised length difference */
		float length;
	} BREN_SYMMAXIS;

	typedef struct
	{
		int num_symms;
		BREN_SYMMAXIS **symms;
	} BREN_SYMMAXIS_IMAGE;

	typedef struct
	{
		float a, b, c, d, e;
		LINE major_axis, minor_axis;
		FPOINT extrem1, extrem2;		/* extremities of major axis */
		float confidence_factor;		/* how good a fit is the ellipse */
	} BREN_ELLIPSE;

	typedef struct
	{
		int num_ellipses;
		BREN_ELLIPSE **ellipses;
	} BREN_ELLIPSE_IMAGE;
	
	/*
	 * The link image defined below is for indexing into a junction image
	 * from an edge image. Just as a junction image indexes into which
	 * edges form the junction, a link image indexes which junctions this
	 * edge belongs to. This struct is good for determining connectivity
	 * of two edges. Strictly speaking, each edge should have a maximum
	 * of two junctions, but I made this variable in the name of 
	 * flexibility.
	 */

	typedef struct
	{
		int numjunctions;
		int *num;			/* junction number for each touching junction */
	} BREN_LINK;

	typedef struct
	{
		char *parameter;
		int numedges;
		BREN_LINK **link;
	} BREN_LINK_IMAGE;

	typedef struct _ribbon
	{
		int edge1, edge2;
		IPRS_EDGE *ed1, *ed2;
		int e2_start, e2_end;
		int *endlist1, *endlist2;
		int numedges1, numedges2;
		IPRS_EDGE *endcurve1, *endcurve2;
		float value;						/* relative value of ribbon */
		struct _ribbon_image *next_level;
	} BREN_RIBBON;

	typedef struct _ribbon_image
	{
		char *parameter;
		int numribbons;
		BREN_RIBBON **ribbon;
	} BREN_RIBBON_IMAGE;

	typedef struct
	{
		int numribbons;
		int *ribbons;
	} BREN_OBJECT;

	typedef struct
	{
		char *parameter;
		int numobjects;
		BREN_OBJECT **object;
	} BREN_OBJECT_IMAGE;

	typedef struct
	{
		int maxlabels, numlabels;
		int *labels;
	} BREN_JOIN_LABEL;

	/*
 	 * This typedef defines a conic section which can be represented in the
 	 * form: ax^2 + bxy + cy^2 + dx + ey + f = 0
 	 * It also includes a covariance matrix indicating the confidence
 	 * intervals of the estimated parameters.
 	 */
	enum {A, B, C, D, E, F};

	typedef struct
	{
		float a[6];		/* conic section ceoffs */
		float covar[6][6];
		float chisq;
	} BREN_CONIC;

    typedef struct
	{
		int numels;
		float *data;
	} BREN_VECTOR;

	typedef struct
	{
		int numrows, numcols;
		float **data;
	} BREN_MATRIX;

#endif
