/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:             bren_region_bdry.c                              *****
*****   AUTHOR:           brendan (mccane@coral.cs.jcu.edu.au)            *****
*****   DATE:             7/01/95                                         *****
*****   COMMENTS:         Extracts the bdry of a region.                  *****
*****   MODIFICATIONS:                                                    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

/* #include <bren_addfuncs.h> */
#include <iprs.h>

void bren_recover_edgesintint(IPRS_INT_IMAGE *sim, IPRS_INT_IMAGE *dim)
{
	int i,j, width, height;

	width = iprs_getwidth_intimage(sim);
	height = iprs_getheight_intimage(sim);
	for (i=0; i<width; i++)
		for (j=0; j<height; j++)
		{
			if ((sim->image[i+j*width]==0)&&((sim->image[i+1+j*width]!=0)||
											 (sim->image[i+1+(j+1)*width]!=0)||
											 (sim->image[i+1+(j-1)*width]!=0)||
											 (sim->image[i+(j+1)*width]!=0)||
											 (sim->image[i+(j-1)*width]!=0)||
											 (sim->image[i-1+(j)*width]!=0)||
											 (sim->image[i-1+(j+1)*width]!=0)||
											 (sim->image[i-1+(j-1)*width]!=0)))
				dim->image[i+j*width]=1;
		}
}

/*
 * This function performs a simple paint on a single region and returns
 * the number of pixels found in the region.
 */
int numcalls=0;
#define MAX_CALLS 60000

int bren_label_region_pf(int w, int h, int *dim, int *sim, int i, int j)
{
	int np=1;

	/* if (numcalls>w*h/2) */
	if (numcalls>MAX_CALLS)
	{
		return((int)0);
	}
	numcalls++;
    if(i<(w-1)) {
        if(sim[i+1+w*j]==sim[i+w*j]) {
            if(dim[i+1+w*j]==0) {
                dim[i+1+w*j]=dim[i+w*j];
                np += bren_label_region_pf(w,h,dim,sim,i+1,j);
            }
        }
    }
    if(i>0) {
        if(sim[i-1+w*j]==sim[i+w*j]) {
            if(dim[i-1+w*j]==0) {
                dim[i-1+w*j]=dim[i+w*j];
                np += bren_label_region_pf(w,h,dim,sim,i-1,j);
            }
        }
	}
    if(j<(h-1)) {
        if(sim[i+w*(j+1)]==sim[i+w*j]) {
            if(dim[i+w*(j+1)]==0) {
                dim[i+w*(j+1)]=dim[i+w*j];
                np += bren_label_region_pf(w,h,dim,sim,i,j+1);
            }
        }
    }
    if(j>0) {
        if(sim[i+w*(j-1)]==sim[i+w*j]) {
            if(dim[i+w*(j-1)]==0) {
                dim[i+w*(j-1)]=dim[i+w*j];
                np += bren_label_region_pf(w,h,dim,sim,i,j-1);
            }
        }
    }

	return(np);
}

/*
 * This function extracts the pixels from inside a single region using
 * a recursive paint function algorithm.
 */

void bren_extract_region_pf(IPRS_REGION *reg, int w, int h, int *sim,
							int i, int j, int *current_num)
{
	if (numcalls++>MAX_CALLS) return;

	sim[i+w*j]++;
    reg->x[*current_num] = i;
    reg->y[(*current_num)++] = j;

    if(i<(w-1)) {
        if(sim[i+1+w*j]==sim[i+w*j]-1) {
                bren_extract_region_pf(reg,w,h,sim,i+1,j, current_num);
        }
    }
    if(i>0) {
        if(sim[i-1+w*j]==sim[i+w*j]-1) {
                bren_extract_region_pf(reg,w,h,sim,i-1,j, current_num);
        }
	}
    if(j<(h-1)) {
        if(sim[i+w*(j+1)]==sim[i+w*j]-1) {
                bren_extract_region_pf(reg,w,h,sim,i,j+1, current_num);
        }
    }
    if(j>0) {
        if(sim[i+w*(j-1)]==sim[i+w*j]-1) {
                bren_extract_region_pf(reg,w,h,sim,i,j-1, current_num);
        }
    }
}


void bren_label_inverseregion_pf(int w, int h, int *dim, int *sim, int i, int j)
{
	int xstack[66000], ystack[66000];
	int stack_index=0;

	xstack[0]=i; ystack[0]=j;
	dim[i+j*h]=1;

	while (stack_index>=0)
	{
		i = xstack[stack_index];
		j = ystack[stack_index];

		stack_index--;
		if(i<(w-1)) {
			if(sim[i+1+w*j]==sim[i+w*j]) {
				if(dim[i+1+w*j]==0) {
					dim[i+1+w*j]=dim[i+w*j];
					xstack[++stack_index]=i+1;
					ystack[stack_index]=j;
				}
			}
		}
		if(i>0) {
			if(sim[i-1+w*j]==sim[i+w*j]) {
				if(dim[i-1+w*j]==0) {
					dim[i-1+w*j]=dim[i+w*j];
					xstack[++stack_index]=i-1;
					ystack[stack_index]=j;
				}
			}
		}
		if(j<(h-1)) {
			if(sim[i+w*(j+1)]==sim[i+w*j]) {
				if(dim[i+w*(j+1)]==0) {
					dim[i+w*(j+1)]=dim[i+w*j];
					xstack[++stack_index]=i;
					ystack[stack_index]=j+1;
				}
			}
		}
		if(j>0) {
			if(sim[i+w*(j-1)]==sim[i+w*j]) {
				if(dim[i+w*(j-1)]==0) {
					dim[i+w*(j-1)]=dim[i+w*j];
					xstack[++stack_index]=i;
					ystack[stack_index]=j-1;
				}
			}
		}
	}
}

void bren_reverse_intimage(IPRS_INT_IMAGE *sim, IPRS_INT_IMAGE *dim)
{
	int i, j, w, h;

	w = iprs_getwidth_intimage(sim);
	h = iprs_getheight_intimage(sim);
	for (i=0; i<w; i++)
		for (j=0; j<h; j++)
		{
			/* if sim=1 dim=0 and vice versa */
			dim->image[i+j*w] = 1-sim->image[i+j*w];
			/* this code adds one more pixel around the region bdry */
		}
}

void bren_erode_dilate(IPRS_INT_IMAGE *im)
{
	IPRS_INT_IMAGE *tmp;
	int i, j, k, l, w, h;

	w = iprs_getwidth_intimage(im);
	h = iprs_getheight_intimage(im);
	tmp = iprs_create_2dintimage(w, h);
	/* erode region */
	for (i=0; i<w; i++)
	{
		for (j=0; j<h; j++)
		{
			if (im->image[i+j*w]==1)
			{
				int cnt=0;
				for (k=-1; k<=1; k++)
					for (l=-1; l<=1; l++)
					{
						if ((k==0)&&(l==0)) continue;
						else if (im->image[i+k+(j+l)*w]==1) cnt++;
					}
				if (cnt>=8) tmp->image[i+j*w]=1;
			}
		}
	}
	iprs_set_intimage(im, 0);
	/* remove isolated pixels */
	for (i=0; i<w; i++)
	{
		for (j=0; j<h; j++)
		{
			int cnt=0;
			if (tmp->image[i+j*w]==1)
			{
				for (k=-1; k<=1; k++)
					for (l=-1; l<=1; l++)
					{
						if ((k==0)&&(l==0)) continue;
						else if (tmp->image[i+k+(j+l)*w]==1) cnt++;
					}
				if (cnt==0) tmp->image[i+j*w]=0;
			}
		}
	}
	/* dilate region */
	for (i=0; i<w; i++)
	{
		for (j=0; j<h; j++)
		{
			int cnt=0;
			for (k=-1; k<=1; k++)
				for (l=-1; l<=1; l++)
				{
					if ((k==0)&&(l==0)) continue;
					else if (tmp->image[i+k+(j+l)*w]==1) cnt++;
				}
			if (cnt>0) im->image[i+j*w]=1;
		}
	}
	for (i=0; i<w; i++)
	{
		for (j=0; j<h; j++)
		{
			int cnt=0;
			if (im->image[i+j*w]==0)
			{
				for (k=-1; k<=1; k++)
					for (l=-1; l<=1; l++)
					{
						if ((k==0)&&(l==0)) continue;
						else if (im->image[i+k+(j+l)*w]==1) cnt++;
					}
				if (cnt>=8) im->image[i+j*w]=1;
			}
		}
	}
	iprs_free_intimage(tmp);
}

void bren_remove_isolatedpixels(IPRS_INT_IMAGE *im)
{
	int i, j, k, l, w, h;
	char change=TRUE;
	IPRS_INT_IMAGE *tmp;

	w = iprs_getwidth_intimage(im);
	h = iprs_getheight_intimage(im);

/* 	while (change) */
/* 	{ */
/* 		change=IPRS_FALSE; */
		for (i=1; i<w-1; i++)
			for (j=1; j<h-1; j++)
			{
				if (im->image[i+j*w]==1)
				{
					int cnt=0;
					for (k=-1; k<=1; k++)
						for (l=-1; l<=1; l++)
						{
							if ((k==0)&&(l==0)) continue;
							else if (im->image[i+k+(j+l)*w]==1) cnt++;
						}
					if ((cnt<=1))/* ||(cnt>2)) */ 
					{
						im->image[i+j*w]=0;
						change=TRUE;
					}
				}
			}
/* 	} */
}

IPRS_EDGE *bren_region_bdry(IPRS_REGION *in_region)
{
	static IPRS_INT_IMAGE *int_regions=NULL, *int_edges=NULL;
	IPRS_INT_IMAGE *inverse;
	IPRS_BYTE_IMAGE *bimage;
	IPRS_EDGE_IMAGE *edges;
	IPRS_EDGE *ed=NULL;
	int width, height, i;
	int min_x=1000, min_y=1000, max_x=0, max_y=0;
	IPRS_REGION *reg;

	reg = bren_ccopy_region(in_region, 0);
	for (i=0; i<reg->numpixels; i++)
	{
		if (reg->x[i]<min_x) min_x = reg->x[i];
		if (reg->x[i]>max_x) max_x = reg->x[i];
		if (reg->y[i]<min_y) min_y = reg->y[i];
		if (reg->y[i]>max_y) max_y = reg->y[i];
	}
	min_x -= 3; min_y -= 3;
	max_x += 3; max_y += 3;
	for (i=0; i<reg->numpixels; i++)
	{
		reg->x[i] -= min_x;
		reg->y[i] -= min_y;
	}
	width = max_x - min_x + 1;
	height = max_y - min_y + 1;
	int_regions = iprs_create_2dintimage(width, height);
	int_edges = iprs_create_2dintimage(width, height);
	iprs_set_intimage(int_regions, 0);
	iprs_set_intimage(int_edges, 0);
	inverse = iprs_create_2dintimage(width, height);
	
	bren_highlight_region(reg, int_regions, 1);
	/* basic dilation type procedure */
	for (i=0; i<reg->numpixels; i++)
	{
		int x, y, ii, jj;
		x = reg->x[i]; y = reg->y[i];
/* 		if (x-1>=0) */
/* 			int_regions->image[x-1+(y)*width]=1; */
/* 		if (x+1<width) */
/* 			int_regions->image[x+1+(y)*width]=1; */
/* 		if (y-1>=0) */
/* 			int_regions->image[x+(y-1)*width]=1; */
/* 		if (y+1<height) */
/* 			int_regions->image[x+(y+1)*width]=1; */
		for (ii=-1; ii<=1; ii++)
			for (jj=-1; jj<=1; jj++)
			{
				if ((bren_in_interval(x+ii, 0, width-1))&&
					(bren_in_interval(y+jj, 0, height-1)))
					int_regions->image[x+ii+(y+jj)*width]=1;
			}
	}
/* 	bren_clear_boundary(int_regions, 0); */
    bren_label_inverseregion_pf(width, height, inverse->image, 
								int_regions->image, 0, 0);
	bren_reverse_intimage(inverse, int_regions);
	iprs_free_intimage(inverse);
	iprs_recover_edgesintint(int_regions, int_edges);
	bren_remove_isolatedpixels(int_edges);
	iprs_label_intedges(int_edges);
	edges = iprs_extract_intedges(int_edges);
	if (edges->numedges==0)      /* region is probably a line */
	{
		int i;
		IPRS_BYTE_IMAGE *byte;
		IPRS_INT_IMAGE *intim;
		/*
		 * If the bdry is nil, then the region is most likely a single line.
		 * Hence, return that line as the bdry.
		 */
		iprs_free_edgeimage(edges);
		byte = iprs_create_2dbyteimage(width, height);
		bren_highlight_regionbyte(reg, byte, 1);
		iprs_thin_edges(byte);
		intim = iprs_bytetoint(byte);
		iprs_recover_edgesintint(intim, int_edges);
		bren_remove_isolatedpixels(int_edges);
		iprs_label_intedges(int_edges);
		edges = iprs_extract_intedges(int_edges);
		iprs_free_intimage(intim);
		iprs_free_byteimage(byte);
	}

	if (edges->numedges>1) 
	{
		int i, max=0;

		fprintf(stderr, "Possible error in region_bdry: >1 edge\n");
		for (i=1; i<edges->numedges; i++)
		{
			if (edges->edge[i]->numpixels>edges->edge[max]->numpixels)
				max = i;
		}
		ed = iprs_ccopy_edge(edges->edge[max]);
	}
	else if (edges->numedges>0)
		ed = iprs_ccopy_edge(edges->edge[0]);
	else ed = NULL;

	if (ed) 
	{
		bren_newreorder_edge(ed);
		for (i=0; i<ed->numpixels; i++)
		{
			ed->x[i] += min_x;
			ed->y[i] += min_y;
		}
	}
	bren_free(edges, iprs_free_edgeimage);

	iprs_free_intimage(int_edges);
	iprs_free_intimage(int_regions);
	iprs_free_region(reg);
	return(ed);
}

IPRS_EDGE_IMAGE *bren_regionimage_bdry(IPRS_REGION_IMAGE *regs)
{
	IPRS_EDGE_IMAGE *edges;
	int i;

	edges = bren_create_edgeimage(regs->numregions);
	for (i=0; i<regs->numregions; i++)
		edges->edge[i] = bren_region_bdry(regs->region[i]);
	return(edges);
}

/*
   This function will only work for certain on a convex region.
*/


IPRS_REGION *bren_regionbdry_toregion(IPRS_EDGE *bdry)
{
	IPRS_INT_IMAGE *edge_im=NULL, *reg_im=NULL;
	IPRS_REGION *reg;
	float c1, c2;
	int x_start, y_start, reg_index, np;
	int width=0, height=0;

	if (!edge_im)
	{
		int i;
		for (i=0; i<bdry->numpixels; i++)
		{
			if (bdry->x[i]>width) width = bdry->x[i];
			if (bdry->y[i]>height) height = bdry->y[i];
		}
		edge_im = iprs_create_2dintimage(width, height);
		reg_im = iprs_create_2dintimage(width, height);
	}
	iprs_set_intimage(edge_im, 0);
	iprs_set_intimage(reg_im, 0);

	bren_centroid_edge(bdry, &c1, &c2);
	x_start = my_round(c1);
	y_start = my_round(c2);

	bren_highlight_edge(bdry, edge_im, 1);
	reg_im->image[x_start+y_start*width] = 1;
	numcalls=0;
	np = bren_label_region_pf(width, height, reg_im->image,
							  edge_im->image, x_start, y_start);

	reg = bren_create_region(np, 0, 0);
	reg_index=0;
	numcalls=0;
	bren_extract_region_pf(reg, width, height, reg_im->image,
						   x_start, y_start, &reg_index);
	iprs_free_intimage(edge_im);
	iprs_free_intimage(reg_im);

	return(reg);
}

IPRS_REGION *bren_regionbdry_toregion_seed(IPRS_EDGE *bdry, 
										   IPRS_REGION *orig)
{
	IPRS_INT_IMAGE *edge_im=NULL, *reg_im=NULL;
	IPRS_REGION *reg;
	float c1, c2;
	int reg_index, np;
	int x_start, y_start, i;
	int width=0, height=0;

	if (!edge_im)
	{
		int i;
		for (i=0; i<bdry->numpixels; i++)
		{
			if (bdry->x[i]>width) width = bdry->x[i];
			if (bdry->y[i]>height) height = bdry->y[i];
		}
		edge_im = iprs_create_2dintimage(width, height);
		reg_im = iprs_create_2dintimage(width, height);
	}
	iprs_set_intimage(edge_im, 0);
	iprs_set_intimage(reg_im, 0);

	bren_highlight_edge(bdry, edge_im, 1);
	for (i=0; i<orig->numpixels; i++)
	{
		if (!edge_im->image[orig->x[i]+orig->y[i]*width])
		{
			x_start = orig->x[i];
			y_start = orig->y[i];
			break;
		}
	}
	reg_im->image[x_start+y_start*width] = 1;
	numcalls=0;
	np = bren_label_region_pf(width, height, reg_im->image,
							  edge_im->image, x_start, y_start);

	reg = bren_create_region(np, 0, 0);
	reg_index=0;
	numcalls=0;
	bren_extract_region_pf(reg, width, height, reg_im->image,
						   x_start, y_start, &reg_index);

	iprs_free_intimage(edge_im);
	iprs_free_intimage(reg_im);

	return(reg);
}

/*
 * This function fills in a region (ie removes holes) by first finding its
 * bdry and then reforming the region.
 */

IPRS_REGION *bren_fill_region(IPRS_REGION *orig)
{
	IPRS_EDGE *bdry;
	IPRS_REGION *ret;

	if (orig->numpixels>50)
	{
		bdry = bren_region_bdry(orig);
		if (bdry)
		{
			int i, x_start, y_start;

			ret = bren_regionbdry_toregion_seed(bdry, orig);
			iprs_free_edge(bdry);
			return(ret);
		}
		else return(bren_ccopy_region(orig, 1));
	}
	else return(bren_ccopy_region(orig, 1));
}

IPRS_REGION_IMAGE *bren_fill_regionimage(IPRS_REGION_IMAGE *regs)
{
	int i;
	IPRS_REGION_IMAGE *new_regs;

	new_regs = bren_create_regionimage(regs->numregions);
	for (i=0; i<regs->numregions; i++)
		new_regs->region[i] = bren_fill_region(regs->region[i]);

	return(new_regs);
}
