/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_add_space.c                                    *****
*****   AUTHOR:       cdillon@cs.mu.oz.au                                 *****
*****   DATE:         05/12/91                                            *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS:                                                    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include "iprs.h"

int iprs_addfeature_featurespace(
	IPRS_FEATURE_SPACE *fsptr,
	int num)			/* number of new features */	
{
	int numpoints;
	int numfeatures,nnumfeatures;
	int i,j;
	float *op,*np;

	iprs_debug_enter("iprs_addfeature_featurespace");
	IPRS_STDERR_IMAGER(fsptr, 0);
	if(num<0) {
		fprintf(stderr, "Can only add positive number of new features");
		return 0;
	}
	numfeatures=fsptr->numfeatures;
	nnumfeatures=fsptr->numfeatures+num;
	numpoints=fsptr->numpoints;
	for(i=0;i<numpoints;i++) {
		op=fsptr->point[i];
		if(nnumfeatures) {
			np=(float *)iprs_calloc(nnumfeatures,sizeof(float));
		} else {
			np=(float *)NULL;
		}
		for(j=0;j<numfeatures;j++) {
			np[j]=op[j];
		}
		if(op) iprs_free(op);
		fsptr->point[i]=np;
	}
	fsptr->numfeatures=nnumfeatures;
	iprs_debug_exit();
	return(numfeatures); /* position of first new feature */
}

int iprs_addpoint_featurespace(
	IPRS_FEATURE_SPACE *fsptr,
	int num)
{
	int nnumpoints,numpoints,i;

	iprs_debug_enter("iprs_addpoint_featurespace");
	IPRS_STDERR_IMAGER(fsptr, 0);
	if(num<0) {
		fprintf(stderr, "Can only add a positive number of points");
		return 0;
	}
	numpoints=fsptr->numpoints;
	nnumpoints=numpoints+num;
	if(nnumpoints) {
		if(!fsptr->numpoints) {
			fsptr->point=(float **)iprs_malloc(nnumpoints*sizeof(float *));
		} else {
			fsptr->point=(float **)iprs_realloc(fsptr->point,nnumpoints*sizeof(float *));
		}
	} else {
		fsptr->point=(float **)NULL;
	}
	for(i=numpoints;i<nnumpoints;i++) {
		if(fsptr->numfeatures) {
			fsptr->point[i]=(float *)iprs_calloc(fsptr->numfeatures,sizeof(float));
		} else {
			fsptr->point[i]=(float *)NULL;
		}
	}
	fsptr->numpoints=nnumpoints;
	iprs_debug_exit();
	return(numpoints);	/* returns number of first new point */
}

int iprs_addpoint_labelspace(
	IPRS_LABEL_SPACE *lsptr,
	int num)
{
    int numpoints;
    int nnumpoints;
    int i;
    int **op,**np;
	int *oop,*nnp;

	iprs_debug_enter("iprs_addpoint_labelspace");
	IPRS_STDERR_IMAGER(lsptr, 0);
	if(num<0) {
		fprintf(stderr, "Can only add a positive number of points");
		return 0;
	}
    op=lsptr->label;
    numpoints=lsptr->numpoints;
    nnumpoints=numpoints+num;
    if(nnumpoints) {
	    np=(int **)iprs_calloc(nnumpoints,sizeof(int *));
	} else {
		np=(int **)NULL;
	}
    for(i=0;i<numpoints;i++) {
        np[i]=op[i];
    }
    if(op) iprs_free(op);
	lsptr->label=np;

	oop=lsptr->numlabels;
	if(nnumpoints) {
		nnp=(int *)iprs_calloc(nnumpoints,sizeof(int));
	} else {
		nnp=(int *)NULL;
	}
    for(i=0;i<numpoints;i++) {
        nnp[i]=oop[i];
    }
    if(oop) iprs_free(oop);
    lsptr->numlabels=nnp;

    lsptr->numpoints=nnumpoints;
    iprs_debug_exit();
    return(numpoints);
}

void iprs_addlabel_labelspace(
	IPRS_LABEL_SPACE *lsptr,
	int num,
	int label)
{
	int *op,*np;
	int i,lnum,ignore;

	iprs_debug_enter("iprs_addlabel_labelspace");
	IPRS_STDERR_IMAGE(lsptr);
	if(num>=lsptr->numpoints) {
		ignore=iprs_addpoint_labelspace(lsptr,num-lsptr->numpoints+1);
	}
	op=lsptr->label[num];
	lnum=lsptr->numlabels[num];
	np=(int *)iprs_calloc(lnum+1,sizeof(int));
	for(i=0;i<lnum;i++) {
		np[i]=op[i];
	}
	np[i]=label;
	lsptr->label[num]=np;
	lsptr->numlabels[num]=lnum+1;
	if(op) iprs_free(op);
	if(!(lsptr->ptsperlabel)) {
        lsptr->minlabel=label;
        lsptr->maxlabel=label;
        lsptr->ptsperlabel=(int *)iprs_calloc(1,sizeof(int));
        lsptr->ptsperlabel[0]=1;
    } else {
		if(label>lsptr->maxlabel) {
			np=(int *)iprs_calloc(label-lsptr->minlabel+1,sizeof(int));
			op=lsptr->ptsperlabel;
			for(i=0;i<(lsptr->maxlabel-lsptr->minlabel+1);i++) {
				np[i]=op[i];
			}
			np[label-lsptr->minlabel]=1;
			if(op) iprs_free(op);
			lsptr->ptsperlabel=np;
			lsptr->maxlabel=label;
		} else {
			if(label<lsptr->minlabel) {
				np=(int *)iprs_calloc(lsptr->maxlabel-label+1,sizeof(int));
				op=lsptr->ptsperlabel;
				for(i=0;i<(lsptr->maxlabel-lsptr->minlabel+1);i++) {
					np[i+lsptr->minlabel-label]=op[i];
				}
				np[0]=1;
				if(op) iprs_free(op);
				lsptr->ptsperlabel=np;
				lsptr->minlabel=label;
			} else {
				lsptr->ptsperlabel[label-lsptr->minlabel]++;
			}
		}
	}
	iprs_debug_exit();
}

int iprs_addrule_splitrule(
	IPRS_SPLIT_RULE *srptr,
	int num)
{
	int i,newnum;

	iprs_debug_enter("iprs_addrule_splitrule");
	IPRS_STDERR_IMAGER(srptr, 0);
	if(num<0) {
		fprintf(stderr, "Cannot add negative rules");
		return -1;
	}
	newnum=srptr->numrules+num;
	if(srptr->numrules) {
		srptr->rule=iprs_realloc(srptr->rule,newnum*sizeof(IPRS_SPLIT *));
	} else {
		srptr->rule=iprs_malloc(newnum*sizeof(IPRS_SPLIT *));
	}
	for(i=srptr->numrules;i<newnum;i++) {
		srptr->rule[i]=iprs_malloc(sizeof(IPRS_SPLIT));
	}
	srptr->numrules=newnum;
	iprs_debug_exit();
	return(newnum-num); /* position of first available new rule */
}

int iprs_addpartition_partitionspace(
	IPRS_PARTITION_SPACE *psptr,
	int num)			/* number of new partitions */	
{
	int numpartitions,nnumpartitions,i;
	IPRS_PARTITION *pp;

	iprs_debug_enter("iprs_addpartition_partitionspace");
	IPRS_STDERR_IMAGER(psptr, 0);
	if(num<0) {
		fprintf(stderr, "Can only add positive number of new partitions");
		return 0;
	}
	numpartitions=psptr->numpartitions;
	nnumpartitions=psptr->numpartitions+num;
	if(numpartitions) {
		psptr->partition=(IPRS_PARTITION **)iprs_realloc(
			psptr->partition,(nnumpartitions*sizeof(IPRS_PARTITION *)));
	} else {
		psptr->partition=(IPRS_PARTITION **)iprs_malloc(
			nnumpartitions*sizeof(IPRS_PARTITION *));
	}
	for(i=numpartitions;i<nnumpartitions;i++) {
		psptr->partition[i]=(IPRS_PARTITION *)
			iprs_malloc(sizeof(IPRS_PARTITION));
		pp=psptr->partition[i];
		if(psptr->numfeatures) {
			pp->lower=(float *)iprs_calloc(psptr->numfeatures,sizeof(float));
			pp->upper=(float *)iprs_calloc(psptr->numfeatures,sizeof(float));
		} else {
			pp->lower=NULL;
			pp->upper=NULL;
		}
		pp->time_created=0;
		pp->num_since=0;
		pp->est_created=0;
	}
	psptr->numpartitions=nnumpartitions;
	iprs_debug_exit();
	return(numpartitions);
}
