/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_extract_edges.c                                *****
*****   AUTHOR:       cdillon@uno.vis.citri.edu.au                        *****
*****   DATE:         21/09/91                                            *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS: modified for isolated edges                        *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include<iprs.h>

IPRS_EDGE_IMAGE *iprs_extract_intedges(IPRS_INT_IMAGE *iptr)
{
	IPRS_EDGE_IMAGE *eptr;
	IPRS_EDGE *ptr;
	int w,h,numedges,*im;
	int i,j;
	int *numpixels,edge;

	iprs_debug_enter("iprs_extract_intedges");
	if(iptr->numdim!=2) {
		fprintf(stderr, "error in iprs_extract_edges\n");
		return NULL;
	}
	im=iptr->image;
	w=iptr->dim[0];
	h=iptr->dim[1];

	numedges=0;
	for(i=0;i<w;i++) {
		for(j=0;j<h;j++) {
			if(im[i+w*j]>numedges) numedges=im[i+w*j];
		}
	}
	numedges-=2;	/* for junctions and edges */
	numpixels=(int *)iprs_calloc(numedges,sizeof(int));
	eptr=iprs_create_edgeimage(numedges);
	for(edge=0;edge<numedges;edge++) {
		numpixels[edge]=0;
	}

	for(i=0;i<w;i++) {
       	for(j=0;j<h;j++) {
           	if(im[i+w*j]>=3) numpixels[im[i+w*j]-3]++;
		}
	}
	for(edge=0;edge<numedges;edge++) {
		ptr=eptr->edge[edge];
		ptr->numpixels=numpixels[edge];
		ptr->x=(int *)iprs_calloc(numpixels[edge],sizeof(int));
		ptr->y=(int *)iprs_calloc(numpixels[edge],sizeof(int));
	}
	for(edge=0;edge<numedges;edge++) {
		numpixels[edge]=0;
	}
    for(i=0;i<w;i++) {
    	for(j=0;j<h;j++) {
			edge=im[i+w*j]-3;
       		if(edge>=0) {
				ptr=eptr->edge[edge];
				ptr->x[numpixels[edge]]=i;
				ptr->y[numpixels[edge]]=j;
				numpixels[edge]++;
            }
        }
	}
	iprs_free(numpixels);
	iprs_debug_exit();
	return(eptr);
}

IPRS_EDGE_IMAGE *iprs_extract_edges(IPRS_IMAGE *ptr)
{
	IPRS_EDGE_IMAGE *eptr;

	iprs_debug_enter("iprs_extract_edges");
	switch(ptr->type[0]) {
		case IPRS_IMAGETYPE_INT:
			eptr=iprs_extract_intedges(ptr->image[0]);
			break;

		default:
			fprintf(stderr, "error in iprs_extract_edges\n");
			return;
	}
	iprs_debug_exit();
	return(eptr);
}
