/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_extract_regions.c                              *****
*****   AUTHOR:       cdillon@uno.vis.citri.edu.au                        *****
*****   DATE:         21/09/91                                            *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS: modified for isolated edges                        *****
*****                  and rewritten 30/09/91                             *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include<iprs.h>

IPRS_REGION_IMAGE *iprs_extract_intregions(IPRS_INT_IMAGE *iptr)
{
	int i,j,num,cr;
	int *xptr,*yptr;
	IPRS_REGION_IMAGE *rptr;
	IPRS_REGION *ptr;
	int w,h,*im,numregions;

	iprs_debug_enter("iprs_extract_regions");
	if(!iptr) fprintf(stderr, "error in iprs_extract_regions.c\n");
	w=iptr->dim[0];
	h=iptr->dim[1];
	im=iptr->image;
	if(!im) fprintf(stderr, "error in iprs_extract_regions.c\n");

	numregions=0;
	for(i=0;i<w;i++) {
		for(j=0;j<h;j++) {
			if(im[i+w*j]>numregions) numregions=im[i+w*j];
		}
	}		/* don't change numregions as region map is one larger anyway */
	printf("numregions=%d\n", numregions);

	rptr=iprs_create_regionimage(numregions);
	for(cr=0;cr<numregions;cr++) {
		ptr=rptr->region[cr];
		num=0;
		for(i=0;i<w;i++) {
			for(j=0;j<h;j++) {
				if(im[i+w*j]==(cr+1)) num++;
			}
		}
		xptr=(int *)iprs_calloc(num,sizeof(int));
		yptr=(int *)iprs_calloc(num,sizeof(int));
		ptr->x=xptr;
		ptr->y=yptr;	
		ptr->numpixels=num;
		num=0;
        for(i=0;i<w;i++) {
            for(j=0;j<h;j++) {
                if(im[i+w*j]==(cr+1)) {
					xptr[num]=i;
					yptr[num]=j;
					num++;
				}
            }
        }
	}
	iprs_debug_exit();
	return(rptr);
}

IPRS_REGION_IMAGE *iprs_extract_regions(IPRS_IMAGE *ptr)
{
	IPRS_REGION_IMAGE *rptr;

	iprs_debug_enter("iprs_extract_regions");
	if(!ptr) {
		fprintf(stderr, "error in iprs_extract_regions.c\n");
		return;
	}
	switch(ptr->type[0]) {
		case IPRS_IMAGETYPE_INT:
			rptr=iprs_extract_intregions(ptr->image[0]);
			break;

		default:
			fprintf(stderr, "error in iprs_extract_regions.c\n");
			return NULL;
	}
	iprs_debug_exit();
	return(rptr);
}

