/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:          iprs_fm.h                                          *****
*****   AUTHOR:        cdillon@uno.vis.citri.edu.au                       *****
*****   DATE:          27/12/91                                           *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS:                                                    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#define IPRS_FILETYPE_UNKNOWN	0
#define IPRS_FILETYPE_IPRS		1
#define IPRS_FILETYPE_TEXT		2
#define IPRS_FILETYPE_GIF		3
#define IPRS_FILETYPE_SGI		4	/* note no magic string */
#define IPRS_FILETYPE_TIFF		5	
#define IPRS_FILETYPE_AE		6	/* Arithmetic Encoding */

#define IPRS_HEADER "IPRS"		/* NOTE: lengths needed in iprs_header.c */
#define TEXT_HEADER "TEXT"
#define GIF_HEADER "GIF"
#define TIFF_HEADER1 "MM"
#define TIFF_HEADER2 "II"
#define AE_HEADER "IPAE"

#define IPRS_FILE FILE
#define IPRS_STDIN stdin
#define IPRS_STDOUT stdout
#define IPRS_STDERR stderr

#include "iprs_fm.proto"

IPRS_IMAGE *iprs_load_gifimage(char *fname);
IPRS_IMAGE *iprs_load_mrsimage(char *fn);
IPRS_IMAGE *iprs_load_sgiimage(char *fn);
IPRS_IMAGE *iprs_load_tiffimage(char *fname);

#ifdef BIG_ENDIAN

void iprs_read_bigendian(FILE *,void *,unsigned);

#define iprs_write_char(a,b,c)  fwrite((b),sizeof(char),(c),(a))
#define iprs_write_int(a,b,c)   fwrite((b),sizeof(int),(c),(a))
#define iprs_write_float(a,b,c) fwrite((b),sizeof(float),(c),(a))

#define iprs_read_char(a,b,c)   iprs_read_bigendian((a),(b),(c))
#define iprs_read_int(a,b,c)    iprs_read_bigendian((a),(b),(c)*sizeof(int))
#define iprs_read_float(a,b,c)  iprs_read_bigendian((a),(b),(c)*sizeof(float))

#endif

#ifdef LITTLE_ENDIAN

#define iprs_write_char(a,b,c)  fwrite((b),sizeof(char),(c),(a))
void iprs_write_int(FILE *,int *,unsigned);
void iprs_write_float(FILE *,float *,unsigned);

void iprs_read_char(FILE *,char *,unsigned);
void iprs_read_int(FILE *,int *,unsigned);
void iprs_read_float(FILE *,float *,unsigned);

#endif

/* This is temporary until people use the right functions */
#define iprs_create_2dbooleanimage iprs_create_2Dbooleanimage
#define iprs_create_2dbyteimage iprs_create_2Dbyteimage
#define iprs_create_2dcompleximage iprs_create_2Dcompleximage
#define iprs_create_2dfloatimage iprs_create_2Dfloatimage
#define iprs_create_2dintimage iprs_create_2Dintimage
#define iprs_create_2drgbimage iprs_create_2Drgbimage
#define iprs_create_2drleimage iprs_create_2Drleimage
#define iprs_create_2dsegmentimage iprs_create_2Dsegmentimage
#define iprs_create_a2dimage iprs_create_a2Dimage

/* Global Variables */

extern int iprs_gv_maxsubimages;	/* maximum subimages before auto-concat */
#define IPRS_GV_MAXSUBIMAGES_DEF 256

/****************************************************************************

The following is the file version information - version changes are listed
below.

0 	< 28/03/93	This is the "before-file-versions" version, and the default
				is somehow the file version information is removed (or not
				present - if you have an old file being read by the new IPRS)

1	28/03/93	The old label space format was changed so as to not save
				the points per label information which is now recomputed
				during loading. This saves disk space as well as headache
				when entering data in TEXT mode. 

****************************************************************************/

#define IPRS_FILE_VERSION	1
