/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_fm_featurespace.c                              *****
*****   AUTHOR:       cdillon@cs.mu.oz.au                                 *****
*****   DATE:         05/09/91                                            *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS: 23/03/93 file versions                             *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include "iprs.h"

IPRS_FEATURE_SPACE *iprs_create_featurespace(int numf,int nump)
{
	IPRS_FEATURE_SPACE *sptr;
	int i;

	iprs_debug_enter("iprs_create_featurespace");
	sptr=(IPRS_FEATURE_SPACE *)iprs_calloc(1,sizeof(IPRS_FEATURE_SPACE));
	sptr->numfeatures=numf;
	sptr->numpoints=nump;
	sptr->parameter=(char *)NULL;
	if(nump) {
		sptr->point=(float **)iprs_calloc(nump,sizeof(float *));
		for(i=0;i<nump;i++) {
			if(numf) {
				sptr->point[i]=(float *)iprs_calloc(numf,sizeof(float));
			}
		}
	} else {
		sptr->point=NULL;
	}
	iprs_debug_exit();
	return(sptr);
}

IPRS_FEATURE_SPACE *iprs_ccopy_featurespace(IPRS_FEATURE_SPACE *sptr)
{
    IPRS_FEATURE_SPACE *optr;
	int i,j,dim,num;

	iprs_debug_enter("iprs_ccopy_featurespace");
	IPRS_STDERR_IMAGEN(sptr);
	dim=sptr->numfeatures;
	num=sptr->numpoints;
    optr=iprs_create_featurespace(dim,num);
	for(i=0;i<num;i++) {
		for(j=0;j<dim;j++) {
			optr->point[i][j]=sptr->point[i][j];
		}
	}
	optr->parameter=iprs_ccopy_parameter(sptr->parameter);
	iprs_debug_exit();
    return(optr);
}

void iprs_free_featurespace(IPRS_FEATURE_SPACE *sptr)
{
	int i;

	iprs_debug_enter("iprs_free_featurespace");
	IPRS_STDERR_IMAGE(sptr);
	for(i=0;i<sptr->numpoints;i++) {
		if(sptr->point[i]) iprs_free(sptr->point[i]);
	}
	if(sptr->numpoints) {
		iprs_free(sptr->point);
	}
	if(sptr->parameter) iprs_free(sptr->parameter);
	iprs_free(sptr);
	iprs_debug_exit();
}

IPRS_FEATURE_SPACE *iprs_liprs_featurespace(
	IPRS_FILE *in,
	char *cptr,
	int hloaded,
	int fileversion)
{
    int i,numf,nump;
    IPRS_FEATURE_SPACE *fsptr;

	iprs_debug_enter("iprs_liprs_featurespace");
	if(!hloaded) iprs_load_header(in,NULL,NULL,&fileversion,&cptr);
    iprs_read_int(in,&(numf),1);
    iprs_read_int(in,&(nump),1);
    fsptr=iprs_create_featurespace(numf,nump);
    fsptr->parameter=cptr;
	for(i=0;i<nump;i++) {
		iprs_read_float(in,fsptr->point[i],numf);
	}
	iprs_debug_exit();
    return(fsptr);
}

IPRS_FEATURE_SPACE *iprs_ltext_featurespace(
	IPRS_FILE *in,
	char *cptr,
	int hloaded,
	int fileversion)
{
	int numfeatures,numpoints;
	int data_in,numf_in,nump_in;
	IPRS_FEATURE_SPACE *fsptr;
	char str[256];
	float bval;
	int i,j;

	iprs_debug_enter("iprs_ltext_featurespace");
	if(!hloaded) iprs_load_header(in,NULL,NULL,&fileversion,&cptr);
	nump_in=0;
	numf_in=0;
	data_in=0;
	i=0;
	j=0;
	while(!data_in) {
		iprs_getstring(in,str);
		if(str[0]=='#') {
			iprs_getline(in,255,str);
		} else {
			if(!numf_in) {
				sscanf(str,"%d",&numfeatures);
				numf_in=1;
			} else {
				if(!nump_in) {
					sscanf(str,"%d",&numpoints);
					nump_in=1;
					fsptr=iprs_create_featurespace(numfeatures,numpoints);
					fsptr->parameter=cptr;
					if((numfeatures==0)||(numpoints==0)) data_in=1;	/* Fixes zero size problem */
				} else {
					if(!data_in) {
						sscanf(str,"%g",&bval);
						fsptr->point[i][j]=bval;
						j++;
						if(j==numfeatures) {
							j=0;
							i++;
							if(i==numpoints) data_in=1;
						}
					}
				}
			}
		}
	}
	iprs_debug_exit();
	return(fsptr);
}

IPRS_FEATURE_SPACE *iprs_load_featurespace(char *fname)
{
    IPRS_FILE *in;
    IPRS_FEATURE_SPACE *fsptr;
    char *cptr[1];
    int ftype,act_type,fileversion;

    iprs_debug_enter("iprs_load_featurespace");
    in=iprs_open_read(fname);
    iprs_load_header(in,&ftype,&act_type,&fileversion,&(cptr[0]));
    if(act_type!=IPRS_SPACETYPE_FEATURE) 
		fprintf(stderr, "inccorect file type\n");
    switch(ftype) {
        case IPRS_FILETYPE_IPRS:
            fsptr=iprs_liprs_featurespace(in,cptr[0],1,fileversion);
            break;

        case IPRS_FILETYPE_TEXT:
            fsptr=iprs_ltext_featurespace(in,cptr[0],1,fileversion);
            break;

        default:
			fprintf(stderr, "unknown storage type\n");
            break;
    }
    iprs_close(in);
    iprs_debug_exit();
    return(fsptr);
}

void iprs_siprs_featurespace(IPRS_FEATURE_SPACE *fsptr,IPRS_FILE *out)
{
	int i;

	iprs_debug_enter("iprs_siprs_featurespace");
    iprs_save_header(out,IPRS_FILETYPE_IPRS,
        IPRS_SPACETYPE_FEATURE,fsptr->parameter);
	iprs_write_int(out,&(fsptr->numfeatures),1);
	iprs_write_int(out,&(fsptr->numpoints),1);
	for(i=0;i<fsptr->numpoints;i++) {
		iprs_write_float(out,fsptr->point[i],fsptr->numfeatures);
	}
	iprs_debug_exit();
}

void iprs_stext_featurespace(IPRS_FEATURE_SPACE *fsptr,IPRS_FILE *out)
{
	int i,j,c;
	char m[80];

	iprs_debug_enter("iprs_stext_featurespace");
    iprs_save_header(out,IPRS_FILETYPE_TEXT,
        IPRS_SPACETYPE_FEATURE,fsptr->parameter);
	sprintf(m,"%d %d\n",fsptr->numfeatures,fsptr->numpoints);
	iprs_outstring(out,m);
	for(i=0;i<fsptr->numpoints;i++) {
		c=0;
		for(j=0;j<fsptr->numfeatures;j++) {
			sprintf(m,"%.9g ",fsptr->point[i][j]);
			iprs_outstring(out,m);
			c++;
			if(c==6) {
				iprs_outstring(out,"\n");
				c=0;
			}
		}
		if(c) iprs_outstring(out,"\n");
	}
	iprs_debug_exit();
}

void iprs_save_featurespace(
	IPRS_FEATURE_SPACE *fsptr,
	char *fname)
{
    IPRS_FILE *out;

    iprs_debug_enter("iprs_save_featurespace");
	IPRS_STDERR_IMAGE(fsptr);
    out=iprs_open_write(fname);
    iprs_siprs_featurespace(fsptr,out);
    iprs_close(out);
    iprs_debug_exit();
}

void iprs_savetext_featurespace(
	IPRS_FEATURE_SPACE *fsptr,
	char *fname)
{
    IPRS_FILE *out;

    iprs_debug_enter("iprs_savetext_featurespace");
	IPRS_STDERR_IMAGE(fsptr);
    out=iprs_open_write(fname);
    iprs_stext_featurespace(fsptr,out);
    iprs_close(out);
    iprs_debug_exit();
}
