/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_fm_fsdesc.c                                    *****
*****   AUTHOR:       ajd@cs.mu.oz.au                                     *****
*****   DATE:         06/09/91                                            *****
*****   COMMENTS:     This saves an image tree as a feature space         *****
*****                 description. This is a new format style, and        *****
*****                 does not contain _any_ new image structures. It     *****
*****                 is equivalent in function to load/save_image.       *****
*****                 However, it is written as a new image structure.    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

/*------------------------------------------------------------
 *  Comments can be included as in the following example.
 *	Example data file:
 *
 *		TEXT fsdesc
 *      PARAMETER {
 *      }
 *		FEATURE SPACES {	# This is a comment
 *			Morphological 2
 *			Unary 4
 *			Binary 3
 *		}
 *		DATA {
 *			CLASS {
 *				VIEW {
 *					Morphological { 1.0 2.0  } 
 *					Unary { 0.5 0.6 0.7 0.8 }  
 *			 		Unary { 1.0 1.2 1.4 1.6 } 
 *			 		Unary { 1.5 1.8 2.1 2.4 }
 *			 		Binary { 10.0 5.0 2.5 } 
 *			 		Binary { 5.0 2.5 1.25 }
 *				}
 *				VIEW {
 *					Morphological { 7.0 3.0 }
 *					Unary { 0.01 0.02 0.03 0.04 }
 *					Unary { 1.01 1.02 1.03 1.04 }
 *					Binary { 7.5 3.5 2.5 }
 *					Binary { 7.25 3.25 2.25 }
 *				}
 *			}
 *			CLASS {
 *				VIEW {
 *					Morphological { 10.5 10.5 }
 *					Unary { 15.0 20.0 15.0 20.0 }
 *					Binary { 1.0 2.0 3.0 }
 *					Binary { 1.5 2.5 3.5 }
 *				}
 *				VIEW {
 *					Morphological { 11.5 11.5 }
 *					Unary { 8.0 9.0 10.0 11.0 }
 *					Unary { 16.0 18.0 20.0 22.0 }
 *					Binary { 1.25 2.25 3.25 }
 *				}
 *			}
 *		}
 *------------------------------------------------------------
 */

#include "iprs.h"

IPRS_IMAGE *iprs_create_image(int n)
{
	IPRS_IMAGE *iptr;

	iprs_debug_enter("iprs_create_image");
	iptr=(IPRS_IMAGE *)iprs_calloc(1,sizeof(IPRS_IMAGE));
	iptr->numimages=n;
	iptr->mode=IPRS_IMSEP;
	iptr->parameter=(char *)NULL;
	iptr->name=(char **)NULL;
	if(n!=0) {
		iptr->type=(int *)iprs_calloc(n,sizeof(int));
		iptr->image=(void **)iprs_calloc(n,sizeof(void *));
	}
	iprs_debug_exit();
	return(iptr);
}

IPRS_IMAGE *iprs_ltext_fsdesc(
	IPRS_FILE *in,
	char *parptr,
	int hloaded,
	int fileversion)
{
	char str[256],**fsname,**tp;
	int i,j,n,numfs,datain,mode,*fsdim,*tpi;
	void **tpv;
	IPRS_IMAGE *ptr,*cptr,*vptr;
	IPRS_FEATURE_SPACE *fsptr;
	float *fp,fv;
	int nextpoint;

	iprs_debug_enter("iprs_ltext_fsdesc");
	if(!hloaded) iprs_load_header(in,NULL,NULL,&fileversion,&parptr);
	fsname=tp=(char **)NULL;
	tpv=(void **)NULL;
	fsdim=tpi=(int *)NULL;
	ptr=cptr=vptr=(IPRS_IMAGE *)NULL;
	fsptr=(IPRS_FEATURE_SPACE *)NULL;
	fp=(float *)NULL;
	numfs=0;
	mode=0;
	datain=0;
	while(!datain) {
		switch(mode) {
			case 0:	/* expecting 'FEATURE' */
				iprs_getcomstring(in,str);
				if(strcasecmp(str,"FEATURE")) {
					fprintf(stderr, "Expecting 'FEATURE' found '%s'",str);
					return((IPRS_IMAGE *)NULL);
				}
				mode++;
				break;
			
			case 1:	/* expecting 'SPACES' */
				iprs_getcomstring(in,str);
				if(strcasecmp(str,"SPACES")) {
					fprintf(stderr, "Expecting 'SPACES' found '%s'",str);
					return((IPRS_IMAGE *)NULL);
				}
				mode++;
				break;

			case 2:	/* loading feature space names until } */
				iprs_getcomstring(in,str);
				if(!strcmp(str,"{")) {
					iprs_getcomstring(in,str);
				} 
				if(!strcmp(str,"}")) {
					mode++;
					break;
				}
				tp=(char **)iprs_calloc(numfs+1,sizeof(char *));
				for(i=0;i<numfs;i++) tp[i]=fsname[i];
				tp[numfs]=strdup(str);
				if(fsname) iprs_free(fsname);
				fsname=tp;
				iprs_getcomstring(in,str);
				sscanf(str,"%d",&n);
				tpi=(int *)iprs_calloc(numfs+1,sizeof(int));
				for(i=0;i<numfs;i++) tpi[i]=fsdim[i];
				tpi[numfs]=n;
				numfs++;
				if(fsdim) iprs_free(fsdim);
				fsdim=tpi;
				break;

			case 3:	/* expecting 'DATA' */
				iprs_getcomstring(in,str);
				if(strcasecmp(str,"DATA")) {
					fprintf(stderr, "Expecting 'DATA' found '%s'",str);
					return((IPRS_IMAGE *)NULL);
				}
				ptr=iprs_create_image(0);
				mode++;
				break;

			case 4: /* main outer loop */
				iprs_getcomstring(in,str);
				if(!strcmp(str,"{")) {
					iprs_getcomstring(in,str);
				}
				if(!strcmp(str,"}")) {
					mode=7;
					break;
				}
				if(strcasecmp(str,"CLASS")) {
					fprintf(stderr, "Expecting 'CLASS' found '%s'",str);
					return((IPRS_IMAGE *)NULL);
				}
				tpv=(void **)iprs_calloc(ptr->numimages+1,sizeof(void *));
				tpi=(int *)iprs_calloc(ptr->numimages+1,sizeof(int));
				for(i=0;i<ptr->numimages;i++) {
					tpv[i]=ptr->image[i];
					tpi[i]=ptr->type[i];
				}
				tpv[ptr->numimages]=(void *)iprs_create_image(0);
				tpi[ptr->numimages]=IPRS_IMAGETYPE_IMAGE;
				if(ptr->image) iprs_free(ptr->image);
				if(ptr->type) iprs_free(ptr->type);
				ptr->numimages++;
				ptr->image=tpv;
				ptr->type=tpi;
				mode=5;
				break;
				
			case 5:	/* within class */
				iprs_getcomstring(in,str);
				if(!strcmp(str,"{")) {
					iprs_getcomstring(in,str);
				}
				if(!strcmp(str,"}")) {
					mode=4;
					break;
				}
				if(strcasecmp(str,"VIEW")) {
					fprintf(stderr, "Expecting 'VIEW' found '%s'",str);
					return((IPRS_IMAGE *)NULL);
				}
				cptr=ptr->image[ptr->numimages-1];
				tpv=(void **)iprs_calloc(cptr->numimages+1,sizeof(void *));
				tpi=(int *)iprs_calloc(cptr->numimages+1,sizeof(int));
				for(i=0;i<cptr->numimages;i++) {
					tpv[i]=cptr->image[i];
					tpi[i]=cptr->type[i];
				}
				vptr=iprs_create_image(numfs);
				for(i=0;i<numfs;i++) {
					vptr->image[i]=
						(void *)iprs_create_featurespace(fsdim[i],0);
					vptr->type[i]=IPRS_SPACETYPE_FEATURE;
				}
				tpv[cptr->numimages]=(void *)vptr;
				tpi[cptr->numimages]=IPRS_IMAGETYPE_IMAGE;
				if(cptr->image) iprs_free(cptr->image);
				if(cptr->type) iprs_free(cptr->type);
				cptr->numimages++;
				cptr->image=tpv;
				cptr->type=tpi;
				mode=6;
				iprs_getcomstring(in,str);
				if(strcmp(str,"{")) {
					fprintf(stderr, "Expecting '{' found '%s'",str);
					return((IPRS_IMAGE *)NULL);
				}
				break;

			case 6:	/* within view */
				iprs_getcomstring(in,str);
				if(!strcmp(str,"}")) {
					mode=5;
					break;
				}
				for(i=0;i<numfs;i++) {
					if(!strcasecmp(str,fsname[i])) break;
				}
				if(i==numfs) {
					fprintf(stderr, "Expecting feature space name found '%s'",str);
					return((IPRS_IMAGE *)NULL);
				}
				cptr=(IPRS_IMAGE *)(ptr->image[ptr->numimages-1]);
				vptr=(IPRS_IMAGE *)(cptr->image[cptr->numimages-1]);
				fsptr=(IPRS_FEATURE_SPACE *)(vptr->image[i]);
				iprs_getcomstring(in,str);
				if(strcmp(str,"{")) {
					fprintf(stderr, "Expecting '{' found '%s'",str);
					return((IPRS_IMAGE *)NULL);
				}
				nextpoint=iprs_addpoint_featurespace(fsptr,1);
				for(j=0;j<fsdim[i];j++) {
					iprs_getcomstring(in,str);
					if(!strcmp(str,"}")) {
						fprintf(stderr, "Expecting number found '%s'",str);
						return((IPRS_IMAGE *)NULL);
					}
					sscanf(str,"%g",&fv);
					iprs_setpoint_featurespace(fsptr,j,nextpoint,fv);
				}
				iprs_getcomstring(in,str);
				if(strcmp(str,"}")) {
					fprintf(stderr, "Expecting '}' found '%s'",str);
					return((IPRS_IMAGE *)NULL);
				}
				break;

			case 7:
				datain=1;
				break;

			default:
				fprintf(stderr, "Internal Error");
				return((IPRS_IMAGE *)NULL);
		}
	}
	ptr->parameter=parptr;
	/*
	for(i=0;i<numfs;i++) if(tp[i]) iprs_free(tp[i]);
	if(tp) iprs_free(tp);
	if(tpi) iprs_free(tpi);
	if(tpv) iprs_free(tpv);
	*/
	iprs_debug_exit();
	return(ptr);
}

IPRS_IMAGE *iprs_load_fsdesc(char *fname)
{
	IPRS_FILE *in;
	IPRS_IMAGE *iptr;
	char *cptr[1];
	int ftype,act_type,fileversion;

	iprs_debug_enter("iprs_load_fsdescimage");
	in=iprs_open_read(fname);
	iprs_load_header(in,&ftype,&act_type,&fileversion,&(cptr[0]));
	if(act_type!=IPRS_DESCTYPE_FS) {
		fprintf(stderr, "Incorrect file type\n");
		return((IPRS_IMAGE *)NULL);
	}
	switch(ftype) {
		case IPRS_FILETYPE_TEXT:
			iptr=iprs_ltext_fsdesc(in,cptr[0],1,fileversion);
			break;

		default:
			fprintf(stderr, "Unknown storage type\n");
			return((IPRS_IMAGE *)NULL);
			break;
	}
	iprs_close(in);
	iprs_debug_exit();
	return(iptr);
}

void iprs_save_fsdescimage(IPRS_IMAGE *ptr,char *fn)
{
	IPRS_FILE *out;
	IPRS_IMAGE *iptr,*iiptr;
	IPRS_FEATURE_SPACE *fsptr;
	char m[256];
	int i,j,k,l,n;

	iprs_debug_enter("iprs_save_fsdescimage");
	IPRS_STDERR_IMAGE(ptr);
	out=iprs_open_write(fn);
	iprs_save_header(out,IPRS_FILETYPE_TEXT,IPRS_DESCTYPE_FS,ptr->parameter);
	iptr=(IPRS_IMAGE *)(ptr->image[0]);
	iiptr=(IPRS_IMAGE *)(iptr->image[0]);
	iprs_outstring(out,"FEATURE SPACES {\n");
	for(i=0;i<iiptr->numimages;i++) {
		fsptr=(IPRS_FEATURE_SPACE *)(iiptr->image[i]);
		sprintf(m,"    fs%d %d\n",i,fsptr->numfeatures);
		iprs_outstring(out,m);
	}
	iprs_outstring(out,"}\nDATA {\n");
	for(i=0;i<ptr->numimages;i++) {
		iprs_outstring(out,"    CLASS {\n");
		iptr=(IPRS_IMAGE *)(ptr->image[i]);
		for(j=0;j<iptr->numimages;j++) {
			iprs_outstring(out,"        VIEW {\n");
			iiptr=(IPRS_IMAGE *)(iptr->image[j]);
			for(k=0;k<iiptr->numimages;k++) {
				fsptr=(IPRS_FEATURE_SPACE *)(iiptr->image[k]);
				for(l=0;l<fsptr->numpoints;l++) {
					sprintf(m,"            fs%d {",k);
					iprs_outstring(out,m);
					for(n=0;n<fsptr->numfeatures;n++) {
						sprintf(m," %.9g",fsptr->point[l][n]);
						iprs_outstring(out,m);
					}
					iprs_outstring(out," }\n");
				}
			}
			iprs_outstring(out,"        }\n");
		}
		iprs_outstring(out,"    }\n");
	}
	iprs_outstring(out,"}\n");
	iprs_close(out);
	iprs_debug_exit();
}
