/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_fm_image.c                                     *****
*****   AUTHOR:       cdillon@uno.vis.citri.edu.au                        *****
*****   DATE:         05/09/91                                            *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS: 23/03/93 file versions                             *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include<iprs.h>
#include<pwd.h>

extern char *iprs_imagename[];

/* The next function recursively sets the mode variable in an  */
/* image hierachy depending on the mode that it is called with */

void iprs_setimagemode(IPRS_IMAGE *in,int mode)
{
	int i;

	in->mode=mode;
	for(i=0;i<in->numimages;i++) {
		if(in->type[i]==IPRS_IMAGETYPE_IMAGE) {
			iprs_setimagemode((IPRS_IMAGE *)(in->image[i]),mode);
		}
	}
}

/* make_filename takes the route filename, the subimage filename and  */
/* generates the destination filename. This is only used in subimages */
/* specified with a filename.                                         */

void make_filename(char *fn,char *subfn,char *dest)
{
	char temp[256];
	int i;

	if(subfn[0]=='/') {
		/* subfilename is full filename */
		sprintf(dest,"%s",subfn);
		return;
	}
	if(subfn[0]=='~') { 	/* need to expand out subfilename */
		if(subfn[0]=='/') {	/* current user directory */
			sprintf(dest,"%s%s",getenv("HOME"),&(subfn[1]));
			return;
		}
		sprintf(temp,"%s",subfn);
		i=0;
		while((temp[i])&&(temp[i]!='/')) i++;
		temp[i]=0;
		sprintf(dest,"%s/%s",getpwnam(&(temp[1]))->pw_dir,&(temp[i+1]));
		return;
	}
	sprintf(temp,"%s",fn);
	i=strlen(temp);
	while((i)&&(temp[i]!='/')) i--;
	if(i) {
		temp[i]=0;
		sprintf(dest,"%s/%s",temp,subfn);
		return;
	}
	sprintf(dest,"%s",subfn);
	return;
}

char *iprs_getname_subimage(IPRS_IMAGE *iptr,int n)
{
	iprs_debug_enter("iprs_getname_subimage");
	if((n<0)||(n>=iptr->numimages)) {
		fprintf(stderr, "error in iprs_fm_image.c\n");
		return NULL;
	}
	if(!(iptr->name)) return NULL;
	iprs_debug_exit();
	return(iptr->name[n]);
}

void iprs_setname_subimage(IPRS_IMAGE *iptr,int n,char *name)
{
	iprs_debug_enter("iprs_setname_subimage");
	if((n<0)||(n>=iptr->numimages)) {
		fprintf(stderr, "error in iprs_fm_image.c\n");
		return;
	}
	if(!(iptr->name)) {
		iptr->name=(char **)iprs_calloc(iptr->numimages,sizeof(char *));
	}
	if(iptr->name[n]) iprs_free(iptr->name[n]);
	iptr->name[n]=strdup(name);
	iprs_debug_exit();
}

IPRS_IMAGE *iprs_create_image(int n)
{
	IPRS_IMAGE *iptr;

	iprs_debug_enter("iprs_create_image");
	iptr=(IPRS_IMAGE *)iprs_calloc(1,sizeof(IPRS_IMAGE));
	iptr->numimages=n;
	iptr->mode=IPRS_IMSEP;
	iptr->parameter=(char *)NULL;
	iptr->name=(char **)NULL;
	if(n!=0) {
		iptr->type=(int *)iprs_calloc(n,sizeof(int));
		iptr->image=(void **)iprs_calloc(n,sizeof(void *));
	}
	iprs_debug_exit();
	return(iptr);
}

IPRS_IMAGE *iprs_load_image(char *fname)
{
	IPRS_FILE *in;
	IPRS_IMAGE *iptr=NULL;
	char *cptr[1];
	int act_type;
	int ftype;
	int fileversion;

	iprs_debug_enter("iprs_load_image");
	in=iprs_open_read(fname);
	iprs_load_header(in,&ftype,&act_type,&fileversion,&(cptr[0]));
	switch(ftype) {
/* 		case IPRS_FILETYPE_IPRS: */
/* 			iptr=iprs_liprs_image(in,act_type,cptr[0],fname,1,fileversion); */
/* 			break; */

/* 		case IPRS_FILETYPE_TEXT: */
/* 			iptr=iprs_ltext_image(in,act_type,cptr[0],fname,1,fileversion); */
/* 			break; */

		case IPRS_FILETYPE_GIF:
			iprs_close(in);
			iptr=iprs_load_gifimage(fname);
			iprs_debug_exit();
			return(iptr);
			break;
		
/* 		case IPRS_FILETYPE_TIFF: */
/* 			iprs_close(in); */
/* 			iptr=iprs_load_tiffimage(fname); */
/* 			iprs_debug_exit(); */
/* 			return(iptr); */
/* 			break; */
		
/* 		case IPRS_FILETYPE_SGI: */
/* 			iprs_close(in); */
/* 			iptr=iprs_load_sgiimage(fname); */
/* 			iprs_debug_exit(); */
/* 			return(iptr); */
/* 			break; */

		default:
			iprs_close(in);
			fprintf(stderr, "error in iprs_fm_image.c\n");
			return((IPRS_IMAGE *)NULL);
	}
	iprs_close(in);
	iprs_debug_exit();
	return(iptr);
}

void iprs_freetype(void *ptr,int type)
{
	iprs_debug_enter("iprs_freetype");
	if(!ptr) {
		fprintf(stderr, "error in iprs_fm_image.c\n");
		return;
	}
    switch(type) {
        case IPRS_IMAGETYPE_IMAGE:
            iprs_free_image(ptr);
            break;

        case IPRS_IMAGETYPE_INT:
            iprs_free_intimage(ptr);
            break;

        default:
            fprintf(stderr, "error in iprs_fm_image.c\n");
			return;
    }
    iprs_debug_exit();
}

void iprs_free_image(IPRS_IMAGE *iptr)
{
    int i;

    iprs_debug_enter("iprs_free_image");
	if(iptr->mode==IPRS_IMSEP_NAME) {
		if(iptr->name) {
			for(i=0;i<iptr->numimages;i++) {
				if(iptr->name[i]) iprs_free(iptr->name[i]);
			}
			iprs_free(iptr->name);
		}
	}
    for(i=0;i<iptr->numimages;i++) {
    	iprs_freetype(iptr->image[i],iptr->type[i]);
    }
    if(iptr->parameter) iprs_free(iptr->parameter);
    if(iptr->type) iprs_free(iptr->type);
    if(iptr->image) iprs_free(iptr->image);
    iprs_debug_exit();
    iprs_free(iptr);
}

IPRS_RGB_IMAGE *iprs_create_2Drgbimage(
	int width,
	int height)
{
    IPRS_RGB_IMAGE *iptr;

    iprs_debug_enter("iprs_create_2Drgbimage");
    if((width<=0)||(height<=0)) {
        return NULL;
	}
    iptr=(IPRS_RGB_IMAGE *)iprs_calloc(1,sizeof(IPRS_RGB_IMAGE));
    if(!iptr) {
		return NULL;
	}
	iptr->numdim=2;
	iptr->dim=(int *)iprs_calloc(2,sizeof(int));
	iptr->dim[0]=width;
	iptr->dim[1]=height;
    iptr->parameter=(char *)NULL;
    iptr->image=(IPRS_RGB *)iprs_calloc(width*height,sizeof(IPRS_RGB));
    if(!iptr->image) {
		return NULL;
	}
    iprs_debug_exit();
    return(iptr);
}

