/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_fm_intimage.c                                  *****
*****   AUTHOR:       cdillon@cs.mu.oz.au                                 *****
*****   DATE:         05/09/91                                            *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS: 23/03/93 file versions                             *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include<iprs.h>

IPRS_INT_IMAGE *iprs_create_intimage(
	int n,
	int *d)
{
	IPRS_INT_IMAGE *iptr;
	int i,nn;

	iprs_debug_enter("iprs_create_intimage");
	iptr=(IPRS_INT_IMAGE *)iprs_calloc(1,sizeof(IPRS_INT_IMAGE));
	if(!iptr) {
		fprintf(stderr, "error in iprs_create_intimage\n");
		return NULL;
	}
	iptr->numdim=n;
	iptr->dim=(int *)iprs_calloc(n,sizeof(int));
	nn=1;
	for(i=0;i<n;i++) {
		nn*=d[i];
		iptr->dim[i]=d[i];
	}
	iptr->parameter=(char *)NULL;
	iptr->image=(int *)iprs_calloc(nn,sizeof(int));
	if(!iptr->image) {
		fprintf(stderr, "error in iprs_create_intimage\n");
		return NULL;
	}
	iprs_debug_exit();
	return(iptr);
}

IPRS_INT_IMAGE *iprs_create_2Dintimage(
	int width,
	int height)
{
    IPRS_INT_IMAGE *iptr;

    iprs_debug_enter("iprs_create_2Dintimage");
    if((width<=0)||(height<=0)) {
        fprintf(stderr, "error in iprs_fm_intimage.c\n");
        return NULL;
	}
    iptr=(IPRS_INT_IMAGE *)iprs_calloc(1,sizeof(IPRS_INT_IMAGE));
    if(!iptr) {
		fprintf(stderr, "error in iprs_create_2Dintimage\n");
		return NULL;
	}
    iptr->numdim=2;
	iptr->dim=(int *)iprs_calloc(2,sizeof(int));
	iptr->dim[0]=width;
    iptr->dim[1]=height;
    iptr->parameter=(char *)NULL;
    iptr->image=(int *)iprs_calloc(width*height,sizeof(int));
    if(!iptr->image) {
		fprintf(stderr, "error in iprs_create_2Dintimage\n");
		return NULL;
	}
    iprs_debug_exit();
    return(iptr);
}

IPRS_BYTE_IMAGE *iprs_create_2Dbyteimage(int width,int height)
{
    IPRS_BYTE_IMAGE *iptr;

    iprs_debug_enter("iprs_create_2Dbyteimage");
    if((width<=0)||(height<=0))
        fprintf(stderr, "error in iprs_fm_create_2Dbyteimage\n");
    iptr=(IPRS_BYTE_IMAGE *)iprs_calloc(1,sizeof(IPRS_BYTE_IMAGE));
    if(!iptr) {
		fprintf(stderr, "error in iprs_fm_create_2Dbyteimage\n");
		return NULL;
	}
    iptr->numdim=2;
	iptr->dim=(int *)iprs_calloc(2,sizeof(int));
	iptr->dim[0]=width;
    iptr->dim[1]=height;
    iptr->parameter=(char *)NULL;
    iptr->image=(char *)iprs_calloc(width*height,sizeof(char));
    if(!iptr->image) {
		fprintf(stderr, "error in iprs_fm_create_2Dbyteimage\n");
		return NULL;
	}
    iprs_debug_exit();
    return(iptr);
}

void iprs_free_byteimage(IPRS_BYTE_IMAGE *iptr)
{
	iprs_debug_enter("iprs_free_byteimage");
	if(iptr->dim) iprs_free(iptr->dim);
	if(iptr->image) iprs_free(iptr->image);
	if(iptr->parameter) iprs_free(iptr->parameter);
	iprs_free(iptr);
	iprs_debug_exit();
}


void iprs__intint(
	IPRS_INT_IMAGE *sptr,
	IPRS_INT_IMAGE *dptr)
{
	int i,n;
	int *s,*d;

	for(n=1,i=0;i<sptr->numdim;i++) n*=sptr->dim[i];
	s=sptr->image;
	d=dptr->image;
	for(i=0;i<n;i++) {
		*d = *s;
		d++;
		s++;
	}
}

IPRS_INT_IMAGE *iprs_inttoint(IPRS_INT_IMAGE *sptr)
{
    IPRS_INT_IMAGE *dptr;

    iprs_debug_enter("iprs_inttoint");
    if(!sptr) {
        fprintf(stderr, "error in iprs_fm_intimage.c\n");
        return NULL;
    }
    if(!sptr->image) {
        fprintf(stderr, "error in iprs_fm_intimage.c\n");
        return NULL;
    }
    dptr=iprs_create_intimage(sptr->numdim,sptr->dim);
    iprs__intint(sptr,dptr);
    iprs_debug_exit();
    return(dptr);
}

IPRS_INT_IMAGE *iprs_ccopy_intimage(IPRS_INT_IMAGE *sptr)
{
    IPRS_INT_IMAGE *dptr;

	iprs_debug_enter("iprs_ccopy_intimage");
    dptr=iprs_inttoint(sptr);
	dptr->parameter=iprs_ccopy_parameter(sptr->parameter);
	iprs_debug_exit();
    return(dptr);
}

void iprs_free_intimage(IPRS_INT_IMAGE *iptr)
{
	iprs_debug_enter("iprs_free_intimage");
	if(iptr->dim) iprs_free(iptr->dim);
	if(iptr->image) iprs_free(iptr->image);
	if(iptr->parameter) iprs_free(iptr->parameter);
	iprs_free(iptr);
	iprs_debug_exit();
}

IPRS_INT_IMAGE *iprs_liprs_intimage(
	IPRS_FILE *in,
	char *cptr,
	int hloaded,
	int fileversion)
{
    int i,j,n,nn,*d;
    IPRS_INT_IMAGE *iptr;

	iprs_debug_enter("iprs_liprs_intimage");
	if(!hloaded) iprs_load_header(in,NULL,NULL,&fileversion,&cptr);
    iprs_read_int(in,&(n),1);
	d=(int *)iprs_calloc(n,sizeof(int));
    iprs_read_int(in,d,n);
    iptr=iprs_create_intimage(n,d);
    iptr->parameter=cptr;
	nn=1;
	for(i=0;i<n;i++) nn*=d[i];
    iprs_read_int(in,iptr->image,nn);
	iprs_free(d);
	iprs_debug_exit();
    return(iptr);
}

IPRS_INT_IMAGE *iprs_ltext_intimage(
	IPRS_FILE *in,
	char *cptr,
	int hloaded,
	int fileversion)
{
    int n,nd,i;
    int numdim,numdim_in,dim_in,data_in,*d;
    IPRS_INT_IMAGE *ptr;
    char str[256];
    int bval;

	iprs_debug_enter("iprs_ltext_intimage");
	if(!hloaded) iprs_load_header(in,NULL,NULL,&fileversion,&cptr);
    dim_in=0;
    numdim_in=0;
    data_in=0;
    while(!data_in) {
        iprs_getstring(in,str);
        if(str[0]=='#') {
            iprs_getline(in,255,str);
        } else {
            if(!numdim_in) {
                sscanf(str,"%d",&numdim);
                d=(int *)iprs_calloc(numdim,sizeof(int));
                numdim_in=1;
                nd=0;
            } else {
                if(!dim_in) {
                    sscanf(str,"%d",&(d[nd]));
                    nd++;
                    if(nd==numdim) {
                        dim_in=1;
                        ptr=iprs_create_intimage(numdim,d);
                        ptr->parameter=cptr;
                        dim_in=1;
                        for(i=0,n=1;i<numdim;i++) n*=d[i];
                        i=0;
                    }
                } else {
                    if(!data_in) {
                        sscanf(str,"%d",&bval);
                        ptr->image[i]=bval;
                        i++;
                        if(i==n) data_in=1;
                    } else {    /* unknown string */
                    }
                }
            }
        }
    }
    iprs_free(d);
	iprs_debug_exit();
    return(ptr);
}

IPRS_INT_IMAGE *iprs_load_intimage(char *fname)
{
	IPRS_FILE *in;
	IPRS_INT_IMAGE *iptr;
	char *cptr[1];
	int ftype,act_type,fileversion;

	iprs_debug_enter("iprs_load_intimage");
	in=iprs_open_read(fname);
	iprs_load_header(in,&ftype,&act_type,&fileversion,&(cptr[0]));
	if(act_type!=IPRS_IMAGETYPE_INT) 
		fprintf(stderr, "error in iprs_fm_intimage.c\n");
	switch(ftype) {
		case IPRS_FILETYPE_IPRS:
			iptr=iprs_liprs_intimage(in,cptr[0],1,fileversion);
			break;

		case IPRS_FILETYPE_TEXT:
			iptr=iprs_ltext_intimage(in,cptr[0],1,fileversion);
			break;

		default:
			fprintf(stderr, "error in iprs_fm_intimage.c\n");
			break;
	}
	iprs_close(in);
	iprs_debug_exit();
	return(iptr);
}

void iprs_siprs_intimage(IPRS_INT_IMAGE *ptr,IPRS_FILE *out)
{
	int i,n;

	iprs_debug_enter("iprs_siprs_intimage");
	iprs_save_header(out,IPRS_FILETYPE_IPRS,
		IPRS_IMAGETYPE_INT,ptr->parameter);
	n=1;
	for(i=0;i<ptr->numdim;i++) n*=ptr->dim[i];
	iprs_write_int(out,&(ptr->numdim),1);
	iprs_write_int(out,ptr->dim,ptr->numdim);
    iprs_write_int(out,ptr->image,n);
    iprs_debug_exit();
}

void iprs_stext_intimage(IPRS_INT_IMAGE *ptr,IPRS_FILE *out)
{
	char m[80];
	int i,j,n;

	iprs_debug_enter("iprs_stext_intimage");
    iprs_save_header(out,IPRS_FILETYPE_TEXT,
        IPRS_IMAGETYPE_INT,ptr->parameter);
    sprintf(m,"%d\n",ptr->numdim);
    iprs_outstring(out,m);
    n=1;
    for(i=0;i<ptr->numdim;i++) {
        sprintf(m,"%d ",ptr->dim[i]);
        iprs_outstring(out,m);
        n*=ptr->dim[i];
    }
    iprs_outstring(out,"\n");
    j=6;
    if(ptr->dim[0]<j) j=ptr->dim[0];    /* newlines */
    for(i=0;i<n;i++) {
        sprintf(m,"%d ",ptr->image[i]);
        iprs_outstring(out,m);
        if((i%j)==(j-1)) iprs_outstring(out,"\n");
    }
	if((i%j)!=(j-1)) iprs_outstring(out,"\n");
    iprs_debug_exit();
}

void iprs_save_intimage(
	IPRS_INT_IMAGE *iptr,
	char *fname)
{
	IPRS_FILE *out;
	
	iprs_debug_enter("iprs_save_intimage");
	out=iprs_open_write(fname);
	iprs_siprs_intimage(iptr,out);
	iprs_close(out);
	iprs_debug_exit();
}

void iprs_savetext_intimage(
	IPRS_INT_IMAGE *iptr,
	char *fname)
{
    IPRS_FILE *out;

    iprs_debug_enter("iprs_savetext_intimage");
    out=iprs_open_write(fname);
    iprs_stext_intimage(iptr,out);
    iprs_close(out);
    iprs_debug_exit();
}
