/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_fm_regionimage.c                               *****
*****   AUTHOR:       cdillon@cs.mu.oz.au                                 *****
*****   DATE:         05/09/91                                            *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS: 23/03/93 file versions                             *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include "iprs.h"

IPRS_REGION_IMAGE *iprs_create_regionimage(int num)
{
	IPRS_REGION_IMAGE *jptr;
	int i;

	iprs_debug_enter("iprs_create_regionimage");
	jptr=(IPRS_REGION_IMAGE *)calloc(1,sizeof(IPRS_REGION_IMAGE));
	jptr->numregions=num;
	jptr->parameter=(char *)NULL;
	jptr->region=(IPRS_REGION **)calloc(num,sizeof(IPRS_REGION*));
	for(i=0;i<num;i++) {
		jptr->region[i]=(IPRS_REGION *)calloc(1,sizeof(IPRS_REGION));
	}
	iprs_debug_exit();
	return(jptr);
}

IPRS_REGION_IMAGE *iprs_creates_regionimage(int w,int h)
{
	IPRS_REGION_IMAGE *dest;
	IPRS_REGION *rptr;

	iprs_debug_enter("iprs_creates_regionimage");
	dest=iprs_create_regionimage(1);
	rptr=dest->region[0];
	rptr->numpixels=2;
	rptr->x=(int *)calloc(2,sizeof(int));
	rptr->y=(int *)calloc(2,sizeof(int));
	rptr->x[0]=0;
	rptr->y[0]=0;
	rptr->x[1]=w-1;
	rptr->y[1]=h-1;
	rptr->numregions=0;
	rptr->region=(int *)NULL;
	rptr->edge=(int *)NULL;
	iprs_debug_exit();
	return(dest);
}

void iprs_free_regionimage(IPRS_REGION_IMAGE *jptr)
{
	int i;
	IPRS_REGION *rptr;

	iprs_debug_enter("iprs_free_regionimage");
	IPRS_STDERR_IMAGE(jptr);
	if(jptr->parameter) free(jptr->parameter);
	if(jptr->region) {
		for(i=0;i<(jptr->numregions);i++) {
			rptr=jptr->region[i];
			if(rptr) {
				if(rptr->x) free(rptr->x);
				if(rptr->y) free(rptr->y);
				if(rptr->edge) free(rptr->edge);
				if(rptr->region) free(rptr->region);
				free(rptr);
			}
		}
		free(jptr->region);
	}
	free(jptr);
	iprs_debug_exit();
}

IPRS_REGION_IMAGE *iprs_liprs_regionimage(
	IPRS_FILE *in,
	char *cptr,
	int hloaded,
	int fileversion)
{
    int i,num;
    IPRS_REGION_IMAGE *jptr;
	IPRS_REGION *ptr;

	iprs_debug_enter("iprs_liprs_regionimage");
	if(!hloaded) iprs_load_header(in,NULL,NULL,&fileversion,&cptr);
    iprs_read_int(in,&(num),1);
    jptr=iprs_create_regionimage(num);
    jptr->parameter=cptr;
	iprs_read_int(in,&(jptr->numedges),1);
	for(i=0;i<num;i++) {
		ptr=jptr->region[i];
		iprs_read_int(in,&(ptr->numpixels),1);
		iprs_read_int(in,&(ptr->numregions),1);
		if(ptr->numpixels) {
			ptr->x=(int *)calloc(ptr->numpixels,sizeof(int));
			ptr->y=(int *)calloc(ptr->numpixels,sizeof(int));
			iprs_read_int(in,ptr->x,ptr->numpixels);
			iprs_read_int(in,ptr->y,ptr->numpixels);
		}
		if(jptr->numedges) {
			ptr->edge=(int *)calloc(jptr->numedges,sizeof(int));
			iprs_read_int(in,ptr->edge,jptr->numedges);
		}
		if(ptr->numregions) {
			ptr->region=(int *)calloc(ptr->numregions,sizeof(int));
			iprs_read_int(in,ptr->region,ptr->numregions);
		}
	}
	iprs_debug_exit();
    return(jptr);
}

IPRS_REGION_IMAGE *iprs_ltext_regionimage(
	IPRS_FILE *in,
	char *cptr,
	int hloaded,
	int fileversion)
{
	IPRS_REGION_IMAGE *rptr;
	IPRS_REGION *ptr;
	char m[256];
	int mode,numregions,regnum,rregnum,edgenum,pixnum,noload;

	iprs_debug_enter("iprs_ltext_regionimage");
	if(!hloaded) iprs_load_header(in,NULL,NULL,&fileversion,&cptr);
	mode=0;
	noload=0;
	while(1) {
		if(!noload) iprs_getcomstring(in,m);
		noload=0;
		switch(mode) {
			case 0:	/* numregions */
				sscanf(m,"%d",&numregions);
				rptr=iprs_create_regionimage(numregions);
				rptr->parameter=cptr;
				mode=1;
				break;

			case 1:	/* numedges */
				sscanf(m,"%d",&(rptr->numedges));
				regnum=0;
				ptr=rptr->region[0];
				mode=2;
				break;

			case 2: /* numpixels */
				sscanf(m,"%d",&(ptr->numpixels));
				if(ptr->numpixels) {
					ptr->x=(int *)calloc(ptr->numpixels,sizeof(int));
					ptr->y=(int *)calloc(ptr->numpixels,sizeof(int));
				} else {
					ptr->x=(int *)NULL;
					ptr->y=(int *)NULL;
				}
				mode=3;
				break;

			case 3:	/* numregions in region */
				sscanf(m,"%d",&(ptr->numregions));
				if(ptr->numregions) {
					ptr->region=(int *)calloc(ptr->numregions,sizeof(int));
				} else {
					ptr->region=(int *)NULL;
				}
				if(rptr->numedges) {
					ptr->edge=(int *)calloc(rptr->numedges,sizeof(int));
				} else {
					ptr->edge=(int *)NULL;
				}
				pixnum=0;
				if(ptr->numpixels) {
					mode=4;
				} else {
					edgenum=0;
					mode=6;
				}
				break;

			case 4:	/* load pixels x */
				sscanf(m,"%d",&(ptr->x[pixnum]));
				mode=5;
				break;

			case 5: /* load pixels y */
				sscanf(m,"%d",&(ptr->y[pixnum]));
				pixnum++;
				if(pixnum<ptr->numpixels) {
					mode=4;
				} else {
					edgenum=0;
					mode=6;
				}
				break;

			case 6:	/* load edge references */
				if(rptr->numedges) {
					sscanf(m,"%d",&(ptr->edge[edgenum]));
					edgenum++;
					if(edgenum<rptr->numedges) break;
				} else {
					noload=1;
				}
				mode=7;
				rregnum=0;
				break;

			case 7:	/* load region references */
				if(ptr->numregions) {
					sscanf(m,"%d",&(ptr->region[rregnum]));
					rregnum++;
					if(rregnum<ptr->numregions) break;
				} else {
					noload=1;
				}
				regnum++;
				if(regnum<rptr->numregions) {
					ptr=rptr->region[regnum];
					mode=2;
					break;
				} else {
					iprs_debug_exit();
					return(rptr);
				}

			default:
				fprintf(stderr, "internal error: illegal switch\n");
				return NULL;
		}
	}
}

IPRS_REGION_IMAGE *iprs_load_regionimage(char *fname)
{
	IPRS_FILE *in;
	IPRS_REGION_IMAGE *rptr;
	char *cptr[1];
	int ftype,act_type,fileversion;

	iprs_debug_enter("iprs_load_regionimage");
	in=iprs_open_read(fname);
    iprs_load_header(in,&ftype,&act_type,&fileversion,&(cptr[0]));
	if(act_type!=IPRS_IMAGETYPE_REGION) {
		fprintf(stderr, "Incorrect file type\n");
		return((IPRS_REGION_IMAGE *)NULL);
	}
    switch(ftype) {
        case IPRS_FILETYPE_IPRS:
            rptr=iprs_liprs_regionimage(in,cptr[0],1,fileversion);
            break;

        case IPRS_FILETYPE_TEXT:
            rptr=iprs_ltext_regionimage(in,cptr[0],1,fileversion);
            break;

        default:
			fprintf(stderr, "Unknown storage type\n");
			return((IPRS_REGION_IMAGE *)NULL);
    }
	iprs_debug_exit();
	return(rptr);
}

void iprs_siprs_regionimage(IPRS_REGION_IMAGE *jptr,IPRS_FILE *out)
{
	int i;
	IPRS_REGION *ptr;

	iprs_debug_enter("iprs_siprs_regionimage");
	iprs_save_header(out,IPRS_FILETYPE_IPRS,
		IPRS_IMAGETYPE_REGION,jptr->parameter);
	iprs_write_int(out,&(jptr->numregions),1);
	iprs_write_int(out,&(jptr->numedges),1);
	for(i=0;i<(jptr->numregions);i++) {
        ptr=jptr->region[i];
        iprs_write_int(out,&(ptr->numpixels),1);
        iprs_write_int(out,&(ptr->numregions),1);
        iprs_write_int(out,ptr->x,ptr->numpixels);
        iprs_write_int(out,ptr->y,ptr->numpixels);
		iprs_write_int(out,ptr->edge,jptr->numedges);
		iprs_write_int(out,ptr->region,ptr->numregions);
    }
    iprs_debug_exit();
}

void iprs_stext_regionimage(IPRS_REGION_IMAGE *rptr,IPRS_FILE *out)
{
	int i,j;
	IPRS_REGION *ptr;

	iprs_debug_enter("iprs_stext_regionimage");
	iprs_save_header(out,IPRS_FILETYPE_TEXT,
		IPRS_IMAGETYPE_REGION,rptr->parameter);
	fprintf(out,"%d %d\n",rptr->numregions,rptr->numedges);
	for(i=0;i<rptr->numregions;i++) {
		ptr=rptr->region[i];
		fprintf(out,"%d %d\n",ptr->numpixels,ptr->numregions);
		for(j=0;j<ptr->numpixels;j++) fprintf(out,"%d %d\n",ptr->x[j],ptr->y[j]);
		for(j=0;j<rptr->numedges;j++) fprintf(out,"%d ",ptr->edge[j]);
		if(rptr->numedges) fprintf(out,"\n");
		for(j=0;j<rptr->numregions;j++) fprintf(out,"%d ",ptr->region[j]);
		if(rptr->numedges) fprintf(out,"\n");
	}
	iprs_debug_exit();
}

void iprs_save_regionimage(IPRS_REGION_IMAGE *jptr,char *fname)
{
	IPRS_FILE *out;

	iprs_debug_enter("iprs_save_regionimage");
	out=iprs_open_write(fname);
	iprs_siprs_regionimage(jptr,out);
	iprs_close(out);
	iprs_debug_exit();
}

void iprs_savetext_regionimage(IPRS_REGION_IMAGE *jptr,char *fname)
{
	IPRS_FILE *out;

	iprs_debug_enter("iprs_savetext_regionimage");
	out=iprs_open_write(fname);
	iprs_stext_regionimage(jptr,out);
	iprs_close(out);
	iprs_debug_exit();
}
