/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_header.c                                       *****
*****   AUTHOR:       cdillon@cs.mu.oz.au                                 *****
*****   DATE:         06/09/91                                            *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS:                                                    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include "iprs.h"

char *iprs_imagename[]={
"image",
"byte",
"float",
"index",
"rgb",
"integer",
"edge",
"junction",
"region",
"neuralnet",
"labelspace",
"featurespace",
"complex",
"ruleactspace",
"ruletable",
"hrectrule",
"hellipserule",
"chyprule",
"segment",
"fsdesc",
"boolean",
"view table",
"featuretable",
"rle",
"none",
"splitrule",
"partitionspace",
""};

void iprs_save_header(IPRS_FILE *out,int ftype,int itype,char *cptr)
{
	char buffer[128];
	int i;

	iprs__setiparam(&cptr,"FILE_VERSION",IPRS_FILE_VERSION);
	switch(ftype) {
		case IPRS_FILETYPE_IPRS:
			sprintf(buffer,"%s",IPRS_HEADER);
			iprs_write_char(out,buffer,strlen(buffer));
			iprs_write_int(out,&itype,1);
			if(cptr) {
				i=0;
				iprs_write_char(out,cptr,strlen(cptr));
			}
			buffer[0]=0;
			iprs_write_char(out,&(buffer[0]),1); /* null termination */
			break;

		case IPRS_FILETYPE_TEXT:
			iprs_outstring(out,TEXT_HEADER);
			if((itype>=0)&&(itype<=IPRS_MAXIMAGES)) {
				sprintf(buffer," %s\n",iprs_imagename[itype]);
			} else {
				fprintf(stderr, "error\n");
			}
			iprs_outstring(out,buffer);
			iprs_outstring(out,"PARAMETER {\n    ");
			if(cptr) {
				i=0;
				while(cptr[i]) {
					if(cptr[i]==';') {
						if(cptr[i+1]==0) iprs_outstring(out,";\n");
						else iprs_outstring(out,";\n    ");    
					} else {
						iprs_write_char(out,&(cptr[i]),1);
					}
					i++;
				}
			}
			iprs_outstring(out,"}\n");
            break;

		case IPRS_FILETYPE_AE:
			sprintf(buffer,"%s",AE_HEADER);
			iprs_write_char(out,buffer,strlen(buffer));
			iprs_write_int(out,&itype,1);
			if(cptr) {
				i=0;
				iprs_write_char(out,cptr,strlen(cptr));
			}
			buffer[0]=0;
			iprs_write_char(out,&(buffer[0]),1); /* null termination */
			break;

		default:
			fprintf(stderr, "error\n");
			return;
	}
}

void iprs_load_header(
	IPRS_FILE *in,
	int *ftype,
	int *act_type,
	int *fileversion,
	char **cptr)
{
	char temp[80],ttemp;
	int i,j,num,pa,pb;
	int filetype;
	char buf[4096];		/* extend for larger parameter strings */
	char stype[256];
	unsigned short *sgi_magic;

	ttemp=' ';
	while((ttemp==' ')||(ttemp=='\n')||(ttemp==8)) {
		iprs_read_char(in,&ttemp,1);
	}
	temp[0]=ttemp;
	iprs_read_char(in,temp+1,3);
	sgi_magic=(unsigned short *)temp;
	filetype=IPRS_FILETYPE_UNKNOWN;
	if(!strncmp(temp,IPRS_HEADER,4)) { 
		filetype=IPRS_FILETYPE_IPRS;
	}	
	if(!strncmp(temp,TEXT_HEADER,4)) {
		filetype=IPRS_FILETYPE_TEXT;
	}
	if(!strncmp(temp,AE_HEADER,4)) {
		filetype=IPRS_FILETYPE_AE;
	}
	if(!strncmp(temp,GIF_HEADER,3)) {
		filetype=IPRS_FILETYPE_GIF;
	}
	if(!strncmp(temp,TIFF_HEADER1,2)) {
		filetype=IPRS_FILETYPE_TIFF;
	}
	if(!strncmp(temp,TIFF_HEADER2,2)) {
		filetype=IPRS_FILETYPE_TIFF;
	}
	if((filetype==IPRS_FILETYPE_UNKNOWN)&&
	  ((*sgi_magic)==(unsigned short)0732)) { /* 0732 - iprs_load_sgiimage */
		filetype=IPRS_FILETYPE_SGI;
	}
	if(ftype) *ftype=filetype;
	switch(filetype) {
		case IPRS_FILETYPE_IPRS:
			if(act_type) {
				iprs_read_int(in,act_type,1); 
			} else {
				/* this should never happen... */
				iprs_read_int(in,&i,1);
			}
			i=0;
			while(1) {
				iprs_read_char(in,&(buf[i]),1);
				if(!buf[i]) break;
				i++;
			}
			if(i>0) {
				num=i+1;
				cptr[0]=(char *)iprs_calloc(num,sizeof(char));
				for(i=0;i<num;i++) cptr[0][i]=buf[i];
			} else {
				cptr[0]=(char *)NULL;
			}
			iprs_process_param(cptr);
			*fileversion=iprs__getiparam(cptr,"FILE_VERSION");
			break;

		case IPRS_FILETYPE_TEXT:
			cptr[0]=(char *)NULL;
			iprs_getstring(in,buf);
			sscanf(buf,"%s",stype);
			for(j=0;j<=IPRS_MAXIMAGES;j++) {
				if(!iprs_imagename[j][0]) {
					fprintf(stderr, "error\n");
					return;
				}
				if(!strcasecmp(stype,iprs_imagename[j])) break;
			}
			if(j==(IPRS_MAXIMAGES+1)) {
				fprintf(stderr, "error\n");
				if(act_type) *act_type= -1;
			}
			if(act_type) *act_type=j;
			iprs_getstring(in,buf);
			while(!strcmp(buf,"#")) {
				iprs_getline(in,4095,buf);
				iprs_getstring(in,buf);
			}
			if(strcmp(buf,"PARAMETER")) {
				fprintf(stderr, "error\n");
				return;
			}
			iprs_getstring(in,buf);
			if(buf[0]!='{') {
				fprintf(stderr, "error\n");
				return;
			}
			while(1) {
				iprs_getline(in,4095,buf);
				i=0;
				while(buf[i]) {	/* read in whitespace */
					if(buf[i]=='}') break;
					if(!((buf[i]==' ')||(buf[i]==8)||
						(buf[i]==10)||(buf[i]==13))) break;
					i++;
				}
				if(buf[i]=='}') break;
				pa= -1;
				pb= -1;
				for(i=0;buf[i];i++) {
					if(pa<0) {
						if(buf[i]=='#') break;
						if(buf[i]!=' ') pa=i;
					}
					if(buf[i]=='=') {
						pb=i+1;
						buf[i]=0;
					}
					if(buf[i]==';') {
						buf[i]=0;
						break;
					}
				}
				iprs__setsparam(cptr,buf+pa,buf+pb);
			}
			iprs_process_param(cptr);
			*fileversion=iprs__getiparam(cptr,"FILE_VERSION");
			break;

		case IPRS_FILETYPE_AE:
			if(act_type) {
				iprs_read_int(in,act_type,1);
			} else {
				iprs_read_int(in,&i,1);
			}
			i=0;
			while(1) {
				iprs_read_char(in,&(buf[i]),1);
				if(!buf[i]) break;
				i++;
			}
			if(i>0) {
				num=i+1;
				cptr[0]=(char *)iprs_calloc(num,sizeof(char));
				for(i=0;i<num;i++) cptr[0][i]=buf[i];
			} else {
				cptr[0]=(char *)NULL;
			}
			iprs_process_param(cptr);
			*fileversion=iprs__getiparam(cptr,"FILE_VERSION");
			break;

		case IPRS_FILETYPE_GIF:
			if(act_type) {
				*act_type=IPRS_IMAGETYPE_RGB;	/* not necessarily true */
			}
			*cptr=NULL;
			*fileversion=0;
			break;
		
		case IPRS_FILETYPE_TIFF:
			if(act_type) {
				*act_type=IPRS_IMAGETYPE_RGB;	/* not necessarily true */
			}
			*cptr=NULL;
			*fileversion=0;
			break;
		
		case IPRS_FILETYPE_SGI:
			if(act_type) {
				*act_type=IPRS_IMAGETYPE_RGB;	/* not necessarily true */
			}
			*cptr=NULL;
			*fileversion=0;
			break;

		default:
			/* return and let the calling function work out what is wrong */
			break;
	}
	if(*fileversion>IPRS_FILE_VERSION) {
		fprintf(stderr, "file version number higher than current IPRS file version\n");
	}
}
