/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:          iprs_images.h                                      *****
*****   AUTHOR:        cdillon@cs.mu.oz.au                                *****
*****                  wild@uno.vis.citri.edu.au                          *****
*****                  jc@uno.vis.citri.edu.au                            *****
*****   DATE:          27/12/91                                           *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS:                                                    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#define IPRS_IMAGETYPE_IMAGE		0
#define IPRS_IMAGETYPE_BYTE 		1
#define IPRS_IMAGETYPE_FLOAT		2
#define IPRS_IMAGETYPE_INDEX  		3
#define IPRS_IMAGETYPE_RGB			4
#define IPRS_IMAGETYPE_INT			5
#define IPRS_IMAGETYPE_EDGE			6
#define IPRS_IMAGETYPE_JUNCTION		7
#define IPRS_IMAGETYPE_REGION		8
#define IPRS_IMAGETYPE_NNET			9
#define IPRS_SPACETYPE_LABEL		10
#define IPRS_SPACETYPE_FEATURE		11
#define IPRS_IMAGETYPE_COMPLEX  	12
#define IPRS_SPACETYPE_RULEACT		13
#define IPRS_TABLETYPE_RULE			14
#define IPRS_RULETYPE_HRECT			15	
#define IPRS_RULETYPE_HELLIPSE		16
#define IPRS_RULETYPE_CHYP			17
#define IPRS_IMAGETYPE_SEGMENT		18
#define IPRS_DESCTYPE_FS			19
#define IPRS_IMAGETYPE_BOOLEAN		20
#define IPRS_TABLETYPE_VIEW			21
#define IPRS_TABLETYPE_FEATURE		22
#define IPRS_IMAGETYPE_RLE			23
#define IPRS_IMAGETYPE_NONE			24	/* reserved for empty array elements */
#define IPRS_RULETYPE_SPLIT			25
#define IPRS_SPACETYPE_PARTITION	26

#define IPRS_MAXIMAGES				26

#define IPRS_IMSEP 		0	/* 'seperated' files are seperate */
#define IPRS_IMCAT 		1	/* 'concatenated' files are concatenated */
#define	IPRS_IMSEP_NAME	2	/* 'seperated_name' but names are given */

/* Next is not implemented yet, but a good idea 
#define IPRS_IMSEP_MAX	3	   'seperated_max' keep going until no files 
*/

extern char *iprs_imagename[];

typedef struct {
	char *parameter;
	int mode;		/* IPRS_IMSEP or IPRS_IMCAT */
	int numimages;
	int *type;		/* IPRS_IMAGETYPE_? */
	void **image;
	char **name;	/* if files have individual names */
	} IPRS_IMAGE;

typedef struct {
	char *parameter;
	int numdim;
	int *dim;
	unsigned long *image;
	} IPRS_BOOLEAN_IMAGE;

typedef struct {
	char *parameter;
	int numdim;
	int *dim;
	char *image;
	} IPRS_BYTE_IMAGE;

typedef struct {
	char *parameter;
	int numdim;
	int *dim;
	float *image;
	} IPRS_FLOAT_IMAGE;

typedef struct {
	char *parameter;
	int num;        		/* number in indices */	
	int *index;     		/* list of indices, stored in linear coords */
	} IPRS_INDEX_IMAGE;	

typedef struct {
	unsigned char r,g,b;
	} IPRS_RGB;

typedef struct {
    char *parameter;
	int numdim;
	int *dim;
    IPRS_RGB *image;
    } IPRS_RGB_IMAGE;

typedef struct {
	char *parameter;
	int numdim;
	int *dim;
	int *image;
	} IPRS_INT_IMAGE;

typedef struct {
	int numpixels;
	int *x,*y;
	float sig;		/* added by B.McCane for the perceptual significance */
	} IPRS_EDGE;

typedef struct {
	char *parameter;
	int numedges;
	IPRS_EDGE **edge;
	} IPRS_EDGE_IMAGE;

typedef struct {
	int numedges;
	int *num;		/* edge number for each touching edge */
	int *x,*y;		/* pixel location of pixel it joins */
	int jun_type;	/* JUN_OC or JUN_TX */
    int x_pos, y_pos; /* junction location : added by Brendan McCane */
	} IPRS_JUNCTION;
	
typedef struct {
	char *parameter;
	int numjunctions;
	IPRS_JUNCTION **junction;
	} IPRS_JUNCTION_IMAGE;

typedef struct {
	int numpixels;		/* number of pixels */
	int *x,*y;			/* pixels */
	int *edge;			/* REG_NC, REG_TO, REG_OC, REG_BO */
	int numregions;		/* number of regions to include */
	int *region;		/* region index */
	} IPRS_REGION;

typedef struct {
	char *parameter;	
	int numregions;
	int numedges;
	IPRS_REGION **region;
	} IPRS_REGION_IMAGE;

typedef struct {
	char *parameter;
	int numfeatures;
	int numpoints;
	float **point;
	} IPRS_FEATURE_SPACE;

typedef struct {
	char *parameter;
	int numpoints;				/* number of points in space */
	int minlabel,maxlabel;		/* start and end label */
	int *ptsperlabel;			/* starts at minlabel, goes to maxlabel */
	int *numlabels;				/* number of labels for each point */
	int **label;				/* labels for each point */
	} IPRS_LABEL_SPACE;

typedef struct {
	char *parameter;
	int numpoints;				/* number of points */
	int numrules;				/* number of rules */
	int maxrulelevel;			/* maximum level of any rule - do we need this*/
	int *rulelevel;				/* level of each rule */
	float **rule;				/* rules x points */
	} IPRS_RULEACT_SPACE;

typedef struct {
	char *parameter;
    int numrules;
    int minclass,maxclass;
    int numtab;
    char **ruletable;
    int **classtable;
    } IPRS_RULE_TABLE;

typedef struct {
	float re,im;
	} IPRS_COMPLEX;

typedef struct {
	char *parameter;
	int numdim;
	int *dim;
	IPRS_COMPLEX *image;
	} IPRS_COMPLEX_IMAGE;

typedef struct {
	char *parameter;
	int numfeatures;
	int numrules;
	int *rules;
	int maxrulelevel;
	int *rulelevel;
	float **upper;
	float **lower;
	} IPRS_HRECT_RULE;
	
typedef struct {
	char *parameter;
	int numfeatures;
	int numrules;
	int *rules;
	int maxrulelevel;
	int *rulelevel;
	float **centre;
	float **s;
	float *maxrsq;
	} IPRS_HELLIPSE_RULE;

typedef struct {
	char *parameter;
	int numfeatures;
	int numrules;
	int *rules;
	int maxrulelevel;
	int *rulelevel;
	int *numvertices;
	float ***vertex;
	} IPRS_CHYP_RULE;

typedef struct {
    float output;	/* weighted sum after non-linearity */
	float net;		/* weighted sum input */
    float del;		/* error term for this neuron */
    float *weight;
    float *dw;		/* previous change in weights (for momentum) */
    } IPRS_NEURON;

typedef struct {
    char *parameter;
    int numlayers;
    int *numneurons;
    IPRS_NEURON **neuron;
    } IPRS_NNET_IMAGE;

typedef struct {
	char *parameter;
	int numdim;
	int *dim;
	int *numlabels;		/* size of the image */
	int **label;		/* size of the image by numpixels at each point */
	} IPRS_SEGMENT_IMAGE;

typedef struct {
	int numviews;		/* number of image/region pairs */
	int *image;			/* image number for the component */
	int *region;		/* region number for the component */
	int numobjects;		/* number of sub-objects to consider */
	int *object;		/* index for each sub-object */
	} IPRS_VIEW;

typedef struct {
	char *parameter;
	int numobjects;		/* Number of objects */
	IPRS_VIEW **object;	/* Array of IPRS_VIEW - one per object */
	} IPRS_VIEW_TABLE;

/* The following section defines the IPRS_FEATURE_TABLE and all the */
/* different feature structures and feature indices.                */

typedef struct {
	char *parameter;
	int numobjects;			/* number of objects in the structure */
	void ***object;			/* array of array of pointers         */
	} IPRS_FEATURE_TABLE;

typedef struct {
	int version;			/* version number of structure */
	IPRS_RGB mean;			/* mean colour value in RGB    */
	IPRS_RGB var;			/* colour variance in RGB      */
	} IPRS_F_COLOUR;

#define IPRS_FI_COLOUR		0

#define IPRS_NUM_FEATURES	1	/* This should be one more than the largest */									/* feature index above...                   */

#define IPRS_FV_COLOUR		1	/* Feature version - 1 is earliest version */

/* This is the end of the IPRS_FEATURE_TABLE definition. */

typedef struct {
	char val;
	char num;
	} IPRS_RLE;

typedef struct {
	char *parameter;
	int numdim;
	int *dim;
	int numrle;
	IPRS_RLE *image;
	} IPRS_RLE_IMAGE;

typedef struct {
	int feature; /* the feature number to split on */
	float split; /* the value in that feature to split on */
	int lower;	 /* lower rule number */
	int upper;	 /* upper rule number */
	} IPRS_SPLIT;

typedef struct {
	char *parameter;
	int numrules;
	IPRS_SPLIT **rule;
	} IPRS_SPLIT_RULE;

typedef struct {
	float *lower;   /* minimum bounds on box - 1 per feature */
	float *upper;   /* maximum bounds on box - 1 per feature */
	int time_created; /* point number at which created */
	int est_created;  /* estimated num. points when created */
	int num_since;    /* number of points since created */
	} IPRS_PARTITION;

typedef struct {
	char *parameter;
	int numfeatures;
	int numpartitions;
	IPRS_PARTITION **partition;
	} IPRS_PARTITION_SPACE;

