#include "iprs.h"

#define NUM_DIRECTIONS 8

static int dx[NUM_DIRECTIONS],dy[NUM_DIRECTIONS];

void ile_propagate(im,w,h,x,y,d) 
int *im;		/* map of labels */
int w,h;
int x,y;        /* position to propagate from */
int d;          /* direction */
{
    int le,ue,dd,i,j,cl,si,sj;

    if(d&1) {   /* odd, so three neighbours */
        le= -1;
        ue=1;
    } else {    /* even, so five neighbours */
        le= -2;
        ue=2;
    }
    for(i=le;i<=ue;i++) {
        dd=(d+i+NUM_DIRECTIONS)%NUM_DIRECTIONS;
        si=x+dx[dd];
        sj=y+dy[dd];
		if((si>=0)&&(si<w)&&(sj>=0)&&(sj<h)) {
        	if(im[si+w*sj]==2) return;
		}
    } /* not blocked */

    cl=im[x+w*y];
    for(i=le;i<=ue;i++) {
        dd=(d+i+NUM_DIRECTIONS)%NUM_DIRECTIONS;
        si=x+dx[dd];
        sj=y+dy[dd];
		if((si>=0)&&(si<w)&&(sj>=0)&&(sj<h)) {
     	   	if(im[si+w*sj]==1) {
       	     	im[si+w*sj]=cl;
       	     	ile_propagate(im,w,h,si,sj,dd);
       	 	}
		}
    }
}
void iprs_label_intedges(IPRS_INT_IMAGE *ptr)
{
	int w,h,counte,countj,i,j,ii,jj;
	int *im;
	int numsegments,count;
	int si,sj,done,d,pass,found;

	iprs_debug_enter("iprs_label_intedges");
	if(ptr->numdim!=2) {
		fprintf(stderr, "image not 2D in iprs_label_intedges\n");
		return;
	}
	w=ptr->dim[0];
	h=ptr->dim[1];
	im=ptr->image;

	dx[0]=dy[4]=dy[5]=dy[6]=dx[6]=dx[7]= -1;
    dx[1]=dx[5]=dy[3]=dy[7]=0;
    dx[2]=dx[3]=dx[4]=dy[0]=dy[1]=dy[2]=1;

	/* do a little bit of error checking */

	for(i=1;i<(w-1);i++) {
		for(j=1;j<(h-1);j++) {
			if(im[i+w*j]==1) {	/* edge pixel */
				counte=0;
				countj=0;
				for(ii= -1;ii<=1;ii++) {
					for(jj= -1;jj<=1;jj++) {
						if((ii==0)&&(jj==0)) continue;
						if(im[i+ii+w*(j+jj)]==1) counte++;
						if(im[i+ii+w*(j+jj)]==2) countj++;
					}
				}
				if((countj==0)&&(counte>2)) {
                    /* Modification by mccane@coral.cs.jcu.edu.au 30/8/93 */
                    fprintf(stderr,"branching problem at : %d, %d\n", i, j);
				}
			}
		}
	}

	/* junctions are 2, edges are 1 */

	numsegments=0;
	done=0;
	while(!done) {
		done=1;
		found=0;
		for(i=1;(i<(w-1))&&(!found);i++) {
			for(j=1;(j<(h-1))&&(!found);j++) {
				if(im[i+w*j]==1) {
					/* have to check neighbourhood for junctions */
					found=0;
					count=0;
					pass=1;
					si=i;
					sj=j;
					for(ii= -1;ii<=1;ii++) {
						for(jj= -1;jj<=1;jj++) {
							if((ii==0)&&(jj==0)) continue;
							if(im[i+ii+w*(j+jj)]==2) pass=0; /* dont use */
							if(im[i+ii+w*(j+jj)]==1) count++;
						}
					}
					if((pass)&&(count==2)) found=1;
				}
			}
		}
		if(found) {
			im[si+w*sj]=numsegments+3;
			for(d=0;d<NUM_DIRECTIONS;d++) {
				ile_propagate(im,w,h,si,sj,d); 
			}
			done=0;
			numsegments++;
		}
	}

	/* for closed, no-junctions regions, start anywhere */
    done=0;
    while(!done) {
        done=1;
        found=0;
        for(i=1;(i<(w-1))&&(!found);i++) {
            for(j=1;(j<(h-1))&&(!found);j++) {
                if(im[i+w*j]==1) {
                    /* have to check neighbourhood for junctions */
                    found=0;
                    count=0;
                    pass=1;
                    si=i;
                    sj=j;
                    for(ii= -1;ii<=1;ii++) {
                        for(jj= -1;jj<=1;jj++) {
                            if((ii==0)&&(jj==0)) continue;
                            if(im[i+ii+w*(j+jj)]==2) pass=0; /* use */
                            if(im[i+ii+w*(j+jj)]==1) count++;
                        }
                    }
                    if((pass==1)&&((count==1)||(count==2))) found=1;
                }
            }
        }
        if(found) {	/* texture line or short edge */
            im[si+w*sj]=numsegments+3;
            for(d=0;d<NUM_DIRECTIONS;d++) {
                ile_propagate(im,w,h,si,sj,d);
            }
            done=0;
            numsegments++;
        }
    }

	/* check that no 1 pixels remain */
	for(i=1;i<(w-1);i++) {
		for(j=1;j<(h-1);j++) {
			if(im[i+w*j]==1) {
/*
 * Modification by Brendan McCane to include isolated pixels as an edge
 * and edges with only two pixels in them.
 */
                im[i+w*j] = numsegments+3;
                for (ii=-1; ii<=1; ii++)
                for (jj=-1; jj<=1; jj++)
                {
                    if ((ii==0)&&(jj==0)) continue;
                    if (im[i+ii+(j+jj)*w]==1)
                        im[i+ii+(j+jj)*w] = im[i+w*j];
                }
                numsegments++;
			}
		}
	}
	iprs_debug_exit();
}
