/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_load_gifimage.c                                *****
*****   AUTHOR:       cdillon@uno.vis.citri.edu.au                        *****
*****   DATE:         05/09/91                                            *****
*****   COMMENTS:     This code loads a GIF image into an IPRS RGB image  *****
*****                 and is based on code modified from xv.              *****
*****                 We make no claim to maintaining it.                 *****
*****   MODIFICATIONS:                                                    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include<iprs.h>

#define EGA_DEFAULT	0	/* cdillon hack */

/*
 * xvgif.c  -  GIF loading code for 'xv'.  Based strongly on...
 *
 * gif2ras.c - Converts from a Compuserve GIF (tm) image to a Sun Raster image.
 *
 * Copyright (c) 1988, 1989 by Patrick J. Naughton
 *
 * Author: Patrick J. Naughton
 * naughton@wind.sun.com
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This file is provided AS IS with no warranties of any kind.  The author
 * shall have no liability with respect to the infringement of copyrights,
 * trade secrets or any patents by this file or any part thereof.  In no
 * event will the author be liable for any lost revenue or profits or
 * other special, indirect and consequential damages.
 *
 */

/*
 * Copyright 1989, 1990, 1991, 1992 by John Bradley and
 *                       The University of Pennsylvania
 *
 * Permission to use, copy, and distribute for non-commercial purposes,
 * is hereby granted without fee, providing that the above copyright
 * notice appear in all copies and that both the copyright notice and this
 * permission notice appear in supporting documentation.
 *
 * The software may be modified for your own purposes, but modified versions
 * may not be distributed.
 *
 * This software is provided "as is" without any expressed or implied warranty.
 *
 * The author may be contacted via:
 *    US Mail:   John Bradley
 *               GRASP Lab, Room 301C
 *               3401 Walnut St.
 *               Philadelphia, PA  19104
 *
 *    Phone:     (215) 898-8813
 *    EMail:     bradley@cis.upenn.edu
 */

/* This used to be xv.h */

  /* AIX and SysV r4 (but not sgi or UMAXV) use dirent */
# if   (defined(SVR4) && !defined(sgi) && !defined(__UMAXV__)) \
       || defined(_IBMR2) || defined(sco)
#  ifndef DIRENT
#   define DIRENT
#  endif
# endif


#if defined(SVR4) || defined(hpux)
#define GETWD(x) getcwd(x, sizeof(x))
#else
#define GETWD(x) getwd(x)
#endif


#if defined(apollo) || defined(pyr)
/* DomainOS 10.2 BSD4.3 version of str[r]chr is broken ([r]index works) */
/* pyramid bsd doesn't have str[r]chr */
#define strchr index
#define strrchr rindex
#endif

int   errno;            /* this SHOULD be in errno.h */
#ifdef _iamasgi			/* mods by cdillon */
char * sys_errlist[];
#else
const char *const sys_errlist[];  /* this SHOULD be in errno.h */
#endif

  /* lots of machines don't have <memory.h> */
# if !defined(__convex__) && \
     !defined(__UMAXV__)  && \
     !defined(pyr)        && \
     !defined(sequent)
# endif

/* lots of things don't have <iprs_malloc.h> */
/* A/UX systems include it from stdlib, from Xos.h */
# if !defined(ibm032)                    && \
     !defined(__convex__)                && \
     !(defined(vax) && !defined(ultrix)) && \
     !defined(mips)                      && \
     !defined(apollo)                    && \
     !defined(pyr)                       && \
     !defined(__UMAXV__)                 && \
     !defined(bsd43)                     && \
     !defined(macII)                     && \
     !defined(sequent)

#  if defined(hp300) || defined(hp800)
#  else
#  endif
# endif

#if defined(NEED_MEMROUTINES)
#define memcpy(d,s,l) bcopy(s,d,l)
#endif


#if defined(NEEDSTIME) || defined(NEEDSDIR)
#include <sys/types.h>    /* only include <sys/types.h> once */
#endif

#ifdef NEEDSTIME
#if defined(SVR4) || defined(macII)
#include <sys/time.h>
#else                    /* not SVR4 */
#if !defined(sgi) && !defined(__UMAXV__)   /* sgi & UMAX don't have timeb.h */
#include <sys/timeb.h>
#endif                   /* not sgi */
#endif                   /* not SVR4 */

#undef SIGCHLD           /* defined in both the Xos.h and signal.h */
#include <signal.h>
#if defined(sco) && !defined(NOTIMER)
#include <sys/itimer.h>
#endif

#ifndef  sigmask
#define  sigmask(m)      (1 << ((m)-1))
#endif

#endif                   /* NEEDSTIME */


/* signal macros */
#if defined(SVR4) && !defined(HPUX7) && !(defined(_AUX_SOURCE))
#define HOLD_SIG         sighold(SIGALRM)  /* block ALRM sig from occurring */
#define RELEASE_SIG      sigrelse(SIGALRM)
#define PAUSE_SIG        sigpause(SIGALRM) /* sleep until ALRM signal */
#else
#define HOLD_SIG         sigblock(sigmask(SIGALRM))
#define RELEASE_SIG      sigblock(0)
#define PAUSE_SIG        sigpause(0)
#endif

#ifdef __UMAXV__
#undef  PAUSE_SIG
#define PAUSE_SIG sigpause(SIGALRM)
#endif


/* default for most folks */
#define UNCOMPRESS "/usr/ucb/uncompress"   /* for uncompressing .Z files */

#if defined(hpux) || defined (SVR4)
#undef  UNCOMPRESS
#define UNCOMPRESS "/usr/bin/uncompress"   /* for uncompressing .Z files */
#endif

#if defined(i386) && !defined(sequent) && !defined(sun386) && !defined(_AIX)
#ifndef MAXPATHLEN
#define MAXPATHLEN    500
#endif
#undef  UNCOMPRESS
#define UNCOMPRESS    "/usr/local/bin/uncompress"   /* uncompress program */
#undef  HOLD_SIG
#define HOLD_SIG      /* don't know how to handle signals  MWS 10/18/90 */
#undef  RELEASE_SIG
#define RELEASE_SIG   /* */
#undef  PAUSE_SIG
#define PAUSE_SIG     /* */
#endif


#ifndef MAXPATHLEN
#define MAXPATHLEN 1024
#endif

#ifdef NO_RANDOM
#define random rand
#endif

/*****************************/
/* END OF CONFIGURATION INFO */
/*****************************/




#define PROGNAME  "xv"             /* used in resource database */

#define DEFINFOGEOM "-10+10"       /* default position of info window */
#define DEFCTRLGEOM "+400+400"     /* default position of ctrl window */
#define DEFGAMGEOM  "+10-10"       /* default position of gamma window */

#define INFOWIDE 500               /* (fixed) size of info window */
#define INFOHIGH 250

#define CTRLWIDE 440               /* (fixed) size of control window */
#define CTRLHIGH 345
#define CTRL_LISTW   (330-70)

#define DIRWIDE  300               /* (fixed) size of directory window */
#define DIRHIGH  420

#define MAXNAMES 4096              /* max # of files in ctrlW list */

/* strings in the INFOBOX (used in SetISTR and GetISTR) */
#define NISTR         9    /* number of ISTRs */
#define ISTR_INFO     0
#define ISTR_WARNING  1
#define ISTR_FILENAME 2
#define ISTR_FORMAT   3
#define ISTR_RES      4
#define ISTR_CROP     5
#define ISTR_EXPAND   6
#define ISTR_COLOR    7
#define ISTR_COLOR2   8

/* potential values of 'infomode', used in info box drawing routines */
#define INF_NONE 0    /* empty box */
#define INF_STR  1    /* just ISTR_INFO */
#define INF_PART 2    /* filename, format, size and infostr */
#define INF_FULL 3    /* INF_PART + clipping, expansion, colorinfo */


/* buttons in the ctrl window */
#define NBUTTS   28
#define BNEXT    0
#define BPREV    1
#define BCROP    2
#define BUNCROP  3
#define BNORM    4
#define BMAX     5
#define BUP2     6
#define BDN2     7
#define BUP10    8
#define BDN10    9
#define BQUIT    10
#define B4BY3    11
#define BSAVE    12
#define BROTL    13
#define BINFO    14
#define BGAMMA   15
#define BASPECT  16
#define BROTR    17
#define BMAXPECT 18
#define BACROP   19
#define BSMOOTH  20
#define BDITH    21
#define BRAW     22
#define BLOAD    23
#define BDELETE  24
#define BFLIPH   25
#define BFLIPV   26
#define BGRAB    27


/* buttons in the 'save' window */
#define S_NBUTTS  3
#define S_BOK     0
#define S_BCANC   1
#define S_BRESCAN 2


/* buttons in the 'gamma' window */
#define G_NBUTTS   24
#define G_BAPPLY   0
#define G_BNOGAM   1
#define G_BRESET   2
#define G_BCLOSE   3
#define G_BUP_BR   4
#define G_BDN_BR   5
#define G_BUP_CN   6
#define G_BDN_CN   7
#define G_B1       8
#define G_B2       9
#define G_B3       10
#define G_B4       11
#define G_BSET     12
#define G_BUNDO    13
#define G_BREDO    14
#define G_BCOLREV  15
#define G_BRNDCOL  16
#define G_BHSVRGB  17
#define G_BCOLUNDO 18
#define G_BRV      19
#define G_BMONO    20
#define G_BMAXCONT 21
#define G_BGETRES  22
#define G_BHISTEQ  23


/* constants for setting radio buttons in dirW */
#define F_COLORS    0
#define F_FORMAT    1

#define F_FULLCOLOR 0
#define F_GREYSCALE 1
#define F_BWDITHER  2
#define F_REDUCED   3

#define F_GIF       0
#define F_PM        1
#define F_PBMRAW    2
#define F_PBMASCII  3
#define F_XBM       4
#define F_SUNRAS    5
#define F_PS        6
#define F_IPRS      7 

#ifdef HAVE_JPEG
#define F_JPEG      8
#endif

#ifdef HAVE_TIFF
#ifdef HAVE_JPEG
#define F_TIFF      9
#else
#define F_TIFF      8
#endif
#endif


/* definitions for page up/down, arrow up/down list control */
#define LS_PAGEUP   0
#define LS_PAGEDOWN 1
#define LS_LINEUP   2
#define LS_LINEDOWN 3
#define LS_HOME     4
#define LS_END      5

/* values 'epicmode' can take */
#define EM_RAW    0
#define EM_DITH   1
#define EM_SMOOTH 2

/* things EventLoop() can return (0 and above reserved for 'goto pic#') */
#define QUIT     -1    /* exit immediately  */
#define NEXTPIC  -2    /* goto next picture */
#define PREVPIC  -3    /* goto prev picture */
#define NEXTQUIT -4    /* goto next picture, quit if none (used by 'wait') */
#define LOADPIC  -5    /* load 'named' pic (from directory box) */
#define NEXTLOOP -6    /* load next pic, loop if we're at end */
#define DFLTPIC  -7    /* load the default image */
#define DELETE   -8    /* just deleted pic.  load 'right' thing */
#define GRABBED  -9    /* just grabbed a pic.  'load' it up */


/* possible values of 'rootMode' */
#define RM_NORMAL  0     /* default X tiling */
#define RM_TILE    1     /* integer tiling */
#define RM_MIRROR  2     /* mirror tiling */
#define RM_IMIRROR 3     /* integer mirror tiling */
#define RM_CENTER  4     /* modes >= RM_CENTER centered on some sort of bg */
#define RM_CENTILE 4     /* centered and tiled.  NOTE: equals RM_CENTER */
#define RM_CSOLID  5     /* centered on a solid bg */
#define RM_CWARP   6     /* centered on a 'warp-effect' bg */
#define RM_CBRICK  7     /* centered on a 'brick' bg */
#define RM_MAX     RM_CBRICK

/* types of 24-to-8 conversion */
#define CONV24_FAST 0
#define CONV24_SLOW 1
#define CONV24_BEST 2

/* definitions of first char of dirnames[i] (filetype) */
#define C_FIFO  'f'    /* FIFO special file */
#define C_CHR   'c'    /* character special file */
#define C_DIR   'd'    /* directory */
#define C_BLK   'b'    /* block special file */
#define C_LNK   'l'    /* symbolic link */
#define C_SOCK  's'    /* socket */
#define C_REG   ' '    /* regular file */


/* random string-placing definitions */
#define SPACING 3      /* vertical space between strings */
#define ASCENT   (mfinfo->ascent)
#define DESCENT  (mfinfo->descent)
#define CHIGH    (ASCENT + DESCENT)
#define LINEHIGH (CHIGH + SPACING)

#define STDINSTR "<stdin>"

typedef unsigned char byte;

#define N_GFB 6
#define GFB_SPLINE 0
#define GFB_LINE   1
#define GFB_ADDH   2
#define GFB_DELH   3
#define GFB_RESET  4
#define GFB_GAMMA  5

#define GVMAX 8

/* MACROS */
#define CENTERX(f,x,str) ((x)-XTextWidth(f,str,strlen(str))/2)
#define CENTERY(f,y) ((y)-((f->ascent+f->descent)/2)+f->ascent)

/* RANGE forces a to be in the range b..c (inclusive) */
#define RANGE(a,b,c) { if (a < b) a = b;  if (a > c) a = c; }

/* PTINRECT returns '1' if x,y is in rect (inclusive) */
#define PTINRECT(x,y,rx,ry,rw,rh) \
           ((x)>=(rx) && (y)>=(ry) && (x)<=(rx)+(rw) && (y)<=(ry)+(rh))

/* MONO returns total intensity of r,g,b components */
#define MONO(rd,gn,bl) (((rd)*11 + (gn)*16 + (bl)*5) >> 5)  /*.33R+ .5G+ .17B*/

/* ISPIPE returns true if the passed in character is considered the
   start of a 'load-from-pipe' or 'save-to-pipe' string */
#define ISPIPE(c) ((c)=='!' || (c)=='|')

/* global vars used by LOAD routines */
 byte          *pic;                   /* ptr to loaded picture */
 int            pWIDE,pHIGH;           /* size of 'pic' */
 byte           r[256],g[256],b[256];  /* colormap */
 char          *cmd;                   /* program name for printf's */
 int            DEBUG;                 /* print debugging info */
 int            mono;                  /* true if displaying grayscale */
 char           formatStr[80];         /* short-form 'file format' */


/* more global variables, used by xv and xvmisc */
 byte          *cpic;         /* cropped version of pic */
 int           cWIDE, cHIGH,  /* size of cropped region */
                    cXOFF, cYOFF;  /* offset of region from 0,0 of pic */

 byte          *epic;         /* expanded version of cpic */
                                   /* points to pic when at 1:1 expansion */
                                   /* this is converted to 'theImage' */
 int           eWIDE, eHIGH;  /* size of epic */
 unsigned int  normFact;      /* factor to shrink picture by for 'norm' */

 int           p_offx, p_offy;  /* offset of reparented windows */
 int           ch_offx,ch_offy; /* ChngAttr ofst for reparented windows */

 byte           rorg[256], gorg[256], borg[256];  /* ORIGINAL colormap */
 byte           rcmap[256], gcmap[256], bcmap[256]; /*post-cmap-editing*/
 byte           rdisp[256],gdisp[256],bdisp[256];  /* DISPLAYED colors */
 byte           colAllocOrder[256];   /* order to allocate cols */
 unsigned long  iprs_freecols[256]; /* list of pixel values to iprs_free */
 byte           rwpc2pc[256]; /* mapping of shared pixels in -rw mode */
 int            nfcols;       /* number of colors to iprs_free */
 unsigned long  cols[256];    /* maps pic pixel values to X pixel vals */
 int            fc2pcol[256]; /* maps iprs_freecols into pic pixel values */
 int            numcols;      /* # of desired colors in picture */

 byte           fsgamcr[256]; /* gamma correction curve (for FS dither) */

 int           ncols;         /* max # of (different) colors to alloc */

 char          str[128],      /* dummy string used for error messages */
                    basefname[128]; /* 'nice' part of current filename */

 int           bwidth,        /* border width of created windows */
                    noglob,        /* force to only use colors it alloced */
                    perfect,       /* perfect color.  install own colormap */
                    fixedaspect,   /* fixed aspect ratio */
                    conv24,        /* 24to8 algorithm to use (CONV24_*) */
                    ninstall,      /* true if using icccm-complaint WM
				      (a WM that will does install CMaps */
                    useroot,       /* true if we should draw in rootW */
		    nolimits,	   /* No limits on picture size */
		    resetroot,     /* true if we should clear in window mode */
                    noqcheck,      /* true if we should NOT do QuickCheck */
                    rwcolor,       /* true if we should use R/W color cells */
                    rwthistime,    /* true if we DID use R/W color cells */
                    owncmap,       /* true if we should always create cmap */
                    epicmode,      /* either SMOOTH, DITH, or RAW */
                    autoclose,     /* if true, autoclose when iconifying */
                    xerrcode;      /* errorcode of last X error */

 float         defaspect,     /* default aspect ratio to use */
                    normaspect;    /* normal aspect ratio of this picture */

 int           crx1, cry1,    /* dimensions of cropping rectangle */
                    crx2, cry2;

 unsigned long rootbg, rootfg;   /* fg/bg for root border */
 int           waitsec;          /* secs btwn pics. -1=wait for event */
 int           waitloop;         /* loop at end of slide show? */
 int           automax;          /* maximize pic on open */
 int           rootMode;         /* mode used for -root images */

 int           nostat;           /* if true, don't stat() in LdCurDir */

 int           ctrlColor;        /* whether or not to use colored butts */

 char         *def_str;          /* used by rd_*() routines */
 int           def_int;
 char         *tmpdir;           /* equal to "/tmp" or $TMPDIR env var */


/* stuff used for 'info' box */
 int           infoUp;       /* boolean:  whether infobox is visible */
 int           infoMode;


/* stuff used for 'ctrl' box */
 int           ctrlUp;       /* boolean:  whether ctrlbox is visible */
 char         *namelist[MAXNAMES];  /* list of file names from argv */
 char         *dispnames[MAXNAMES]; /* truncated names shown in listbox */
 int           numnames, curname;

/* stuff used for 'directory' box */
 int           dirUp;       /* is dirW mapped or not */

/* stuff used for 'gamma' box */
 int           gamUp;       /* is gamW mapped or not */
 int           editColor;   /* currently selected color # */
 int           hsvmode;     /* true if in HSVmode */
 int cellgroup[256], curgroup, maxgroup;  /* grouped colorcell stuff */
 int           cmapInGam;

/* stuff used for 'ps' box */
 int           psUp;       /* is psW mapped, or what? */


#ifdef HAVE_JPEG
/* stuff used for 'jpeg' box */
 int           jpegUp;       /* is jpegW mapped, or what? */
#endif


#ifdef HAVE_TIFF
/* stuff used for 'tiff' box */
 int           tiffUp;       /* is tiffW mapped, or what? */
#endif

/* function declarations for ally-callable functions */

/****************************** XV.C ****************************/
void FixAspect(int, int *, int *);
int  DeleteCmd(void);
void HandleDispMode(void);
char *lower_str(char *);
int  rd_int(char *);
int  rd_str(char *);
int  rd_flag(char *);
int  rd_str_cl(char *, char *);

/****************************** XV.C ****************************/
void FixAspect(), HandleDispMode();
int  DeleteCmd(), rd_int(), rd_str(), rd_flag(), rd_str_cl();
char *lower_str();

/* xv.h ends here */

typedef int boolean;

#define NEXTBYTE (*ptr++)
#define EXTENSION     0x21
#define IMAGESEP      0x2c
#define TRAILER       0x3b
#define INTERLACEMASK 0x40
#define COLORMAPMASK  0x80

FILE *fp;

int BitOffset = 0,		/* Bit Offset of next code */
    XC = 0, YC = 0,		/* Output X and Y coords of current pixel */
    Pass = 0,			/* Used by output routine if interlaced pic */
    OutCount = 0,		/* Decompressor output 'stack count' */
    RWidth, RHeight,		/* screen dimensions */
    Width, Height,		/* image dimensions */
    LeftOfs, TopOfs,		/* image offset */
    BitsPerPixel,		/* Bits per pixel, read from GIF header */
    BytesPerScanline,		/* bytes per scanline in output raster */
    ColorMapSize,		/* number of colors */
    Background,			/* background color */
    CodeSize,			/* Code size, read from GIF header */
    InitCodeSize,		/* Starting code size, used during Clear */
    Code,			/* Value returned by ReadCode */
    MaxCode,			/* limiting value for current code size */
    ClearCode,			/* GIF clear code */
    EOFCode,			/* GIF end-of-information code */
    CurCode, OldCode, InCode,	/* Decompressor variables */
    FirstFree,			/* First iprs_free code, generated per GIF spec */
    FreeCode,			/* Decompressor,next iprs_free slot in hash table */
    FinChar,			/* Decompressor variable */
    BitMask,			/* AND mask for data size */
    ReadMask,			/* Code AND mask for current code size */
    Misc;               /* miscellaneous bits (interlace, local cmap)*/

boolean Interlace, HasColormap;

byte *RawGIF;			/* The heap array to hold it, raw */
byte *Raster;			/* The raster data stream, unblocked */

    /* The hash table used by the decompressor */
int Prefix[4096];
int Suffix[4096];

    /* An output array used by the decompressor */
int OutCode[4097];

int   gif89 = 0;
char *id87 = "GIF87a";
char *id89 = "GIF89a";

static int EGApalette[16][3] = {
  {0,0,0},       {0,0,128},     {0,128,0},     {0,128,128}, 
  {128,0,0},     {128,0,128},   {128,128,0},   {200,200,200},
  {100,100,100}, {100,100,255}, {100,255,100}, {100,255,255},
  {255,100,100}, {255,100,255}, {255,255,100}, {255,255,255} };

static int  ReadCode();
static void DoInterlace();
static int  GifError();

int filesize;

/*****************************/
int LoadGIF(fname,nc)
  char *fname;
  int   nc;
/*****************************/
{
  register byte  ch, ch1;
  register byte *ptr, *ptr1, *picptr;
  register int   i, block;
  int            npixels, maxpixels, aspect;

  /* initialize variables */
  BitOffset = XC = YC = Pass = OutCount = npixels = maxpixels = 0;
  RawGIF = Raster = pic = NULL;
  gif89 = 0;

  fp = fopen(fname,"r");
  if (!fp) {
    fprintf(stderr,"%s: LoadGIF() - unable to open file '%s'\n",cmd,fname);
    return 1;
  }
  
  /* find the size of the file */
  fseek(fp, 0L, 2);
  filesize = ftell(fp);
  fseek(fp, 0L, 0);
  
  /* the +256's are so we can read truncated GIF files without fear of 
     segmentation violation */
  if (!(ptr = RawGIF = (byte *) iprs_calloc(filesize+256,1)))
    return( GifError("not enough memory to read gif file") );
  
  if (!(Raster = (byte *) iprs_calloc(filesize+256,1)))    
    return( GifError("not enough memory to read gif file") );
  
  if (fread(ptr, filesize, 1, fp) != 1) 
    return( GifError("GIF data read failed") );
  
  if      (strncmp((char *) ptr, id87, 6)==0) gif89 = 0;
  else if (strncmp((char *) ptr, id89, 6)==0) gif89 = 1;
  else    return( GifError("not a GIF file"));
  
  ptr += 6;
  
  /* Get variables from the GIF screen descriptor */
  
  ch = NEXTBYTE;
  RWidth = ch + 0x100 * NEXTBYTE;	/* screen dimensions... not used. */
  ch = NEXTBYTE;
  RHeight = ch + 0x100 * NEXTBYTE;
  
  ch = NEXTBYTE;
  HasColormap = ((ch & COLORMAPMASK) ? 1 : 0);
  
  BitsPerPixel = (ch & 7) + 1;
  numcols = ColorMapSize = 1 << BitsPerPixel;
  BitMask = ColorMapSize - 1;
  
  Background = NEXTBYTE;		/* background color... not used. */
  
  aspect = NEXTBYTE;
  if (aspect) {
    if (!gif89) return(GifError("corrupt GIF file (screen descriptor)"));
    else normaspect = (float) (aspect + 15) / 64.0;   /* gif89 aspect ratio */
    if (DEBUG) fprintf(stderr,"GIF89 aspect = %f\n", normaspect);
  }
  
  
  /* Read in global colormap. */
  
  if (HasColormap)
    for (i=0; i<ColorMapSize; i++) {
      r[i] = NEXTBYTE;
      g[i] = NEXTBYTE;
      b[i] = NEXTBYTE;
    }
  else {  /* no colormap in GIF file */
#if EGA_DEFAULT
    /* put std EGA palette (repeated 16 times) into colormap, for lack of
       anything better to do */

    for (i=0; i<256; i++) {
      r[i] = EGApalette[i&15][0];
      g[i] = EGApalette[i&15][1];
      b[i] = EGApalette[i&15][2];
    }
#else
	for(i=0;i<256;i++) {
		r[i]=g[i]=b[i]=i;
	}
#endif
  }

  /* possible things at this point are:
   *   an application extension block
   *   a comment extension block
   *   an (optional) graphic control extension block
   *       followed by either an image
   *	   or a plaintext extension
   */

  while (1) {
    block = NEXTBYTE;

    if (block == EXTENSION) {  /* parse extension blocks */
      int i, fn, blocksize, aspnum, aspden;

      /* read extension block */
      fn = NEXTBYTE;

      if (DEBUG) fprintf(stderr,"GIF extension type 0x%02x\n", fn);

      if (fn == 'R') {                  /* GIF87 aspect extension */
	blocksize = NEXTBYTE;
	if (blocksize == 2) {
	  aspnum = NEXTBYTE;
	  aspden = NEXTBYTE;
	  if (aspden>0 && aspnum>0) 
	    normaspect = (float) aspnum / (float) aspden;
	  else { normaspect = 1.0;  aspnum = aspden = 1; }

	  if (DEBUG) fprintf(stderr,"GIF87 aspect extension: %d:%d = %f\n\n", 
			     aspnum, aspden,normaspect);
	}
	else {
	  for (i=0; i<blocksize; i++) NEXTBYTE;
	}
      }

      else if (fn == 0xFE) {  /* Comment Extension.  just eat it */
	int ch, j, sbsize;
	
	if (DEBUG) fprintf(stderr,"Comment extension:  ");
	/* read (and ignore) data sub-blocks */
	do {
	  j = 0;  sbsize = NEXTBYTE;
	  while (j<sbsize) {
	    ch = NEXTBYTE;  j++;
	    if (DEBUG) fprintf(stderr,"%c", ch);
	  }
	} while (sbsize);
	if (DEBUG) fprintf(stderr,"\n\n");
      }

      else if (fn == 0x01) {  /* PlainText Extension */
	int j,sbsize,ch;
	int tgLeft, tgTop, tgWidth, tgHeight, cWidth, cHeight, fg, bg;
      
	sbsize   = NEXTBYTE;
	tgLeft   = NEXTBYTE;  tgLeft   += (NEXTBYTE)<<8;
	tgTop    = NEXTBYTE;  tgTop    += (NEXTBYTE)<<8;
	tgWidth  = NEXTBYTE;  tgWidth  += (NEXTBYTE)<<8;
	tgHeight = NEXTBYTE;  tgHeight += (NEXTBYTE)<<8;
	cWidth   = NEXTBYTE;
	cHeight  = NEXTBYTE;
	fg       = NEXTBYTE;
	bg       = NEXTBYTE;
	i=12;
	for ( ; i<sbsize; i++) NEXTBYTE;   /* read rest of first subblock */
      
	if (DEBUG) fprintf(stderr,
	   "PlainText: tgrid=%d,%d %dx%d  cell=%dx%d  col=%d,%d\n",
	   tgLeft, tgTop, tgWidth, tgHeight, cWidth, cHeight, fg, bg);
	
	/* read (and ignore) data sub-blocks */
	do {
	  j = 0;
	  sbsize = NEXTBYTE;
	  while (j<sbsize) {
	    ch = NEXTBYTE;  j++;
	    if (DEBUG) fprintf(stderr,"%c", ch);
	  }
	} while (sbsize);
	if (DEBUG) fprintf(stderr,"\n\n");
      }


      else if (fn == 0xF9) {  /* Graphic Control Extension */
	int j, sbsize;

	if (DEBUG) fprintf(stderr,"Graphic Control extension\n\n");

	/* read (and ignore) data sub-blocks */
	do {
	  j = 0; sbsize = NEXTBYTE;
	  while (j<sbsize) { NEXTBYTE;  j++; }
	} while (sbsize);
      }
      

      else { /* unknown extension */
	int j, sbsize;

	if (DEBUG) fprintf(stderr,"unknown GIF extension 0x%02x\n\n", fn);

	/* read (and ignore) data sub-blocks */
	do {
	  j = 0; sbsize = NEXTBYTE;
	  while (j<sbsize) { NEXTBYTE;  j++; }
	} while (sbsize);
      }
    }
    
    else if (block == IMAGESEP) break;   /* read an image */

    else if (block == TRAILER) {
      return( GifError("no image data found in GIF file") );
    }

    else return (GifError("Unknown block type found in file."));
  }


  /* read in values from the image descriptor */
  
  ch = NEXTBYTE;
  LeftOfs = ch + 0x100 * NEXTBYTE;
  ch = NEXTBYTE;
  TopOfs = ch + 0x100 * NEXTBYTE;
  ch = NEXTBYTE;
  Width = ch + 0x100 * NEXTBYTE;
  ch = NEXTBYTE;
  Height = ch + 0x100 * NEXTBYTE;

  Misc = NEXTBYTE;
  Interlace = ((Misc & INTERLACEMASK) ? 1 : 0);

  if (Misc & 0x80) {
    for (i=0; i< 1 << ((Misc&7)+1); i++) {
      r[i] = NEXTBYTE;
      g[i] = NEXTBYTE;
      b[i] = NEXTBYTE;
    }
  }


  if (!HasColormap && !(Misc&0x80)) {
    /* no global or local colormap */
  }
    

  
  /* Start reading the raster data. First we get the intial code size
   * and compute decompressor constant values, based on this code size.
   */
  
  CodeSize = NEXTBYTE;

  ClearCode = (1 << CodeSize);
  EOFCode = ClearCode + 1;
  FreeCode = FirstFree = ClearCode + 2;
  
  /* The GIF spec has it that the code size is the code size used to
   * compute the above values is the code size given in the file, but the
   * code size used in compression/decompression is the code size given in
   * the file plus one. (thus the ++).
   */
  
  CodeSize++;
  InitCodeSize = CodeSize;
  MaxCode = (1 << CodeSize);
  ReadMask = MaxCode - 1;
  


  /* UNBLOCK:
   * Read the raster data.  Here we just transpose it from the GIF array
   * to the Raster array, turning it from a series of blocks into one long
   * data stream, which makes life much easier for ReadCode().
   */
  
  ptr1 = Raster;
  do {
    ch = ch1 = NEXTBYTE;
    while (ch--) { *ptr1 = NEXTBYTE; ptr1++; }
    if ((ptr - RawGIF) > filesize) {
      break;
    }
  } while(ch1);
  iprs_free(RawGIF);	 RawGIF = NULL; 	/* We're done with the raw data now */



  if (DEBUG) {
    fprintf(stderr,"xv: LoadGIF() - picture is %dx%d, %d bits, %sinterlaced\n",
	    Width, Height, BitsPerPixel, Interlace ? "" : "non-");
  }
  

  /* Allocate the 'pic' */
  pWIDE = Width;  pHIGH = Height;
  maxpixels = Width*Height;
  picptr = pic = (byte *) iprs_malloc(maxpixels);
  sprintf(formatStr, "%dx%d GIF%s.", pWIDE, pHIGH, (gif89) ? "89" : "87");

  if (!pic) 
    return( GifError("not enough memory for 'pic'") );

  
  /* Decompress the file, continuing until you see the GIF EOF code.
   * One obvious enhancement is to add checking for corrupt files here.
   */
  
  Code = ReadCode();
  while (Code != EOFCode) {
    /* Clear code sets everything back to its initial value, then reads the
     * immediately subsequent code as uncompressed data.
     */

    if (Code == ClearCode) {
      CodeSize = InitCodeSize;
      MaxCode = (1 << CodeSize);
      ReadMask = MaxCode - 1;
      FreeCode = FirstFree;
      Code = ReadCode();
      CurCode = OldCode = Code;
      FinChar = CurCode & BitMask;
      if (!Interlace) *picptr++ = FinChar;
         else DoInterlace(FinChar);
      npixels++;
    }
    else {
      /* If not a clear code, must be data: save same as CurCode and InCode */

      /* if we're at maxcode and didn't get a clear, stop loading */
      if (FreeCode>=4096) { /* printf("iprs_freecode blew up\n"); */
			    break; }

      CurCode = InCode = Code;
      
      /* If greater or equal to FreeCode, not in the hash table yet;
       * repeat the last character decoded
       */
      
      if (CurCode >= FreeCode) {
	CurCode = OldCode;
	if (OutCount > 4096) {  /* printf("outcount1 blew up\n"); */ break; }
	OutCode[OutCount++] = FinChar;
      }
      
      /* Unless this code is raw data, pursue the chain pointed to by CurCode
       * through the hash table to its end; each code in the chain puts its
       * associated output code on the output queue.
       */
      
      while (CurCode > BitMask) {
	if (OutCount > 4096) break;   /* corrupt file */
	OutCode[OutCount++] = Suffix[CurCode];
	CurCode = Prefix[CurCode];
      }
      
      if (OutCount > 4096) { /* printf("outcount blew up\n"); */ break; }
      
      /* The last code in the chain is treated as raw data. */
      
      FinChar = CurCode & BitMask;
      OutCode[OutCount++] = FinChar;
      
      /* Now we put the data out to the Output routine.
       * It's been stacked LIFO, so deal with it that way...
       */

      /* safety thing:  prevent exceeding range of 'pic' */
      if (npixels + OutCount > maxpixels) OutCount = maxpixels-npixels;
	
      npixels += OutCount;
      if (!Interlace) for (i=OutCount-1; i>=0; i--) *picptr++ = OutCode[i];
                else  for (i=OutCount-1; i>=0; i--) DoInterlace(OutCode[i]);
      OutCount = 0;

      /* Build the hash table on-the-fly. No table is stored in the file. */
      
      Prefix[FreeCode] = OldCode;
      Suffix[FreeCode] = FinChar;
      OldCode = InCode;
      
      /* Point to the next slot in the table.  If we exceed the current
       * MaxCode value, increment the code size unless it's already 12.  If it
       * is, do nothing: the next code decompressed better be CLEAR
       */
      
      FreeCode++;
      if (FreeCode >= MaxCode) {
	if (CodeSize < 12) {
	  CodeSize++;
	  MaxCode *= 2;
	  ReadMask = (1 << CodeSize) - 1;
	}
      }
    }
    Code = ReadCode();
    if (npixels >= maxpixels) break;
  }
  iprs_free(Raster);  Raster = NULL;
  
  if (npixels != maxpixels) {
    if (!Interlace)
      memset(pic+npixels, 0, maxpixels-npixels);  /* clear to EOBuffer */
  }

  if (fp != stdin) fclose(fp);


  return 0;
}


/* Fetch the next code from the raster data stream.  The codes can be
 * any length from 3 to 12 bits, packed into 8-bit bytes, so we have to
 * maintain our location in the Raster array as a BIT Offset.  We compute
 * the byte Offset into the raster array by dividing this by 8, pick up
 * three bytes, compute the bit Offset into our 24-bit chunk, shift to
 * bring the desired code to the bottom, then mask it off and return it. 
 */

static int ReadCode()
{
  int RawCode, ByteOffset;
  
  ByteOffset = BitOffset / 8;
  RawCode = Raster[ByteOffset] + (Raster[ByteOffset + 1] << 8);
  if (CodeSize >= 8)
    RawCode += ( ((int) Raster[ByteOffset + 2]) << 16);
  RawCode >>= (BitOffset % 8);
  BitOffset += CodeSize;

  return(RawCode & ReadMask);
}


/***************************/
static void DoInterlace(Index)
     byte Index;
{
  static byte *ptr = NULL;
  static int   oldYC = -1;
  
  if (oldYC != YC) {  ptr = pic + YC * Width;  oldYC = YC; }
  
  if (YC<Height)
    *ptr++ = Index;
  
  /* Update the X-coordinate, and if it overflows, update the Y-coordinate */
  
  if (++XC == Width) {
    
    /* deal with the interlace as described in the GIF
     * spec.  Put the decoded scan line out to the screen if we haven't gone
     * past the bottom of it
     */
    
    XC = 0;
    
    switch (Pass) {
    case 0:
      YC += 8;
      if (YC >= Height) { Pass++; YC = 4; }
      break;
      
    case 1:
      YC += 8;
      if (YC >= Height) { Pass++; YC = 2; }
      break;
      
    case 2:
      YC += 4;
      if (YC >= Height) { Pass++; YC = 1; }
      break;
      
    case 3:
      YC += 2;  break;
      
    default:
      break;
    }
  }
}
      
/*****************************/
static int GifError(st)
     char *st;
{
  fprintf(stderr,"%s: LoadGIF() - %s\n",cmd,st);

  if (RawGIF != NULL) iprs_free(RawGIF);
  if (Raster != NULL) iprs_free(Raster);

  if (pic    != NULL) iprs_free(pic);
  
  return -1;
}

/* The well written code starts here... */

IPRS_IMAGE *iprs_load_gifimage(char *fname)
{
	IPRS_IMAGE *imgptr;
	IPRS_RGB_IMAGE *rgbptr;
	int dummy;
	int WhatIsThisFor=256;
	int i,j;
	IPRS_RGB *ptr;

	iprs_debug_enter("iprs_load_gifimage");

/* here is where programmer's get down and pray... */
	dummy=LoadGIF(fname,WhatIsThisFor);
/* and now stand up again, hopefully in relief... */
	
	rgbptr=iprs_create_2Drgbimage(Width,Height);
	imgptr=iprs_create_image(1);
	imgptr->type[0]=IPRS_IMAGETYPE_RGB;
	imgptr->image[0]=(void *)rgbptr;
	ptr=rgbptr->image;
	for(j=0;j<Height;j++) {
		for(i=0;i<Width;i++) {
			ptr->r=r[pic[i+(Height-j-1)*Width]];
			ptr->g=g[pic[i+(Height-j-1)*Width]];
			ptr->b=b[pic[i+(Height-j-1)*Width]];
			ptr++;
		}
	}
	iprs_debug_exit();
	return(imgptr);
}
