/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_param.c                                        *****
*****   AUTHOR:       cdillon@cs.mu.oz.au                                 *****
*****   DATE:         11/09/91                                            *****
*****   COMMENTS:     functions for setting parameter arguments           *****
*****                 Most of these functions are implimented as          *****
*****                 #defines in the include files, not called directly  *****
*****   MODIFICATIONS: 07/04/93 - enumerated and arrays added             *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include "iprs.h"

void iprs_process_param(char **p) /* removes whitespace in front of names */
	/* This is a bit of a fudge for the load functions, but it is easier */
	/* to do it here as it means that others can use this function if they */
	/* need to in a graphical user interface or whatever... */
{
	char *pp;
	int i,j,inname;

	if(!p) return;
	if(!(*p)) return;

	i=0;
	pp=*p;
	inname=1;
	while(pp[i]) {
		if(inname) {
			if((pp[i]==' ')||(pp[i]==8)||(pp[i]==9)) {
				j=i;
				while(1) {
					pp[j]=pp[j+1];
					if(pp[j]==0) break;
					j++;
				}
				continue;
			} else {
				if(pp[i]=='=') inname=0;
				i++;
			}
		} else {
			if(pp[i]==';') inname=1;
			i++;
		}
	}
}

void iprs__setsparam(char **p,char *pname,char *var)
{
	char *newp,*pp;
	int nl,i,s,t,tt;
	char n[256];

/* 	if(iprs_param_flag==IPRS_FALSE) return; */
	if((!pname)||(!var)) { 
		fprintf(stderr, "error\n");
		return;
	}
	s=0;
	pp=*p;
	if(!pp) { /* if no parameter string, create */
		nl=strlen(pname)+strlen(var)+3;
		newp=(char *)iprs_calloc(nl,sizeof(char));
		if(!newp) return;	/* just return on memory error */
		strcpy(newp,pname);
		strcat(newp,"=");
		strcat(newp,var);
		strcat(newp,";");
		*p=newp;
		return;
	}
	while(pp[s]) {
		t=0;
		tt= -1;
		while(1) { 
			if(pp[s+t]==0) break;
			if(pp[s+t]==';') {
				t++;
				break;
			}
			n[t]=pp[s+t];
			if(n[t]=='=') tt=t;
			t++;
		}
		n[t]=0;
		if(t) {
			if(tt<0) {
				fprintf(stderr, "error\n");
				return;
			}
			n[tt]=0;
			if(!strcmp(n,pname)) {	/* found label already */
				nl=strlen(pp)-t+strlen(pname)+strlen(var)+3;
				newp=(char *)iprs_calloc(nl,sizeof(char));
				if(!newp) return; /* just return on memory error */
				for(i=0;i<s;i++) newp[i]=pp[i];
				pp[i]=0;
				strcat(newp,pname);
				strcat(newp,"=");
				strcat(newp,var);
				strcat(newp,";");
				strcat(newp,&(pp[s+t]));
				iprs_free(*p);
				*p=newp;
				return;
			}
		}
		s+=t;
	}
	nl=strlen(pp)+strlen(pname)+strlen(var)+3;
	newp=(char *)iprs_calloc(nl,sizeof(char));
	if(!newp) return; /* return on memory error */
	strcpy(newp,pp);
	strcat(newp,pname);
	strcat(newp,"=");
	strcat(newp,var);
	strcat(newp,";");
	iprs_free(*p);
	*p=newp;
	return;
}

void iprs__getsparam(char **p,char *pname,char *var)
{
	char *pp;
	int nl,i,s,t,tt;
	char n[256];

	if((!p)||(!pname)||(!var)) return;
	s=0;
	pp=*p;
	if(!pp) {
		var[0]=0;
		return;
	}
	while(pp[s]) {
		t=0;
		tt= -1;
		while(1) { 
			if(pp[s+t]==0) break;
			if(pp[s+t]==';') {
				t++;
				break;
			}
			n[t]=pp[s+t];
			if(n[t]=='=') tt=t;
			t++;
		}
		n[t]=0;
		if(t) {
			if(tt<0) {
				fprintf(stderr, "error\n");
				return;
			}
			n[tt]=0;
			if(!strcmp(n,pname)) {	/* found label */
				nl=strlen(pp)-t+strlen(pname)+strlen(var)+3;
				for(i=0;i<t-tt-2;i++) var[i]=pp[s+tt+i+1];
				var[i]=0;
				return;
			}
		}
		s+=t;
	}
	var[0]=0; /* parameter not found */
	return;
}

void iprs__delparam(char **p,char *pname)
{
	char *newp,*pp;
	int nl,i,s,t,tt;
	char n[256];

	if(!pp) {
		return;
	}
	if(!pname) { /* free all parameters */
		if(*p) iprs_free(*p);
		*p=(char *)NULL;
		return;
	}
	s=0;
	pp=*p;
	while(pp[s]) {
		t=0;
		tt= -1;
		while(1) { 
			if(pp[s+t]==0) break;
			if(pp[s+t]==';') {
				t++;
				break;
			}
			n[t]=pp[s+t];
			if(n[t]=='=') tt=t;
			t++;
		}
		n[t]=0;
		if(t) {
			if(tt<0) {
				fprintf(stderr, "error\n");
				return;
			}
			n[tt]=0;
			if(!strcmp(n,pname)) {	/* found label */
				nl=strlen(pp)-t+1;
				if(nl) {
					newp=(char *)iprs_calloc(nl,sizeof(char));
					if(!newp) return; /* return on memory error */
					for(i=0;i<s;i++) newp[i]=pp[i];
					pp[i]=0;
					strcat(newp,&(pp[s+t]));
					iprs_free(*p);
					*p=newp;
				} else {
					iprs_free(*p);
					*p=NULL;
				}
				return;
			}
		}
		s+=t;
	}
	return;
}

void iprs__setiparam(char **p,char *name,int value)
{
	char s[256];

	sprintf(s,"%d",value);
	iprs__setsparam(p,name,s);
}

int iprs__getiparam(char **p,char *name)
{
	char s[256];
	int i;

	iprs__getsparam(p,name,s);
	if(s[0]) {
		sscanf(s,"%d",&i);
		return(i);
	}
	return 0; /* default value - if this is not zero, loading file version */
			  /* will not work!.......                                     */
}

void iprs__setfparam(char **p,char *name,float value)
{
	char s[256];

	sprintf(s,"%f",value);
	iprs__setsparam(p,name,s);
}

float iprs__getfparam(char **p,char *name)
{
	char s[256];
	float f;

	iprs__getsparam(p,name,s);
	if(s[0]) {
		sscanf(s,"%f",&f);
		return(f);
	}
	return 0.0;	/* default value */
}

void iprs__setaiparam(char **p,char *name,int valuenum,int value)
{
	char s[256];

	sprintf(s,"%s[%d]",name,valuenum);
	iprs__setiparam(p,s,value);
}


int iprs__getaiparam(char **p,char *name,int valuenum)
{
	char s[256];

	sprintf(s,"%s[%d]",name,valuenum);
	return(iprs__getiparam(p,s));
}

void iprs__setall_iparam(char **p,char *name,int numvalues,int *array)
{
	char s[256];
	int i;

	for(i=0;i<numvalues;i++) {
		sprintf(s,"%s[%d]",name,i);
		iprs__setiparam(p,s,array[i]);
	}
}

void iprs__getall_iparam(char **p,char *name,int numvalues,int *value)
{
	char s[256];
	int i;

	for(i=0;i<numvalues;i++) {
		sprintf(s,"%s[%d]",name,i);
		value[i]=iprs__getiparam(p,s);
	}
}

void iprs__setafparam(char **p,char *name,int valuenum,float value)
{
	char s[256];

	sprintf(s,"%s[%d]",name,valuenum);
	iprs__setfparam(p,s,value);
}


float iprs__getafparam(char **p,char *name,int valuenum)
{
	char s[256];

	sprintf(s,"%s[%d]",name,valuenum);
	return(iprs__getfparam(p,s));
}

void iprs__setall_fparam(char **p,char *name,int numvalues,float *array)
{
	char s[256];
	int i;

	for(i=0;i<numvalues;i++) {
		sprintf(s,"%s[%d]",name,i);
		iprs__setfparam(p,s,array[i]);
	}
}

void iprs__getall_fparam(char **p,char *name,int numvalues,float *value)
{
	char s[256];
	int i;

	for(i=0;i<numvalues;i++) {
		sprintf(s,"%s[%d]",name,i);
		value[i]=iprs__getfparam(p,s);
	}
}

void iprs__setasparam(char **p,char *name,int valuenum,char *value)
{
	char s[256];

	sprintf(s,"%s[%d]",name,valuenum);
	iprs__setsparam(p,s,value);
}


void iprs__getasparam(char **p,char *name,int valuenum,char *var)
{
	char s[256];

	sprintf(s,"%s[%d]",name,valuenum);
	iprs__getsparam(p,s,var);
}

void iprs__setall_sparam(char **p,char *name,int numvalues,char **array)
{
	char s[256];
	int i;

	for(i=0;i<numvalues;i++) {
		sprintf(s,"%s[%d]",name,i);
		iprs__setsparam(p,s,array[i]);
	}
}

void iprs__getall_sparam(char **p,char *name,int numvalues,char **value)
{
	char s[256];
	int i;

	for(i=0;i<numvalues;i++) {
		sprintf(s,"%s[%d]",name,i);
		iprs__getsparam(p,s,value[i]);
	}
}

void iprs_popparam_r(IPRS_IMAGE *iptr,IPRS_IMAGE *dptr,char *param)
{
	int i;
	char s[256];

	for(i=0;i<iptr->numimages;i++) {
		switch(iptr->type[i]) {
			case IPRS_IMAGETYPE_IMAGE:
				iprs_popparam_r(iptr->image[i],dptr,param);
				break;

			case IPRS_IMAGETYPE_BYTE:
				iprs_getsparam((IPRS_BYTE_IMAGE *)(iptr->image[i]),param,s);
				break;

			case IPRS_IMAGETYPE_INT:
				iprs_getsparam((IPRS_INT_IMAGE *)(iptr->image[i]),param,s);
				break;

			case IPRS_IMAGETYPE_FLOAT:
				iprs_getsparam((IPRS_FLOAT_IMAGE *)(iptr->image[i]),param,s);
				break;

			case IPRS_IMAGETYPE_RGB:
				iprs_getsparam((IPRS_RGB_IMAGE *)(iptr->image[i]),param,s);
				break;

			default:
				fprintf(stderr, "error\n");
				break;
		}		
		if(s[0]) {
			iprs_setsparam(dptr,param,s);
		}
	}
}

void iprs_popparam(IPRS_IMAGE *iptr,char *param)
{
	iprs_debug_enter("iprs_popparam");
	if(!param) {
		fprintf(stderr, "error\n");
		return;
	}
	IPRS_STDERR_IMAGE(iptr);
	iprs_popparam_r(iptr,iptr,param);
	iprs_debug_exit();
}

void iprs_pushparam_r(IPRS_IMAGE *iptr,IPRS_IMAGE *sptr,char *param)
{
	int i;
	char s[256];

	iprs_getsparam(sptr,param,s);
	if(s[0]) {
		for(i=0;i<iptr->numimages;i++) {
			switch(iptr->type[i]) {
				case IPRS_IMAGETYPE_IMAGE:
					iprs_pushparam_r(iptr->image[i],sptr,param);
					break;
	
				case IPRS_IMAGETYPE_BYTE:
					iprs_setsparam((IPRS_BYTE_IMAGE *)(iptr->image[i]),param,s);
					break;
	
				case IPRS_IMAGETYPE_INT:
					iprs_setsparam((IPRS_INT_IMAGE *)(iptr->image[i]),param,s);
					break;
	
				case IPRS_IMAGETYPE_FLOAT:
					iprs_setsparam((IPRS_FLOAT_IMAGE *)
						(iptr->image[i]),param,s);
					break;
	
				case IPRS_IMAGETYPE_RGB:
					iprs_setsparam((IPRS_RGB_IMAGE *)(iptr->image[i]),param,s);
					break;

				case IPRS_IMAGETYPE_REGION:
					iprs_setsparam((IPRS_REGION_IMAGE *)(iptr->image[i]),
						param,s);
					break;

				case IPRS_SPACETYPE_FEATURE:
					iprs_setsparam((IPRS_FEATURE_SPACE *)(iptr->image[i]),
						param,s);
					break;
	
				default:
					fprintf(stderr, "error\n");
					break;
			}
		}
	}
}

void iprs_pushparam(IPRS_IMAGE *iptr,char *param)
{
	iprs_debug_enter("iprs_pushparam");
	if(!param) {
		fprintf(stderr, "error\n");
		return;
	}
	IPRS_STDERR_IMAGE(iptr);
	iprs_pushparam_r(iptr,iptr,param);
	iprs_debug_exit();
}

void iprs__seteparam(char **p,char *name,int value,char **estring)
{
	iprs__setsparam(p,name,estring[value]);
}

int iprs__geteparam(char **p,char *name,char **estring)
{
	int i;
	char s[256];

	iprs__getsparam(p,name,s);
	for(i=0;strlen(estring[i]);i++) {
		if(!strcmp(estring[i],s)) {
			return i; /* found it */
		}
	}
	return -1; /* did not find it */
}

char *iprs_strdup(char *s)
{
	char *r;
	int i;

	if(!s) return NULL;
	if(!strlen(s)) return NULL;
	r=malloc(strlen(s)+1);
	for(i=0;s[i];i++) r[i]=s[i];
	r[i]=0;
	return r;
}

char *iprs_ccopy_parameter(char *p)
/* Create a copy of the parameter string */
{
	char *s;

	if(!p) return NULL;
	if(!strlen(p)) return NULL;
	s=iprs_strdup(p);
	return s;
}	
