/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_recover_edges.c                                *****
*****   AUTHOR:       cdillon@uno.vis.citri.edu.au                        *****
*****   DATE:         21/09/91                                            *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS:                                                    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include<iprs.h>

/* junction_filter is of form 012 */
/*						      3 4 */
/*							  576 */

/* These functions load a file from iprs/files called junction_filter */
/* which is basically a lookup-table. See the files in that directory */
/* for more information.                                              */

void iprs_recover_edgesintint(
	IPRS_INT_IMAGE *sptr,
	IPRS_INT_IMAGE *dptr)
{
	int i,j,done;
	int w,h;
	int *sim,*dim;
	int numlabs;
	int jmap[256],n;
	char fn[256];
	FILE *in;

	iprs_debug_enter("iprs_recover_edgesintint");
	if(sptr->numdim!=2) {
		fprintf(stderr,"IPRS_NOT_2D in iprs_recover_edgesintint\n");
		return;
	}
	w=sptr->dim[0];
	h=sptr->dim[1];
	sim=sptr->image;
	dim=dptr->image;

	for(i=0;i<w*h;i++) dim[i]=0;

	for(i=1;i<w;i++) {		
		for(j=1;j<h;j++) {
			if((sim[i+w*j]!=sim[i-1+w*j])||(sim[i+w*j]!=sim[i+w*(j-1)])) dim[i+w*j]=1;
		}
	}
	/* gives crude edges - pass through the junction filter */
	sprintf(fn,"junction_filter");
	in=fopen(fn,"r");
	if(!in) {
		fprintf(stderr, "couldn't open junction filter in iprs_thin_edges\n");
		return;
	}
	for(i=0;i<256;i++) {
		fscanf(in," %d",&(jmap[i]));
	}
	fclose(in);
	for(i=1;i<(w-1);i++) {
        for(j=1;j<(h-1);j++) {
			if(dim[i+j*w]) {
				n=dim[i-1+w*(j-1)];
				n+=2*dim[i+w*(j-1)];
				n+=4*dim[i+1+w*(j-1)];
				n+=8*dim[i+1+w*j];
				n+=16*dim[i+1+w*(j+1)];
				n+=32*dim[i+w*(j+1)];
				n+=64*dim[i-1+w*(j+1)];
				n+=128*dim[i-1+w*j];
				dim[i+j*w]=jmap[n];
			}
		}
	}
	iprs_debug_exit();
}

/* junction_filter is of form 012 			*/
/*						      3 4 			*/
/*							  576 			*/
/* And file we use file junction_filter2 	*/
/* This has the affect of skeletonisation	*/

void iprs_thin_edges(IPRS_BYTE_IMAGE *ptr)
{
	int i,j,done;
	int w,h;
	char *im;
	int numchanges;
	int jmap[256],n;
	FILE *in;
	int minx,maxx,miny,maxy;
	char fn[256];

	w=ptr->dim[0];
	h=ptr->dim[1];
	im=ptr->image;

	/* gives crude edges - pass through the junction filter */
	sprintf(fn,"junction_filter2");
	in=fopen(fn,"r");
	if(!in) {
		fprintf(stderr, "couldn't open junction filter in iprs_thin_edges\n");
		return;
	}
	for(i=0;i<256;i++) {
		fscanf(in," %d",&(jmap[i]));
	}
	fclose(in);
	done=0;	
	numchanges=0;
	for(i=0;i<w;i++) {
		for(j=0;j<h;j++) {
			if(im[i+j*w]!=0) im[i+j*w]=1;
		}
	}
	minx=1;
	maxx=w-2;
	miny=1;
	maxy=h-2;
	while(!done) {
		done=1;
		for(i=minx;i<=maxx;i++) {
       		for(j=miny;j<=maxy;j++) {
				if(im[i+j*w]) {
					n=(im[i-1+w*(j-1)]!=0);
					n+=2*(im[i+w*(j-1)]!=0);
					n+=4*(im[i+1+w*(j-1)]!=0);
					n+=8*(im[i+1+w*j]!=0);
					n+=16*(im[i+1+w*(j+1)]!=0);
					n+=32*(im[i+w*(j+1)]!=0);
					n+=64*(im[i-1+w*(j+1)]!=0);
					n+=128*(im[i-1+w*j]!=0);
					if(!jmap[n]) {
						im[i+j*w]=0;
						done=0;
						numchanges++;
					}
				}
			}
		}
		printf("%d\n",numchanges);
	}
}
