/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:          iprs_rm_space.c                                    *****
*****   AUTHOR:        cdillon@uno.vis.citri.edu.au                       *****
*****   DATE:          05/09/91                                           *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS:                                                    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include "iprs.h"

void iprs_rmfeature_featurespace(fsptr,fnum)
IPRS_FEATURE_SPACE *fsptr;
int fnum;
{
	int i,j;

	for(i=0;i<fsptr->numpoints;i++) {
		for(j=fnum;j<(fsptr->numfeatures-1);j++) {
			fsptr->point[i][j]=fsptr->point[i][j+1];
		}
	}
	(fsptr->numfeatures)--;
}

void iprs_rmpoint_featurespace(fsptr,pnum)
IPRS_FEATURE_SPACE *fsptr;
int pnum;
{
	int i;

	if((pnum<0)||(pnum>=fsptr->numpoints)) {
		fprintf(stderr, "Point number out of range");
		return;
	}
	if(fsptr->point[pnum]) iprs_free(fsptr->point[pnum]);
	for(i=pnum;i<(fsptr->numpoints-1);i++) {
		fsptr->point[i]=fsptr->point[i+1];
	}
	(fsptr->numpoints)--;
}

void iprs_rmpoint_labelspace(
	IPRS_LABEL_SPACE *lsptr,
	int pnum)
{
    int i;

	iprs_debug_enter("iprs_rmpoint_labelspace");
	if((pnum<0)||(pnum>=lsptr->numpoints)) {
		fprintf(stderr, "error\n");
		return;
	}
	for(i=0;i<(lsptr->numlabels[pnum]);i++) {
		lsptr->ptsperlabel[lsptr->label[pnum][i]-lsptr->minlabel]--;
	}
	if(lsptr->label[pnum]) iprs_free(lsptr->label[pnum]);
    for(i=pnum;i<(lsptr->numpoints-1);i++) {
        lsptr->label[i]=lsptr->label[i+1];
		lsptr->numlabels[i]=lsptr->numlabels[i+1];
    }
    (lsptr->numpoints)--;
	iprs_debug_exit();
}   

void iprs_rmlabel_labelspace(lsptr,pnum,lnum)
IPRS_LABEL_SPACE *lsptr;
int pnum,lnum;
{
    int i;

    for(i=0;i<lsptr->numlabels[pnum];i++) {
		if(lsptr->label[pnum][i]==lnum) break;
	}
	if(i!=lsptr->numlabels[pnum]) {
		(lsptr->numlabels[pnum])--;
		(lsptr->ptsperlabel[lsptr->label[pnum][i]-lsptr->minlabel])--;
	}
	while(i<lsptr->numlabels[pnum]) {
		lsptr->label[pnum][i]=lsptr->label[pnum][i+1];
		i++;
	}
}

void iprs_rmpoint_ruleactspace(
    IPRS_RULEACT_SPACE *rasptr,
    int pnum)
{
    int i;

    iprs_debug_enter("iprs_rmpoint_ruleactspace");
	if((pnum<0)||(pnum>=rasptr->numpoints)) {
		fprintf(stderr, "error\n");
		return;
	}
    if(rasptr->rule[pnum]) iprs_free(rasptr->rule[pnum]);
    for(i=pnum;i<(rasptr->numpoints-1);i++) {
        rasptr->rule[i]=rasptr->rule[i+1];
    }
    (rasptr->numpoints)--;
    iprs_debug_exit();
}

void iprs_rmpartition_partitionspace(
    IPRS_PARTITION_SPACE *psptr,
    int pnum)
{
    int i;

    iprs_debug_enter("iprs_rmpartition_partitionspace");
	if((pnum<0)||(pnum>=psptr->numpartitions)) {
		fprintf(stderr, "error\n");
		return;
	}
    if(psptr->partition[pnum]->lower) iprs_free(psptr->partition[pnum]->lower);
    if(psptr->partition[pnum]->upper) iprs_free(psptr->partition[pnum]->upper);
    iprs_free(psptr->partition[pnum]);
    for(i=pnum;i<(psptr->numpartitions-1);i++) {
        psptr->partition[i]=psptr->partition[i+1];
    }
    (psptr->numpartitions)--;
    iprs_debug_exit();
}
