/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_rw.c                                           *****
*****   AUTHOR:       cdillon@cs.mu.oz.au                                 *****
*****   DATE:         06/09/91                                            *****
*****   COMMENTS:                                                         *****
*****   MODIFICATIONS: 29/01/93 rewritten on fread/fwrite                 *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include "iprs.h"

#ifdef BIG_ENDIAN

void iprs_read_bigendian(
	FILE *file,
	void *buf,
	unsigned nbytes)
{
	fread((char *)buf,sizeof(char),nbytes,file);
	if(feof(file)) {
		fprintf(stderr, "error\n");
	}
	if(ferror(file)) {
		fprintf(stderr, "error\n");
	}
}

#endif

#ifdef LITTLE_ENDIAN

void iprs_read_char(
    FILE *file,
    char *buf,
    unsigned nbytes)
{
	fread((char *)buf,sizeof(char),nbytes,file);
	if(feof(file)) {
		fprintf(stderr, "error\n");
	}
	if(ferror(file)) {
		fprintf(stderr, "error\n");
	}
}

void iprs_read_int(
    FILE *file,
    int *buf,
    unsigned nitems)
{
	int i;
	char *a,b;

	iprs_read_char(file,(char *)buf,nitems*sizeof(int));
	for(i=0;i<nitems;i++) {
		a=(char *)&(buf[i]);
		b=a[0];
		a[0]=a[3];
		a[3]=b;
		b=a[1];
		a[1]=a[2];
		a[2]=b;
	}
}

void iprs_read_float(
    FILE *file,
    float *buf,
    unsigned nitems)
{
    int i;
    char *a,b;

    iprs_read_char(file,(char *)buf,nitems*sizeof(float));
    for(i=0;i<nitems;i++) {
        a=(char *)&(buf[i]);
        b=a[0];
        a[0]=a[3];
        a[3]=b;
        b=a[1];
        a[1]=a[2];
        a[2]=b;
    }
}

void iprs_write_int(
    FILE *file,
    int *buf,
    unsigned nitems)
{
	int i;
	char *a,b[4];

	for(i=0;i<nitems;i++) {
		a=(char *)&(buf[i]);
		b[0]=a[3];
		b[1]=a[2];
		b[2]=a[1];
		b[3]=a[0];
		fwrite(b,sizeof(char),4,file);
	}
}

void iprs_write_float(
    FILE *file,
    float *buf,
    unsigned nitems)
{
    int i;
    char *a,b[4];

    for(i=0;i<nitems;i++) {
        a=(char *)&(buf[i]);
        b[0]=a[3];
        b[1]=a[2];
        b[2]=a[1];
        b[3]=a[0];
        fwrite(b,sizeof(char),4,file);
    }
}

#endif
