/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:         iprs_to.c                                           *****
*****   AUTHOR:       cdillon@uno.vis.citri.edu.au                        *****
*****   DATE:         05/09/91                                            *****
*****   COMMENTS:     All the image conversion routines                   *****
*****   MODIFICATIONS:                                                    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#include<iprs.h>
#include "iprs_to.proto"

void iprs__byteint(
	IPRS_BYTE_IMAGE *sptr,
	IPRS_INT_IMAGE *dptr)
{
	int i,n;
	char *s;
	int *d;

	for(n=1,i=0;i<sptr->numdim;i++) n*=sptr->dim[i];
	s=sptr->image;
	d=dptr->image;
	for(i=0;i<n;i++) {
		*d = (int) *s;
		d++;
		s++;
	}
}

IPRS_INT_IMAGE *iprs_bytetoint(IPRS_BYTE_IMAGE *sptr)
{
	IPRS_INT_IMAGE *dptr;

	iprs_debug_enter("iprs_bytetoint");
	if(!sptr) {
		fprintf(stderr, "error in iprs_to.c\n");
		return NULL;
	}
	if(!sptr->image) {
		fprintf(stderr, "error in iprs_to.c\n");
		return NULL;
	}
	dptr=iprs_create_intimage(sptr->numdim,sptr->dim);
	iprs__byteint(sptr,dptr);
	iprs_debug_exit();
	return(dptr);
}

void iprs__rgbint(
	IPRS_RGB_IMAGE *sptr,
	IPRS_INT_IMAGE *dptr)
{
	int i,n;
	IPRS_RGB *s;
	int *d;

	for(n=1,i=0;i<sptr->numdim;i++) n*=sptr->dim[i];
	s=sptr->image;
	d=dptr->image;
	for(i=0;i<n;i++) {
		*d = (int)(s->r+s->g+s->b)/3;
		d++;
		s++;
	}
}

IPRS_INT_IMAGE *iprs_rgbtoint(IPRS_RGB_IMAGE *sptr)
{
    IPRS_INT_IMAGE *dptr;

    iprs_debug_enter("iprs_rgbtoint");
    if(!sptr) {
        fprintf(stderr, "error in iprs_to.c\n");
        return NULL;
    }
    if(!sptr->image) {
        fprintf(stderr, "error in iprs_to.c\n");
        return NULL;
    }
    dptr=iprs_create_intimage(sptr->numdim,sptr->dim);
    iprs__rgbint(sptr,dptr);
    iprs_debug_exit();
    return(dptr);
}

void iprs__floatint(
	IPRS_FLOAT_IMAGE *sptr,
	IPRS_INT_IMAGE *dptr)
{
	int i,n;
	float *s;
	int *d;

	for(n=1,i=0;i<sptr->numdim;i++) n*=sptr->dim[i];
	s=sptr->image;
	d=dptr->image;
	for(i=0;i<n;i++) {
		*d = (int) *s;
		d++;
		s++;
	}
}

IPRS_INT_IMAGE *iprs_floattoint(IPRS_FLOAT_IMAGE *iptr)
{
	IPRS_INT_IMAGE *bptr;

	iprs_debug_enter("iprs_floattoint");
	if(!iptr) {
		fprintf(stderr, "error in iprs_to.c\n");
		return NULL;
    }   
    if(!iptr->image) {
		fprintf(stderr, "error in iprs_to.c\n");
		return NULL;
    }
	bptr=iprs_create_intimage(iptr->numdim,iptr->dim);
	iprs__floatint(iptr,bptr);
	iprs_debug_exit();
	return(bptr);
}

IPRS_IMAGE *iprs_imagetoint(IPRS_IMAGE *iptr)
{
    int i;
    IPRS_IMAGE *ptr;

	iprs_debug_enter("iprs_imagetoint");
    if(!iptr) {
		fprintf(stderr, "error in iprs_to.c\n");
		return NULL;
    }
    ptr=iprs_create_image(iptr->numimages);
    for(i=0;i<iptr->numimages;i++) {
        switch(iptr->type[i]) {
            case IPRS_IMAGETYPE_IMAGE:
                ptr->image[i]=iprs_imagetoint(iptr->image[i]);
                ptr->type[i]=IPRS_IMAGETYPE_IMAGE;
                break;

            case IPRS_IMAGETYPE_BYTE:
                ptr->image[i]=iprs_bytetoint(iptr->image[i]);
                ptr->type[i]=IPRS_IMAGETYPE_INT;
                break;

            case IPRS_IMAGETYPE_INT:
                ptr->image[i]=iprs_ccopy_intimage(iptr->image[i]);
                ptr->type[i]=IPRS_IMAGETYPE_INT;
                break;

            case IPRS_IMAGETYPE_RGB:
                ptr->image[i]=iprs_rgbtoint(iptr->image[i]);
                ptr->type[i]=IPRS_IMAGETYPE_INT;
                break;
            
			case IPRS_IMAGETYPE_FLOAT:
                ptr->image[i]=iprs_floattoint(iptr->image[i]);
                ptr->type[i]=IPRS_IMAGETYPE_INT;
                break;

            default:
				fprintf(stderr, "error in iprs_to.c\n");
				return NULL;
        }
    }
	iprs_debug_exit();
    return(ptr);
}
