/*****************************************************************************\
*******************************************************************************
*******                                                                 *******
*****   FILE:             list.h                                          *****
*****   AUTHOR:           brendan (mccane@coral.cs.jcu.edu.au)            *****
*****   DATE:             23/7/93                                         *****
*****   COMMENTS:         Generic linked list structure.                  *****
*****   MODIFICATIONS:                                                    *****
*******                                                                 *******
*******************************************************************************
\*****************************************************************************/

#ifndef BREN_LIST
#define BREN_LIST

#include <stdio.h>

typedef struct _Node
{
	struct _Node *next;
	struct _Node *prev;
	void *info;
} Node;

typedef struct _List
{
	Node *head;
	Node *tail;
	Node *current;
	int num_elts;
} List;

extern Node *Node_alloc(void *info);
extern List *List_init();
extern void List_insert_tail(List *list, void *info);
extern void List_insert_head(List *list, void *info);
extern void *List_find_head(List *list);
extern void *List_find_tail(List *list);
extern void *List_find_next(List *list);
extern void *List_delete_head(List *list);
extern void *List_delete_tail(List *list);
extern void List_delete_list(List *list);
extern void *List_delete_current(List *list);
extern void List_insert_ordered(List *, void *, int (*)(void *, void *));
extern void List_reset_current(List *);
extern void *List_return_info(Node *);
extern List *List_ccopy(List *);
extern char List_find_info(List *, void *, int (*)(void *, void *));
extern char List_find_unordered(List *, void *, int (*)(void *, void *));
extern void List_print(List *, void (*)(void*));
extern int List_number_elements(List *);

#endif
